@@ -41,7 +41,7 @@ typedef mglParser* HMPR;
 typedef mglFormula* HMEX;
 typedef mglFormulaC* HAEX;
 typedef const mglDataA* HCDT;
-
+//-----------------------------------------------------------------------------
 std::string MGL_EXPORT mgl_data_to_string(HCDT d, long ns);
 std::string MGL_EXPORT mgl_datac_to_string(HCDT d, long ns);
 /// Get section separated by symbol ch. This is analog of QString::section().
@@ -52,6 +52,7 @@ std::vector<std::string> MGL_EXPORT mgl_str_args(const std::string &str, char ch
 std::string MGL_EXPORT mgl_str_num(double val);
 /// Get string from complex number
 std::string MGL_EXPORT mgl_str_num(dual val);
+//-----------------------------------------------------------------------------
 
 extern "C" {
 
@@ -67,6 +68,11 @@ typedef void *HMPR;
 typedef const void *HCDT;
 #endif
 
+/// Set buffer size for number of primitives as (1<<bsize)^2.
+/** I.e. as 10^12 for bsize=20 or 4*10^9 for bsize=16 (default). NOTE: you set it only once. The current value is returned. */
+unsigned MGL_EXPORT mgl_bsize(unsigned bsize);
+unsigned MGL_EXPORT mgl_bsize_(unsigned *bsize);
+
 /// Set seed for random numbers
 void MGL_EXPORT mgl_srnd(long seed);
 void MGL_EXPORT mgl_srnd_(int *seed);
@@ -78,15 +84,25 @@ double MGL_EXPORT mgl_rnd_();
 void MGL_EXPORT mgl_data_set_name(mglDataA *dat, const char *name);
 void MGL_EXPORT mgl_data_set_name_(uintptr_t *dat, const char *name,int);
 void MGL_EXPORT mgl_data_set_name_w(mglDataA *dat, const wchar_t *name);
+/// Get name of data variable
+MGL_EXPORT const wchar_t *mgl_data_get_name_w(HCDT dat);
 /// Set callback function which is called at deleting variable
 void MGL_EXPORT mgl_data_set_func(mglDataA *dat, void (*func)(void *), void *par);
 
+#define mgl_datac_set_id	mgl_data_set_id
+#define mgl_datac_set_id_	mgl_data_set_id_
+/// Set names for columns (slices)
+void MGL_EXPORT mgl_data_set_id(mglDataA *d, const char *ids);
+void MGL_EXPORT mgl_datac_set_id_(uintptr_t *d, const char *eq,int );
+/// Get names for columns (slices)
+MGL_EXPORT const char *mgl_data_get_id(HCDT d);
+
 /// Save whole data array (for ns=-1) or only ns-th slice to text file
 void MGL_EXPORT mgl_data_save(HCDT dat, const char *fname,long ns);
 void MGL_EXPORT mgl_data_save_(uintptr_t *dat, const char *fname,int *ns,int);
 /// Export data array (for ns=-1) or only ns-th slice to PNG file according color scheme
-void MGL_EXPORT mgl_data_export(HCDT dat, const char *fname, const char *scheme,mreal v1,mreal v2,long ns);
-void MGL_EXPORT mgl_data_export_(uintptr_t *dat, const char *fname, const char *scheme,mreal *v1,mreal *v2,int *ns,int,int);
+void MGL_EXPORT mgl_data_export(HCDT dat, const char *fname, const char *scheme, double v1, double v2, long ns);
+void MGL_EXPORT mgl_data_export_(uintptr_t *dat, const char *fname, const char *scheme, mreal *v1, mreal *v2, int *ns,int,int);
 /// Save data to HDF file
 void MGL_EXPORT mgl_data_save_hdf(HCDT d,const char *fname,const char *data,int rewrite);
 void MGL_EXPORT mgl_data_save_hdf_(uintptr_t *d, const char *fname, const char *data, int *rewrite,int l,int n);
@@ -153,9 +169,47 @@ mreal MGL_EXPORT mgl_data_linear_ext_(uintptr_t *dat, mreal *x,mreal *y,mreal *z
 void MGL_EXPORT mgl_mutex_lock(void *);
 void MGL_EXPORT mgl_mutex_unlock(void *);
 
+//-----------------------------------------------------------------------------
+/// Create HMEX object for expression evaluating
+HMEX MGL_EXPORT mgl_create_expr(const char *expr);
+uintptr_t MGL_EXPORT mgl_create_expr_(const char *expr, int);
+/// Delete HMEX object
+void MGL_EXPORT mgl_delete_expr(HMEX ex);
+void MGL_EXPORT mgl_delete_expr_(uintptr_t *ex);
+/// Return value of expression for given x,y,z variables
+double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z);
+double MGL_EXPORT mgl_expr_eval_(uintptr_t *ex, mreal *x, mreal *y, mreal *z);
+/// Return value of expression for given variables
+double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *vars);
+/// Return value of expression differentiation over variable dir for given x,y,z variables
+double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z);
+double MGL_EXPORT mgl_expr_diff_(uintptr_t *ex, const char *dir, mreal *x, mreal *y, mreal *z, int);
+/// Return value of expression differentiation over variable dir for given variables
+double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *vars);
+//-----------------------------------------------------------------------------
+/// Create HAEX object for expression evaluating
+HAEX MGL_EXPORT mgl_create_cexpr(const char *expr);
+uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int);
+/// Delete HAEX object
+void MGL_EXPORT mgl_delete_cexpr(HAEX ex);
+void MGL_EXPORT mgl_delete_cexpr_(uintptr_t *ex);
+/// Return value of expression for given x,y,z variables
+mdual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z);
+mdual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, dual *x, dual *y, dual *z);
+/// Return value of expression for given variables
+mdual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *vars);
+
 //-----------------------------------------------------------------------------
 /// Callback function for asking user a question. Result shouldn't exceed 1024.
 extern MGL_EXPORT void (*mgl_ask_func)(const wchar_t *quest, wchar_t *res);
+/// Console function for asking user a question. Result shouldn't exceed 1024.
+void MGL_EXPORT mgl_ask_gets(const wchar_t *quest, wchar_t *res);
+/// Callback function for displaying progress of something.
+extern MGL_EXPORT void (*mgl_progress_func)(int value, int maximal, HMGL gr);
+/// Console function for displaying progress of something.
+void MGL_EXPORT mgl_progress_txt(int value, int maximal, HMGL gr);
+/// Display progress of something.
+void MGL_EXPORT mgl_progress(int value, int maximal, HMGL gr);
 //-----------------------------------------------------------------------------
 #ifdef __cplusplus
 }
@@ -165,7 +219,7 @@ struct MGL_EXPORT mglNum
 {
 	mreal d;		///< Number itself
 	dual c;
-	std::wstring s;	///< Number name
+	mglString s;	///< Number name
 	mglNum(mreal val=0):d(val),c(val)	{}
 	mglNum(const mglNum &n):d(n.d),c(n.c),s(n.s) {}
 	const mglNum &operator=(const mglNum &n)
@@ -176,13 +230,28 @@ struct MGL_EXPORT mglNum
 class MGL_EXPORT mglDataA
 {
 public:
-	std::wstring s;	///< Data name
+	mglString s;	///< Data name
+	mglString id;	///< column (or slice) names
+
 	bool temp;		///< This is temporary variable
 	void (*func)(void *);	///< Callback function for destroying
 	void *o; 		///< Pointer to external object
 
 	mglDataA()	{	temp=false;	func=0;	o=0;	}
 	virtual ~mglDataA()	{	if(func)	func(o);	}
+	/// Set name for data variable (can be used in mgl_formula_calc() or in MGL scripts)
+	inline void Name(const char *name)		{	s = name;	}
+	inline void Name(const wchar_t *name)	{	s = name;	}
+	/// Get name of data variable
+	inline const wchar_t *Name()	const	{	return s.w;	}
+
+	/// Set names for columns (slices)
+	inline void SetColumnId(const char *ids)	{	id = ids;	}
+	/// Make new id
+	inline void NewId()	{	id = "";	}
+	/// Get names for columns (slices)
+	inline const char *GetColumnId() const	{	return id.s;	}
+	
 	virtual void set_v(mreal /*val*/, long /*i*/,long /*j*/=0,long /*k*/=0)	{}
 	/// Get the interpolated value and its derivatives in given data cell without border checking
 	virtual mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const =0;
