@@ -33,6 +33,12 @@
 #else
 #define MGL_PUSH(a,v,m)	a.push_back(v);
 #endif
+
+#if MGL_HAVE_PTHREAD
+#define MGL_PUSHs(func,m)	{pthread_mutex_lock(&m);	func;	pthread_mutex_unlock(&m);}
+#else
+#define MGL_PUSHs(func,m)	{func;}
+#endif
 //-----------------------------------------------------------------------------
 inline mreal mgl_d(mreal v,mreal v1,mreal v2) { return v2!=v1?(v-v1)/(v2-v1):NAN; }
 //-----------------------------------------------------------------------------
@@ -44,55 +50,66 @@ mglPoint GetZ(HCDT z, int i, int j, int k=0);
 // NOTE memcpy is used --> no memory allocation in T
 template <class T> class mglStack
 {
-	T** dat;
-	size_t pb;	///< size of buffer (real size is 2^pb == 1L<<pb)
-	size_t np;	///< allocated pointers
-	size_t m;	///< used pointers (allocated size is m*nb)
+	T **dat;
+	unsigned MGL_PB;	///< size of buffer (real size is 2^pb == 1L<<pb). 
+	/** NOTE This limit the number of maximal points and primitives as (1<<MGL_PB)^2, 
+	  * i.e. as 10^12 for MGL_PB=20 or 4*10^9 for MGL_PB=16. See mgl_bsize().*/
+	size_t nb;	///< used blocks
 	size_t n;	///< used cells
 	void *mutex;
 public:
 	mglStack(const mglStack<T> &st)
-	{	np=st.np;	dat = (T**)malloc(np*sizeof(T*));
-		pb=st.pb;	m=n=0;	reserve(st.n);
-		for(size_t i=0;i<m;i++)	memcpy(dat[i],st.dat[i],((size_t)1<<pb)*sizeof(T));
-		n=st.n;		mutex = 0;	}
-	mglStack(size_t Pbuf=10)
-	{	np=16;	pb=Pbuf;	dat = (T**)malloc(np*sizeof(T*));
-		dat[0] = new T[(size_t)1<<pb];	n=0;	m=1;	mutex = 0;	}
-	~mglStack()	{	clear();	delete [](dat[0]);	free(dat);	}
-	inline void set_mutex(void *mtx)	{	mutex = mtx;	}
+	{	mutex = 0;	n=0;	nb=1;	MGL_PB = mgl_bsize(0);
+		dat = new T*[(size_t)1<<MGL_PB];
+		*dat = new T[(size_t)1<<MGL_PB];	reserve(st.n);
+		for(size_t i=0;i<nb;i++)
+			memcpy(dat[i],st.dat[i],((size_t)1<<MGL_PB)*sizeof(T));
+		n=st.n;	}
+	mglStack()
+	{	mutex = 0;	n=0;	nb=1;	MGL_PB = mgl_bsize(0);
+		dat = new T*[(size_t)1<<MGL_PB];
+		*dat = new T[(size_t)1<<MGL_PB];	}
+	~mglStack()	{	clear();	delete [](*dat);	delete []dat;	}
+	inline void set_mutex(void *mtx)	{	mutex=mtx;	}
+	inline size_t allocate(size_t num)
+	{	reserve(num);	size_t r=n;	n+=num;	return r;	}
 	void reserve(size_t num)
 	{
-		num+=n;
-		if(num>(m<<pb))
+		num = num?num+n:n+1;	// final required size
+		if(num>(nb<<MGL_PB))
 		{
-			num = 1+ (num>>pb);
-			if(num>np)
-			{	dat = (T**)realloc(dat, num*sizeof(T*));	np=num;	}
-			for(size_t i=m;i<num;i++)	dat[i] = new T[(size_t)1<<pb];
-			m = num;
+			num = 1 + (num>>MGL_PB);
+			for(;nb<num;nb++)	dat[nb] = new T[(size_t)1<<MGL_PB];
 		}
 	}
 	void clear()
 	{
 		if(mutex)	mgl_mutex_lock(mutex);
-		for(size_t i=0;i<m;i++)	delete [](dat[i]);
-		dat[0] = new T[(size_t)1<<pb];	n=0;	m=1;
+		for(size_t i=1;i<nb;i++)	delete [](dat[i]);
+		n=0;	nb=1;
 		if(mutex)	mgl_mutex_unlock(mutex);
 	}
-	T &operator[](size_t i)	{	size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
-	const T &operator[](size_t i)	const	{	size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
+	T &operator[](size_t i)
+	{	size_t d=i>>MGL_PB;	return dat[d][i-(d<<MGL_PB)];	}
+	const T &operator[](size_t i)	const
+	{	size_t d=i>>MGL_PB;	return dat[d][i-(d<<MGL_PB)];	}
 	void push_back(const T &t)
 	{
-		if(n>=(m<<pb))	reserve(1);
-		size_t d=n>>pb;
-		dat[d][n-(d<<pb)] = t;	n++;
+		if(n>=(nb<<MGL_PB))	reserve(1);
+		size_t d=n>>MGL_PB;	dat[d][n-(d<<MGL_PB)] = t;	n++;
+	}
+	void push_back(size_t num, const T *t)
+	{
+		if(n+num>=(nb<<MGL_PB))	reserve(num);
+		for(size_t i=0;i<num;i++)
+		{	size_t d=n>>MGL_PB;	dat[d][n-(d<<MGL_PB)] = t[i];	n++;	}
 	}
 	size_t size()	const	{	return n;	}
 	const mglStack<T> &operator=(const mglStack<T> &st)
 	{
-		pb=st.pb;	clear();	reserve(st.n);
-		for(size_t i=0;i<st.m && i<m;i++)	memcpy(dat[i],st.dat[i],((size_t)1<<pb)*sizeof(T));
+		clear();	reserve(st.n);
+		for(size_t i=0;i<nb;i++)
+			memcpy(dat[i],st.dat[i],((size_t)1<<MGL_PB)*sizeof(T));
 		n = st.n;	return st;
 	}
 };
@@ -100,21 +117,21 @@ public:
 /// Structure for transformation matrix
 struct MGL_EXPORT mglMatrix
 {
-	mreal b[9];
-	mreal x,y,z,pf;
+	float x,y,z,pf;
+	float b[9];
 	bool norot;	// flag to disable pnts rotation
-	mglMatrix()	{	memset(this,0,sizeof(mglMatrix));	clear();	}
-	mglMatrix(const mglMatrix &aa) : x(aa.x),y(aa.y),z(aa.z),pf(aa.pf),norot(aa.norot) 	{	memcpy(b,aa.b,9*sizeof(mreal));	}
+	mglMatrix()	{	clear();	}
+	mglMatrix(const mglMatrix &aa) : x(aa.x),y(aa.y),z(aa.z),pf(aa.pf),norot(aa.norot) 	{	memcpy(b,aa.b,9*sizeof(float));	}
 	void Rotate(mreal tetz,mreal tetx,mreal tety);
 	void RotateN(mreal Tet,mreal x,mreal y,mreal z);
-	inline void clear()	{	x=y=z=pf=0;	memset(b,0,9*sizeof(mreal));	b[0]=b[4]=b[8]=1;	norot=false;	}
+	inline void clear()	{	x=y=z=pf=0;	memset(b,0,9*sizeof(float));	b[0]=b[4]=b[8]=1;	norot=false;	}
 	inline const mglMatrix &operator=(const mglMatrix &a)
-	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(mreal));	norot=false;	return a;	}
+	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(float));	norot=false;	return a;	}
 };
 inline bool operator==(const mglMatrix &a, const mglMatrix &b)
-{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)==0)&&!memcmp(b.b,a.b,9*sizeof(mreal));}
+{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)==0)&&!memcmp(b.b,a.b,9*sizeof(float));}
 inline bool operator!=(const mglMatrix &a, const mglMatrix &b)
-{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)!=0)||memcmp(b.b,a.b,9*sizeof(mreal));	}
+{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)!=0)||memcmp(b.b,a.b,9*sizeof(float));	}
 //-----------------------------------------------------------------------------
 /// Structure for simplest primitives
 struct MGL_EXPORT mglPrim	// NOTE: use float for reducing memory size
@@ -136,7 +153,8 @@ struct MGL_EXPORT mglPrim	// NOTE: use float for reducing memory size
 		};
 		uint64_t m;
 	};
-	mglPrim(int t=0):n1(0),n2(0),n3(0),n4(0),type(t),angl(0),id(0),z(0),w(0),m(0)	{}
+	mglPrim():n1(0),n2(0),n3(0),n4(0),type(0),angl(0),id(0),z(0),w(0),m(0)	{}
+	explicit mglPrim(int t):n1(0),n2(0),n3(0),n4(0),type(t),angl(0),id(0),z(0),w(0),m(0)	{}
 	mglPrim(const mglPrim &aa) : n1(aa.n1),n2(aa.n2),n3(aa.n3),n4(aa.n4),type(aa.type),angl(aa.angl),id(aa.id),z(aa.z),w(aa.w),m(aa.m) 	{}
 	const mglPrim &operator=(const mglPrim &aa)	{	memcpy(this, &aa, sizeof(mglPrim));	return aa;	}
 };
@@ -146,33 +164,33 @@ bool operator>(const mglPrim &a,const mglPrim &b);
 /// Structure for light source
 struct MGL_EXPORT mglLight
 {
-	mglLight():n(false),a(0),b(0)	{}
-	mglLight(const mglLight &aa) : n(aa.n),d(aa.d),r(aa.r),q(aa.q),p(aa.p),a(aa.a),b(aa.b),c(aa.c)	{}
+	mglLight():a(0),b(0),n(false)	{}
+	mglLight(const mglLight &aa) : d(aa.d),r(aa.r),q(aa.q),p(aa.p),c(aa.c),a(aa.a),b(aa.b),n(aa.n)	{}
 	const mglLight &operator=(const mglLight &aa)
 	{	memcpy(this,&aa,sizeof(mglLight));	return aa;	}
 
-	bool n;			///< Availability of light sources
 	mglPoint d;		///< Direction of light sources
 	mglPoint r;		///< Position of light sources (NAN for infinity)
 	mglPoint q;		///< Actual position of light sources (filled by LightScale() function)
 	mglPoint p;		///< Actual direction of light sources (filled by LightScale() function)
-	mreal a;		///< Aperture of light sources
-	mreal b;		///< Brightness of light sources
 	mglColor c;		///< Color of light sources
+	float a;		///< Aperture of light sources
+	float b;		///< Brightness of light sources
+	bool n;			///< Availability of light sources
 };
 //-----------------------------------------------------------------------------
 /// Structure for inplot
 struct MGL_EXPORT mglBlock
 {
-	int id;		///< object id
 	long n1,n2,n3,n4;	///< coordinates of corners {n1=x1,n2=x2,n3=y1,n4=y2}
 
 	mglLight light[10];	///< Light sources
-	mreal AmbBr;		///< Default ambient light brightness
-	mreal DifBr;		///< Default diffusive light brightness
+	float AmbBr;		///< Default ambient light brightness
+	float DifBr;		///< Default diffusive light brightness
 	mglMatrix B;		///< Transformation matrix
+	int id;		///< object id
 
-	mglBlock():id(0),n1(0),n2(0),n3(0),n4(0),AmbBr(0.5),DifBr(0.5)	{}
+	mglBlock():n1(0),n2(0),n3(0),n4(0),AmbBr(0.5),DifBr(0.5),id(0)	{}
 	mglBlock(const mglBlock &aa)	{	memcpy(this, &aa, sizeof(mglBlock));	}
 	const mglBlock &operator=(const mglBlock &aa)	{	memcpy(this, &aa, sizeof(mglBlock));	return aa;	}
 };
@@ -196,9 +214,9 @@ struct MGL_EXPORT mglText
 {
 	std::wstring text;
 	std::string stl;
-	mreal val;
-	mglText(const wchar_t *txt=L"", const char *fnt="", mreal v=0) : text(txt), stl(fnt), val(v) {}
-	mglText(const std::wstring &txt, mreal v=0): text(txt), val(v)	{}
+	float val;
+	mglText(const wchar_t *txt=L"", const char *fnt="", float v=0) : text(txt), stl(fnt), val(v) {}
+	mglText(const std::wstring &txt, float v=0): text(txt), val(v)	{}
 	mglText(const mglText &aa) : text(aa.text),stl(aa.stl),val(aa.val)	{}
 #if MGL_HAVE_RVAL
 	mglText(mglText &&aa) : text(aa.text),stl(aa.stl),val(aa.val)	{}
@@ -209,32 +227,31 @@ struct MGL_EXPORT mglText
 /// Structure for internal point representation
 struct MGL_EXPORT mglPnt	// NOTE: use float for reducing memory size
 {
-	union {	float dat[16];	struct {
+	union {	float dat[15];	struct {
 		float x,y,z;	// coordinates
 		float u,v,w;	// normales
 		float r,g,b,a;	// RGBA color
 		float xx,yy,zz;	// original coordinates
-		float c,t,ta;	// index in color scheme
+		float c,ta;	// index in color scheme
 	}; };
-	short sub;		// subplot id and rotation information (later will be in subplot)
-	mglPnt(float X, float Y=0, float Z=0, float U=0, float V=0, float W=0, float R=0, float G=0, float B=0, float A=0):x(X),y(Y),z(Z),u(U),v(V),w(W),r(R),g(G),b(B),a(A),xx(X),yy(Y),zz(Z),c(0),t(0),ta(0),sub(0)	{}
-	mglPnt():x(0),y(0),z(0),u(0),v(0),w(0),r(0),g(0),b(0),a(0),xx(0),yy(0),zz(0),c(0),t(0),ta(0),sub(0)	{}
-	mglPnt(const mglPnt &aa) : sub(aa.sub)	{	memcpy(dat,aa.dat,16*sizeof(float));	}
-	inline const mglPnt&operator=(const mglPnt &aa)	{ sub=aa.sub;	memcpy(dat,aa.dat,16*sizeof(float));	return aa;	}
+	int sub;		// subplot id and rotation information (later will be in subplot)
+	mglPnt(float X=0, float Y=0, float Z=0, float U=0, float V=0, float W=0, float R=0, float G=0, float B=0, float A=0, short s=0) :x(X),y(Y),z(Z),u(U),v(V),w(W),r(R),g(G),b(B),a(A),xx(X),yy(Y),zz(Z),c(0),ta(0),sub(s)	{}
+	mglPnt(const mglPnt &aa) : sub(aa.sub)	{	memcpy(dat,aa.dat,15*sizeof(float));	}
+	inline const mglPnt&operator=(const mglPnt &aa)	{ sub=aa.sub;	memcpy(dat,aa.dat,15*sizeof(float));	return aa;	}
 	inline bool same(const mglPnt &p, mreal d)	const {	return fabs(x-p.x)<d && fabs(y-p.y)<d;	}
 };
 inline mglPnt operator+(const mglPnt &a, const mglPnt &b)
-{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]+b.dat[i];	p.sub=a.sub;	return p;	}
-//{	return mglPnt(a.x+b.x,a.y+b.y,a.z+b.z, a.u+b.u,a.v+b.v,a.w+b.w, a.r+b.r,a.g+b.g,a.b+b.b,a.a+b.a);	}
+//{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]+b.dat[i];	p.sub=a.sub;	return p;	}
+{	return mglPnt(a.x+b.x,a.y+b.y,a.z+b.z, a.u+b.u,a.v+b.v,a.w+b.w, a.r+b.r,a.g+b.g,a.b+b.b,a.a+b.a, a.sub);	}
 inline mglPnt operator-(const mglPnt &a, const mglPnt &b)
-{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]-b.dat[i];	p.sub=a.sub;	return p;	}
-//{	return mglPnt(a.x-b.x,a.y-b.y,a.z-b.z, a.u-b.u,a.v-b.v,a.w-b.w, a.r-b.r,a.g-b.g,a.b-b.b,a.a-b.a);	}
+//{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]-b.dat[i];	p.sub=a.sub;	return p;	}
+{	return mglPnt(a.x-b.x,a.y-b.y,a.z-b.z, a.u-b.u,a.v-b.v,a.w-b.w, a.r-b.r,a.g-b.g,a.b-b.b,a.a-b.a, a.sub);	}
 inline mglPnt operator*(const mglPnt &a, float b)
-{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]*b;	p.sub=a.sub;	return p;	}
-//{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b);	}
+//{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]*b;	p.sub=a.sub;	return p;	}
+{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b, a.sub);	}
 inline mglPnt operator*(float b, const mglPnt &a)
-{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]*b;	p.sub=a.sub;	return p;	}
-//{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b);	}
+//{	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]*b;	p.sub=a.sub;	return p;	}
+{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b, a.sub);	}
 //-----------------------------------------------------------------------------
 /// Structure for glyph representation
 struct MGL_EXPORT mglGlyph
@@ -263,25 +280,31 @@ struct MGL_EXPORT mglGlyph
 struct MGL_EXPORT mglTexture
 {
 	mglColor *col;	///< Colors itself
-	long n;				///< Number of initial colors along u
+	long n;			///< Number of initial colors along u
+	mglColor *c0;	///< Initial colors
+	float *val;	///< Initial color positions
 
-	char Sch[260];		///< Color scheme used
-	int Smooth;			///< Type of texture (smoothing and so on)
-	mreal Alpha;			///< Transparency
+	char Sch[260];	///< Color scheme used
+	int Smooth;		///< Type of texture (smoothing and so on)
+	float Alpha;	///< Transparency
 
-	mglTexture():n(0),Smooth(0),Alpha(1)
+	mglTexture():n(0),c0(NULL),val(NULL),Smooth(0),Alpha(1)
 	{	col = new mglColor[MGL_TEXTURE_COLOURS];	}
-	mglTexture(const char *cols, int smooth=0,mreal alpha=1):n(0)
+	mglTexture(const char *cols, int smooth=0,mreal alpha=1):n(0),c0(NULL),val(NULL)
 	{	col = new mglColor[MGL_TEXTURE_COLOURS];	Set(cols,smooth,alpha);	}
 	mglTexture(const mglTexture &aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
 	{	col = new mglColor[MGL_TEXTURE_COLOURS];	memcpy(Sch,aa.Sch,260);
-		memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	}
+		memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));
+		c0 = new mglColor[2*aa.n];	memcpy(c0,aa.c0,2*aa.n*sizeof(mglColor));
+		val = new float[aa.n];		memcpy(val,aa.val,aa.n*sizeof(float));	}
 #if MGL_HAVE_RVAL
-	mglTexture(mglTexture &&aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
-	{	col = aa.col;	memcpy(Sch,aa.Sch,260);	aa.col=0;	}
+	mglTexture(mglTexture &&aa) : n(aa.n),c0(aa.c0),val(aa.val),Smooth(aa.Smooth),Alpha(aa.Alpha)
+	{	col = aa.col;	memcpy(Sch,aa.Sch,260);	aa.col=aa.c0=0;	aa.val=0;	}
 #endif
-	~mglTexture()	{	if(col)	delete []col;	}
-	void Clear()	{	n=0;	}
+	~mglTexture()	{	if(col)	delete []col;	Clear();	}
+	void Clear()
+	{	if(c0)	{	delete []c0;	delete []val;	}
+		n=0;	c0=NULL;	val=NULL;	}
 	void Set(const char *cols, int smooth=0,mreal alpha=1);
 	void Set(HCDT val, const char *cols);
 	void GetC(mreal u,mreal v,mglPnt &p) const;
@@ -311,15 +334,15 @@ struct MGL_EXPORT mglActivePos
 };
 //-----------------------------------------------------------------------------
 #if defined(_MSC_VER)
-template class MGL_EXPORT mglStack<mglPnt>;
-template class MGL_EXPORT mglStack<mglPrim>;
-template class MGL_EXPORT std::vector<mglGroup>;
-template class MGL_EXPORT std::vector<mglText>;
-template class MGL_EXPORT std::vector<mglTexture>;
-template class MGL_EXPORT std::vector<mglGlyph>;
-template class MGL_EXPORT std::vector<mglBlock>;
-template class MGL_EXPORT std::vector<mglMatrix>;
-template class MGL_EXPORT mglStack<mglActivePos>;
+MGL_EXTERN template class MGL_EXPORT mglStack<mglPnt>;
+MGL_EXTERN template class MGL_EXPORT mglStack<mglPrim>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglGroup>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglText>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglTexture>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglGlyph>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglBlock>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglMatrix>;
+MGL_EXTERN template class MGL_EXPORT mglStack<mglActivePos>;
 #endif
 //-----------------------------------------------------------------------------
 /// Base class for canvas which handle all basic drawing
@@ -434,11 +457,11 @@ public:
 	/// Set number of mesh lines
 	inline void SetMeshNum(int val)	{	MeshNum=val;	}
 	/// Set relative width of rectangles in Bars, Barh, BoxPlot
-	inline void SetBarWidth(mreal val)	{	BarWidth=val;	}
+	inline void SetBarWidth(mreal val)	{	BarWidth=val>0?val:-BarWidth*val;	}
 	/// Set size of marks
-	inline void SetMarkSize(mreal val)	{	MarkSize=0.02*val;	}
+	inline void SetMarkSize(mreal val)	{	MarkSize=val>0?0.02*val:-val*MarkSize;	}
 	/// Set size of arrows
-	inline void SetArrowSize(mreal val)	{	ArrowSize=0.03*val;	}
+	inline void SetArrowSize(mreal val)	{	ArrowSize=val>0?0.03*val:-val*ArrowSize;	}
 	/// Get unscaled arrow size
 	inline mreal GetArrowSize() const	{	return ArrowSize/0.03;	}
 
@@ -506,8 +529,24 @@ public:
 	inline long AddPnt(mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
 	{	return AddPnt(&B,p,c,n,a,scl);	}
 	long AddPnt(const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
+	bool AddPntQ(mglPnt &q, const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
+	inline bool AddPntQ(mglPnt &q, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	{	return AddPntQ(q,&B,p,c,n,a,scl);	}
+	inline bool AddPntQ(long id, const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	{	return AddPntQ(Pnt[id],M,p,c,n,a,scl);	}
+	inline bool AddPntQ(long id, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	{	return AddPntQ(Pnt[id],&B,p,c,n,a,scl);	}
+	inline void SetPntOff(size_t id)	{	Pnt[id].x=NAN;	}
+	long AllocPnts(size_t num);
+	long PushPnts(size_t num, const mglPnt *qq);
 	long CopyNtoC(long k, mreal c);
+	bool CopyNtoC(mglPnt &q, long k, mreal c);
+	inline bool CopyNtoC(long id, long k, mreal c)
+	{	return (id>=0)?CopyNtoC(Pnt[id],k,c):false;	}
 	long CopyProj(long from, mglPoint p, mglPoint n, short sub=0);
+	bool CopyProj(mglPnt &q, long from, mglPoint p, mglPoint n, short sub=0);
+	void CopyProj(long id, long from, mglPoint p, mglPoint n, short sub=0)
+	{	if(id>=0)	CopyProj(Pnt[id],from,p,n,sub);	}
 	void SetRGBA(long k, const mglColor &c)
 	{	if(k>=0)	{mglPnt &p=Pnt[k];	p.r = c.r;	p.g = c.g;	p.b = c.b;	p.a = c.a;}	}
 	virtual void Reserve(long n);	///< Allocate n-cells for Pnt and return current position
@@ -540,9 +579,9 @@ public:
 	{
 		if(i<0 || j<0)	return true;
 		const mglPnt &p=Pnt[i], &q=Pnt[j];
-//		return GetWidth()>1 ? (long(p.x)==long(q.x) && long(p.y)==long(q.y)): (p.x==q.x && p.y==q.y);
-		return p.x==q.x && p.y==q.y;	
+		return mgl_isnan(p.x) || mgl_isnan(q.x) || (p.x==q.x && p.y==q.y);
 	}
+	inline bool ValidPnt(size_t i)	{	return mgl_isnum(Pnt[i].x);	}
 //	inline mglPrim &GetPrm(long i)		{	return Prm[i];		}
 	inline mglPrim &GetPrm(long i, bool sort=true)
 	{	return (sort && PrmInd) ? Prm[PrmInd[i]]:Prm[i];	}
@@ -556,6 +595,9 @@ public:
 	/// Scale coordinates and cut off some points
 	virtual bool ScalePoint(const mglMatrix *M, mglPoint &p, mglPoint &n, bool use_nan=true) const;
 
+	inline const mglBlock &GetSub(size_t i) const	{	return Sub[i];	}
+	inline const mglBlock &GetSub() const	{	return Sub.back();	}
+
 	virtual mreal GetOrgX(char dir, bool inv=false) const=0;	///< Get Org.x (parse NAN value)
 	virtual mreal GetOrgY(char dir, bool inv=false) const=0;	///< Get Org.y (parse NAN value)
 	virtual mreal GetOrgZ(char dir, bool inv=false) const=0;	///< Get Org.z (parse NAN value)
@@ -584,11 +626,12 @@ public:
 	virtual void trig_plot(long p1, long p2, long p3)=0;
 	virtual void quad_plot(long p1, long p2, long p3, long p4)=0;
 	virtual void smbl_plot(long p1, char id, double size)=0;
+	void curve_plot(size_t n, size_t kq, size_t step=1);
 	virtual void Glyph(mreal x, mreal y, mreal f, int style, long icode, mreal col)=0;
 	virtual float GetGlyphPhi(const mglPnt &q, float phi)=0;
 	virtual mreal text_plot(long p,const wchar_t *text,const char *fnt,mreal size=-1,mreal sh=0,mreal  col=-('k'),bool rot=true)=0;
 	void vect_plot(long p1, long p2, mreal s=1);
-	
+
 	// check if visible
 	virtual bool trig_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3) const =0;
 	virtual bool quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4) const =0;
@@ -649,11 +692,11 @@ protected:
 	char FontDef[32];	///< Font specification (see mglGraph::Puts). Default is Roman with align at center.
 	int Quality;		///< Quality of plot (0x0-pure, 0x1-fast; 0x2-fine; 0x4 - low memory)
 
-	mglFormula *fx;		///< Transformation formula for x direction.
-	mglFormula *fy;		///< Transformation formula for y direction.
-	mglFormula *fz;		///< Transformation formula for z direction.
-	mglFormula *fa;		///< Transformation formula for coloring.
-	mglFormula *fc;		///< Cutting off condition (formula).
+	HMEX fx;			///< Transformation formula for x direction.
+	HMEX fy;			///< Transformation formula for y direction.
+	HMEX fz;			///< Transformation formula for z direction.
+	HMEX fa;			///< Transformation formula for coloring.
+	HMEX fc;			///< Cutting off condition (formula).
 
 	long CurrPal;		///< Current palette index
 	mreal MarkSize;		///< The size of marks for 1D plots.
