@@ -28,7 +28,6 @@
 mreal MGL_EXPORT mglLinear(const mreal *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
 mreal MGL_EXPORT mglSpline3(const mreal *a, long nx, long ny, long nz, mreal x, mreal y, mreal z,mreal *dx=0, mreal *dy=0, mreal *dz=0);
 mreal MGL_EXPORT mglSpline3s(const mreal *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
-std::string MGL_EXPORT mgl_data_to_string(HCDT d, long ns);
 //-----------------------------------------------------------------------------
 /// Class for working with data array
 class MGL_EXPORT mglData : public mglDataA
@@ -39,14 +38,13 @@ using mglDataA::Momentum;
 	long ny;		///< number of points in 2nd dimensions ('y' dimension)
 	long nz;		///< number of points in 3d dimensions ('z' dimension)
 	mreal *a;		///< data array
-	std::string id;	///< column (or slice) names
 	bool link;		///< use external data (i.e. don't free it)
 
 	/// Initiate by other mglData variable
 	mglData(const mglData &d)	{	a=0;	mgl_data_set(this,&d);		}	// NOTE: must be constructor for mglData& to exclude copy one
 #if MGL_HAVE_RVAL
-	mglData(mglData &&d):nx(d.nx),ny(d.ny),nz(d.nz),a(d.a),id(d.id),link(d.link)
-	{	s=d.s;	temp=d.temp;	func=d.func;	o=d.o;	d.a=0;	d.func=0;	}
+	mglData(mglData &&d):nx(d.nx),ny(d.ny),nz(d.nz),a(d.a),link(d.link)
+	{	s=d.s;	temp=d.temp;	func=d.func;	o=d.o;	id=d.id;	d.a=0;	d.func=0;	}
 #endif
 	mglData(const mglDataA *d)
 	{	a=0;	if(d)	mgl_data_set(this, d);	else	mgl_data_create(this,1,1,1);		}
@@ -230,11 +228,6 @@ using mglDataA::Momentum;
 	/// Put array to data element(s)
 	inline void Put(const mglDataA &dat, long i=-1, long j=-1, long k=-1)
 	{	mgl_data_put_dat(this,&dat,i,j,k);	}
-	/// Set names for columns (slices)
-	inline void SetColumnId(const char *ids)
-	{	mgl_data_set_id(this,ids);	}
-	/// Make new id
-	inline void NewId()	{	id.clear();	}
 
 	/// Read data from tab-separated text file with auto determining size
 	inline bool Read(const char *fname)
@@ -318,9 +311,12 @@ using mglDataA::Momentum;
 	{	return mglData(true,mgl_data_evaluate(this,&idat,&jdat,0,norm));	}
 	inline mglData Evaluate(const mglData &idat, const mglData &jdat, const mglData &kdat, bool norm=true) const
 	{	return mglData(true,mgl_data_evaluate(this,&idat,&jdat,&kdat,norm));	}
-	/// Find roots for set of nonlinear equations defined by textual formula
+	/// Find roots for nonlinear equation defined by textual formula
 	inline mglData Roots(const char *eq, char var='x') const
 	{	return mglData(true,mgl_data_roots(eq, this, var));	}
+	/// Find roots for set of nonlinear equations defined by textual formula
+	inline mglData MultiRoots(const char *eq, const char *vars) const
+	{	return mglData(true,mgl_find_roots_txt(eq, vars, this));	}
 	/// Find correlation with another data arrays
 	inline mglData Correl(const mglDataA &dat, const char *dir) const
 	{	return mglData(true,mgl_data_correl(this,&dat,dir));	}
@@ -390,6 +386,9 @@ using mglDataA::Momentum;
 	/// Limit the data to be inside [-v,v], keeping the original sign
 	inline void Limit(mreal v)
 	{	mgl_data_limit(this, v);	}
+	/// Project the periodical data to range [v1,v2] (like mod() function). Separate branches by NAN if sep=true.
+	inline void Coil(mreal v1, mreal v2, bool sep=true)
+	{	mgl_data_coil(this, v1, v2, sep);	}
 
 	/// Apply Hankel transform
 	inline void Hankel(const char *dir)	{	mgl_data_hankel(this,dir);	}
@@ -448,8 +447,8 @@ using mglDataA::Momentum;
 	inline void operator-=(mreal d)		{	mgl_data_sub_num(this,d);	}
 #ifndef SWIG
 	/// Direct access to the data cell
-	inline mreal &operator[](long i)	{	return a[i];	}
-	// NOTE see 13.10 for operator(), operator[] -- m.b. I should add it ???
+	inline mreal operator[](long i) const	{	return a[i];	}
+	inline mreal &operator[](long i)		{	return a[i];	}
 #endif
 
 #ifndef DEBUG
@@ -472,14 +471,17 @@ using mglDataA::Momentum;
 	mreal vthr(long i) const {	return a[i];	}
 	// add for speeding up !!!
 	mreal dvx(long i,long j=0,long k=0) const
-	{   long i0=i+nx*(j+ny*k);
-		return i>0? (i<nx-1? (a[i0+1]-a[i0-1])/2:a[i0]-a[i0-1]) : a[i0+1]-a[i0];	}
+	{   long i0 = size_t(i)<size_t(nx-1) ? i+nx*(j+ny*k):nx*(1+j+ny*k)-2;	return a[i0+1]-a[i0];	}
+//	{   long i0=i+nx*(j+ny*k);
+//		return i>0? (i<nx-1? (a[i0+1]-a[i0-1])/2:a[i0]-a[i0-1]) : a[i0+1]-a[i0];	}
 	mreal dvy(long i,long j=0,long k=0) const
-	{   long i0=i+nx*(j+ny*k);
-		return j>0? (j<ny-1? (a[i0+nx]-a[i0-nx])/2:a[i0]-a[i0-nx]) : a[i0+nx]-a[i0];}
+	{   long i0 = size_t(j)<size_t(ny-1) ? i+nx*(j+ny*k):i+nx*(ny*(k+1)-2);	return a[i0+nx]-a[i0];	}
+//	{   long i0=i+nx*(j+ny*k);
+//		return j>0? (j<ny-1? (a[i0+nx]-a[i0-nx])/2:a[i0]-a[i0-nx]) : a[i0+nx]-a[i0];}
 	mreal dvz(long i,long j=0,long k=0) const
-	{   long i0=i+nx*(j+ny*k), n=nx*ny;
-		return k>0? (k<nz-1? (a[i0+n]-a[i0-n])/2:a[i0]-a[i0-n]) : a[i0+n]-a[i0];	}
+	{   long n=nx*ny, i0 = size_t(k)<size_t(nz-1) ? i+nx*(j+ny*k):i+nx*(j+ny*(nz-2));	return a[i0+n]-a[i0];	}
+//	{   long i0=i+nx*(j+ny*k), n=nx*ny;
+//		return k>0? (k<nz-1? (a[i0+n]-a[i0-n])/2:a[i0]-a[i0-n]) : a[i0+n]-a[i0];	}
 };
 //-----------------------------------------------------------------------------
 #ifndef SWIG
