@@ -148,6 +148,10 @@ void MGL_EXPORT mgl_data_norm_slice_(uintptr_t *dat, mreal *v1,mreal *v2,char *d
 /// Limit the data to be inside [-v,v], keeping the original sign
 void MGL_EXPORT mgl_data_limit(HMDT dat, mreal v);
 void MGL_EXPORT mgl_data_limit_(uintptr_t *dat, mreal *v);
+/// Project the periodical data to range [v1,v2] (like mod() function). Separate branches by NAN if sep=true.
+void MGL_EXPORT mgl_data_coil(HMDT dat, mreal v1, mreal v2, int sep);
+void MGL_EXPORT mgl_data_coil_(uintptr_t *dat, mreal &v1, mreal *v2, int *sep);
+
 /// Get sub-array of the data with given fixed indexes
 HMDT MGL_EXPORT mgl_data_subdata(HCDT dat, long xx,long yy,long zz);
 uintptr_t MGL_EXPORT mgl_data_subdata_(uintptr_t *dat, int *xx,int *yy,int *zz);
@@ -166,9 +170,6 @@ uintptr_t MGL_EXPORT mgl_data_section_(uintptr_t *d, uintptr_t *ids, const char
 HMDT MGL_EXPORT mgl_data_section_val(HCDT dat, long id, char dir, mreal val);
 uintptr_t MGL_EXPORT mgl_data_section_val_(uintptr_t *d, int *id, const char *dir, mreal *val,int);
 
-/// Set names for columns (slices)
-void MGL_EXPORT mgl_data_set_id(HMDT d, const char *id);
-void MGL_EXPORT mgl_data_set_id_(uintptr_t *dat, const char *id,int l);
 /// Equidistantly fill the data to range [x1,x2] in direction dir
 void MGL_EXPORT mgl_data_fill(HMDT dat, mreal x1,mreal x2,char dir);
 void MGL_EXPORT mgl_data_fill_(uintptr_t *dat, mreal *x1,mreal *x2,const char *dir,int);
@@ -505,24 +506,11 @@ mreal MGL_EXPORT mgl_find_root_txt_(const char *func, mreal *ini, const char *va
 /// Find roots for nonlinear equation defined by textual formula
 HMDT MGL_EXPORT mgl_data_roots(const char *func, HCDT ini, char var_id);
 uintptr_t MGL_EXPORT mgl_data_roots_(const char *func, uintptr_t *ini, const char *var_id,int,int);
-
-//-----------------------------------------------------------------------------
-/// Create HMEX object for expression evaluating
-HMEX MGL_EXPORT mgl_create_expr(const char *expr);
-uintptr_t MGL_EXPORT mgl_create_expr_(const char *expr, int);
-/// Delete HMEX object
-void MGL_EXPORT mgl_delete_expr(HMEX ex);
-void MGL_EXPORT mgl_delete_expr_(uintptr_t *ex);
-/// Return value of expression for given x,y,z variables
-double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z);
-double MGL_EXPORT mgl_expr_eval_(uintptr_t *ex, mreal *x, mreal *y, mreal *z);
-/// Return value of expression for given variables
-double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *vars);
-/// Return value of expression differentiation over variable dir for given x,y,z variables
-double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z);
-double MGL_EXPORT mgl_expr_diff_(uintptr_t *ex, const char *dir, mreal *x, mreal *y, mreal *z, int);
-/// Return value of expression differentiation over variable dir for given variables
-double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *vars);
+/// Find roots for set of nonlinear equations defined by textual formulas
+HMDT MGL_EXPORT mgl_find_roots_txt(const char *func, const char *vars, HCDT ini);
+uintptr_t MGL_EXPORT mgl_find_roots_txt_(const char *func, const char *vars, uintptr_t *ini,int,int);
+/// Find roots for set of nonlinear equations defined by function
+bool MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f, void *par), mreal *x0, void *par);
 //-----------------------------------------------------------------------------
 #ifdef __cplusplus
 }
