@@ -184,9 +184,6 @@ uintptr_t MGL_EXPORT mgl_datac_section_(uintptr_t *d, uintptr_t *ids, const char
 HADT MGL_EXPORT mgl_datac_section_val(HCDT dat, long id, char dir, mreal val);
 uintptr_t MGL_EXPORT mgl_datac_section_val_(uintptr_t *d, int *id, const char *dir, mreal *val,int);
 
-/// Set names for columns (slices)
-void MGL_EXPORT mgl_datac_set_id(HADT d, const char *id);
-void MGL_EXPORT mgl_datac_set_id_(uintptr_t *dat, const char *id,int l);
 /// Equidistantly fill the data to range [x1,x2] in direction dir
 void MGL_EXPORT mgl_datac_fill(HADT dat, dual x1,dual x2,char dir);
 void MGL_EXPORT mgl_datac_fill_(uintptr_t *dat, dual *x1,dual *x2,const char *dir,int);
@@ -373,18 +370,10 @@ uintptr_t MGL_EXPORT mgl_gspline_init_(uintptr_t *x, uintptr_t *v);
 /// Evaluate global spline (and its derivatives d1, d2 if not NULL) using prepared coefficients \a coef
 mdual MGL_EXPORT mgl_gsplinec(HCDT coef, mreal dx, dual *d1, dual *d2);
 mdual MGL_EXPORT mgl_gsplinec_(uintptr_t *c, mreal *dx, dual *d1, dual *d2);
-//-----------------------------------------------------------------------------
-/// Create HAEX object for expression evaluating
-HAEX MGL_EXPORT mgl_create_cexpr(const char *expr);
-uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int);
-/// Delete HAEX object
-void MGL_EXPORT mgl_delete_cexpr(HAEX ex);
-void MGL_EXPORT mgl_delete_cexpr_(uintptr_t *ex);
-/// Return value of expression for given x,y,z variables
-mdual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z);
-mdual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, dual *x, dual *y, dual *z);
-/// Return value of expression for given variables
-mdual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *vars);
+
+/// Find roots for set of nonlinear equations defined by textual formulas
+HADT MGL_EXPORT mgl_find_roots_txt_c(const char *func, const char *vars, HCDT ini);
+uintptr_t MGL_EXPORT mgl_find_roots_txt_c_(const char *func, const char *vars, uintptr_t *ini,int,int);
 
 #ifdef __cplusplus
 }
