@@ -50,7 +50,7 @@ struct mglGlyphDescr
 inline bool operator<(const mglGlyphDescr &a,const mglGlyphDescr &b)	{	return a.id<b.id;	}
 inline bool operator>(const mglGlyphDescr &a,const mglGlyphDescr &b)	{	return a.id>b.id;	}
 #if defined(_MSC_VER)
-template class MGL_EXPORT std::vector<mglGlyphDescr>;
+MGL_EXTERN template class MGL_EXPORT std::vector<mglGlyphDescr>;
 #endif
 //-----------------------------------------------------------------------------
 extern const float mgl_fact;
