@@ -1,6 +1,6 @@
 /***************************************************************************
  * qt.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -30,6 +30,8 @@ uintptr_t MGL_EXPORT mgl_create_graph_qt_(const char *title, int);
 /// Run main Qt loop for event handling.
 int MGL_EXPORT mgl_qt_run();
 int MGL_EXPORT mgl_qt_run_();
+/// Return pointer to widget (QMathGL*) used for plotting
+MGL_EXPORT void *mgl_qt_widget(HMGL gr);
 #ifdef __cplusplus
 }
 //-----------------------------------------------------------------------------
@@ -38,7 +40,7 @@ int MGL_EXPORT mgl_qt_run_();
 /// Wrapper class for windows displaying graphics
 class MGL_EXPORT mglQT : public mglWnd
 {
-	mglQT(const mglQT &t) {}	// copying is not allowed
+	mglQT(const mglQT &) {}	// copying is not allowed
 	const mglQT &operator=(const mglQT &t)	{	return t;	}
 public:
 	mglQT(const char *title="MathGL") : mglWnd()
@@ -52,6 +54,8 @@ public:
 		mgl_set_click_func(gr, mgl_click_class);	}
 	virtual ~mglQT() {}
 	int Run()	{	return mgl_qt_run();	}	///< Run main loop for event handling
+	/// Return pointer to widget (QMathGL*) used for plotting
+	void *Widget()	{	return mgl_qt_widget(gr);	}
 };
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_ask_qt(const wchar_t *quest, wchar_t *res);
