@@ -24,6 +24,15 @@
 #include <omp.h>
 #endif
 
+//-----------------------------------------------------------------------------
+static unsigned mgl_pb=0;
+unsigned MGL_EXPORT mgl_bsize(unsigned bsize)
+{
+	if(!mgl_pb)	mgl_pb = (bsize>0 && bsize<100)?bsize:16;
+	return mgl_pb;
+}
+unsigned MGL_EXPORT mgl_bsize_(unsigned *bsize)
+{	return mgl_bsize(*bsize);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_mutex_unlock(void *mutex)
 {
@@ -332,18 +341,42 @@ void mglBase::DefineGlyph(HCDT x, HCDT y, unsigned char id)
 //-----------------------------------------------------------------------------
 //		Add points to the buffer
 //-----------------------------------------------------------------------------
+long mglBase::PushPnts(size_t num, const mglPnt *qq)
+{
+	long k;
+#pragma omp critical(pnt)
+	MGL_PUSHs({k=Pnt.size();Pnt.push_back(num,qq);},mutexPnt);
+	return k;
+}
+//-----------------------------------------------------------------------------
+long mglBase::AllocPnts(size_t num)
+{
+	long k;
+#pragma omp critical(pnt)
+	MGL_PUSHs({k=Pnt.allocate(num);},mutexPnt);
+	return k;
+}
+//-----------------------------------------------------------------------------
 void inline mgl_put_inbox(mreal a1, mreal a2, mreal &a)
 {
 	if(a1<a2)	{	if(a<a1)	a=a1;	if(a>a2)	a=a2;	}
 	else		{	if(a<a2)	a=a2;	if(a>a1)	a=a1;	}
 }
-void MGL_NO_EXPORT mgl_coor_box(HMGL gr, mglPoint &p)
+static void mgl_coor_box(HMGL gr, mglPoint &p)
 {
 	mgl_put_inbox(gr->Min.x, gr->Max.x, p.x);
 	mgl_put_inbox(gr->Min.y, gr->Max.y, p.y);
 	mgl_put_inbox(gr->Min.z, gr->Max.z, p.z);
 }
 long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mreal a, int scl)
+{
+	mglPnt q;
+	if(!AddPntQ(q,mat,p,c,n,a,scl))	return -1;
+	long k;
+#pragma omp critical(pnt)
+	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
+}
+bool mglBase::AddPntQ(mglPnt &q, const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mreal a, int scl)
 {
 	// scl=0 -- no scaling
 	// scl&1 -- usual scaling
@@ -351,22 +384,21 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 	// scl&4 -- disable NAN for normales if no light
 	// scl&8 -- bypass palette for enabling alpha
 	// scl&16 -- put points inside axis range
-	if(mgl_isnan(c) || mgl_isnan(a))	return -1;
+	if(mgl_isnan(c) || mgl_isnan(a))	{	q.x=NAN;	return false;	}
 	bool norefr = mgl_isnan(n.x) && mgl_isnan(n.y) && !mgl_isnan(n.z);
 	if(scl>0)
 	{
 		if(scl&16)	mgl_coor_box(this, p);
 		ScalePoint(mat,p,n,!(scl&2));
 	}
-	if(mgl_isnan(p.x))	return -1;
+	if(mgl_isnan(p.x))	{	q.x=NAN;	return false;	}
 	a = (a>=0 && a<=1) ? a : AlphaDef;
 	c = (c>=0) ? c:CDef;
 
-	mglPnt q;
 	if(get(MGL_REDUCEACC))
 	{
 		q.x=q.xx=int(p.x*10)*0.1;	q.y=q.yy=int(p.y*10)*0.1;	q.z=q.zz=int(p.z*10)*0.1;
-		q.c=int(c*100)*0.01;	q.t=q.ta=int(a*100)*0.01;
+		q.c=int(c*100)*0.01;		q.ta=int(a*100)*0.01;
 		q.u=mgl_isnum(n.x)?int(n.x*100)*0.01:NAN;
 		q.v=mgl_isnum(n.y)?int(n.y*100)*0.01:NAN;
 		q.w=mgl_isnum(n.z)?int(n.z*100)*0.01:NAN;
@@ -374,7 +406,7 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 	else
 	{
 		q.x=q.xx=p.x;	q.y=q.yy=p.y;	q.z=q.zz=p.z;
-		q.c=c;	q.t=q.ta=a;	q.u=n.x;	q.v=n.y;	q.w=n.z;
+		q.c=c;	q.ta=a;	q.u=n.x;	q.v=n.y;	q.w=n.z;
 	}
 	long ci=long(c);
 	if(ci<0 || ci>=(long)Txt.size())	ci=0;	// NOTE never should be here!!!
@@ -389,40 +421,52 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 	// add gap for texture coordinates for compatibility with OpenGL
 	const mreal gap = 0./MGL_TEXTURE_COLOURS;
 	q.c = ci+(q.c-ci)*(1-2*gap)+gap;
-	q.t = q.t*(1-2*gap)+gap;
-	q.ta = q.t;
+	q.ta = q.ta*(1-2*gap)+gap;
 
 	if(scl&8 && scl>0)	q.a=a;	// bypass palette for enabling alpha in Error()
 	if(!get(MGL_ENABLE_ALPHA))	{	q.a=1;	if(txt.Smooth!=2)	q.ta=1-gap;	}
 	if(norefr)	q.v=0;
 	if(!get(MGL_ENABLE_LIGHT) && !(scl&4))	q.u=q.v=NAN;
-	q.sub=mat->norot?-1*(short)Sub.size():Sub.size()-1;
-	long k;
-#pragma omp critical(pnt)
-	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
+	q.sub=mat->norot?-1*(int)Sub.size():Sub.size()-1;
+	return true;
 }
 //-----------------------------------------------------------------------------
 long mglBase::CopyNtoC(long from, mreal c)
 {
-	if(from<0)	return -1;
-	mglPnt p=Pnt[from];
-	if(mgl_isnum(c))	{	p.c=c;	p.t=1;	Txt[long(c)].GetC(c,0,p);	p.a=1;	}
+	mglPnt q;
+	if(!CopyNtoC(q,from,c))	return -1;
 	long k;
 #pragma omp critical(pnt)
-	{k=Pnt.size();	MGL_PUSH(Pnt,p,mutexPnt);}	return k;
+	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
+}
+//-----------------------------------------------------------------------------
+bool mglBase::CopyNtoC(mglPnt &q, long from, mreal c)
+{
+	if(from<0)	return false;
+	q = Pnt[from];
+	if(mgl_isnum(c))	{	q.c=c;	q.ta=1;	Txt[long(c)].GetC(c,0,q);	q.a=1;	}
+	else	q.x = NAN;
+	return mgl_isnum(q.x);
 }
 //-----------------------------------------------------------------------------
 long mglBase::CopyProj(long from, mglPoint p, mglPoint n, short sub)
 {
-	if(from<0)	return -1;
-	mglPnt q=Pnt[from];	q.sub = sub;
-	q.x=q.xx=p.x;	q.y=q.yy=p.y;	q.z=q.zz=p.z;
-	q.u = n.x;		q.v = n.y;		q.w = n.z;
+	mglPnt q;
+	if(!CopyProj(q,from,p,n,sub))	return -1;
 	long k;
 #pragma omp critical(pnt)
 	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
 }
 //-----------------------------------------------------------------------------
+bool mglBase::CopyProj(mglPnt &q, long from, mglPoint p, mglPoint n, short sub)
+{
+	if(from<0)	return false;
+	q=Pnt[from];	q.sub = sub;
+	q.x=q.xx=p.x;	q.y=q.yy=p.y;	q.z=q.zz=p.z;
+	q.u = n.x;		q.v = n.y;		q.w = n.z;
+	return mgl_isnum(q.x);
+}
+//-----------------------------------------------------------------------------
 void mglBase::Reserve(long n)
 {
 	if(TernAxis&12)	n*=4;
@@ -968,7 +1012,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 {
 	// NOTE: New syntax -- colors are CCCCC or {CNCNCCCN}; options inside []
 	if(!s || !s[0])	return;
-	mgl_strncpy(Sch,s,259);	Smooth=smooth;	Alpha=alpha;
+	mgl_strncpy(Sch,s,259);	Smooth=smooth;	Alpha=alpha;	Clear();
 
 	long l=strlen(s);
 	bool map = smooth==2 || mglchr(s,'%'), sm = smooth>=0 && !strchr(s,'|');	// Use mapping, smoothed colors
@@ -982,8 +1026,8 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	}
 	if(n<=0)	return;
 	bool man=sm;
-	mglColor *c = new mglColor[2*n];		// Colors itself
-	mreal *val = new mreal[n];
+	c0 = new mglColor[2*n];	// Colors itself
+	val = new float[n];
 	for(long i=0, m=0, j=n=0;i<l;i++)	// fill colors
 	{
 		if(smooth>=0 && s[i]==':' && j<1)	break;
@@ -994,18 +1038,18 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 		if(strchr(MGL_COLORS,s[i]) && j<1 && (m==0 || s[i-1]=='{'))	// {CN,val} format, where val in [0,1]
 		{
 			if(m>0 && s[i+1]>'0' && s[i+1]<='9')// ext color
-			{	c[2*n].Set(s[i],(s[i+1]-'0')/5.f);	i++;	}
-			else	c[2*n].Set(s[i]);	// usual color
-			val[n]=-1;	c[2*n].a = -1;	n++;
+			{	c0[2*n].Set(s[i],(s[i+1]-'0')/5.f);	i++;	}
+			else	c0[2*n].Set(s[i]);	// usual color
+			val[n]=-1;	c0[2*n].a = -1;	n++;
 		}
 		if(s[i]=='x' && i>0 && s[i-1]=='{' && j<1)	// {xRRGGBB,val} format, where val in [0,1]
 		{
 			uint32_t id = strtoul(s+1+i,0,16);
-			if(memchr(s+i+1,'}',8) || memchr(s+i+1,',',8))	c[2*n].a = -1;
-			else	{	c[2*n].a = (id%256)/255.;	id /= 256;	}
-			c[2*n].b = (id%256)/255.;	id /= 256;
-			c[2*n].g = (id%256)/255.;	id /= 256;
-			c[2*n].r = (id%256)/255.;
+			if(memchr(s+i+1,'}',8) || memchr(s+i+1,',',8))	c0[2*n].a = -1;
+			else	{	c0[2*n].a = (id%256)/255.;	id /= 256;	}
+			c0[2*n].b = (id%256)/255.;	id /= 256;
+			c0[2*n].g = (id%256)/255.;	id /= 256;
+			c0[2*n].r = (id%256)/255.;
 			while(strchr("0123456789abcdefABCDEFx",s[i]))	i++;
 			val[n]=-1;	n++;	i--;
 		}
@@ -1017,19 +1061,19 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	}
 	for(long i=0;i<n;i++)	// default texture
 	{
-		if(c[2*i].a<0)	c[2*i].a=alpha;
-		c[2*i+1]=c[2*i];
-		if(man)	c[2*i].a=0;
+		if(c0[2*i].a<0)	c0[2*i].a=alpha;
+		c0[2*i+1]=c0[2*i];
+		if(man)	c0[2*i].a=0;
 	}
 	if(map && sm && n>1)		// map texture
 	{
 		if(n==2)
-		{	c[1]=c[2];	c[2]=c[0];	c[0]=BC;	c[3]=c[1]+c[2];	}
+		{	c0[1]=c0[2];	c0[2]=c0[0];	c0[0]=BC;	c0[3]=c0[1]+c0[2];	}
 		else if(n==3)
-		{	c[1]=c[2];	c[2]=c[0];	c[0]=BC;	c[3]=c[4];	n=2;}
+		{	c0[1]=c0[2];	c0[2]=c0[0];	c0[0]=BC;	c0[3]=c0[4];	n=2;}
 		else
-		{	c[1]=c[4];	c[3]=c[6];	n=2;	}
-		c[0].a = c[1].a = c[2].a = c[3].a = alpha;
+		{	c0[1]=c0[4];	c0[3]=c0[6];	n=2;	}
+		c0[0].a = c0[1].a = c0[2].a = c0[3].a = alpha;
 		val[0]=val[1]=-1;
 	}
 	// TODO if(!sm && n==1)	then try to find color in palette ???
@@ -1053,7 +1097,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	if(!sm)	for(long i=0;i<256;i++)
 	{
 		long j = 2*long(v*i);	//u-=j;
-		col[2*i] = c[j];	col[2*i+1] = c[j+1];
+		col[2*i] = c0[j];	col[2*i+1] = c0[j+1];
 	}
 	else	for(long i=i1=0;i<256;i++)
 	{
@@ -1063,13 +1107,12 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 			for(;i1<n-1 && i>=255*val[i1];i1++);
 			v2 = i1<n?1/(val[i1]-val[i1-1]):0;
 			j=i1-1;	u=(i/255.-val[j])*v2;
-			col[2*i] = c[2*j]*(1-u)+c[2*j+2]*u;
-			col[2*i+1]=c[2*j+1]*(1-u)+c[2*j+3]*u;
+			col[2*i] = c0[2*j]*(1-u)+c0[2*j+2]*u;
+			col[2*i+1]=c0[2*j+1]*(1-u)+c0[2*j+3]*u;
 		}
 		else
-		{	col[2*i] = c[2*n-2];col[2*i+1] = c[2*n-1];	}
+		{	col[2*i] = c0[2*n-2];col[2*i+1] = c0[2*n-1];	}
 	}
-	delete []c;	delete []val;
 }
 //-----------------------------------------------------------------------------
 mglColor mglTexture::GetC(mreal u,mreal v) const
@@ -1594,3 +1637,48 @@ void mglBase::ClearPrmInd()
 	{	if(PrmInd)	delete []PrmInd;	PrmInd=NULL;	}
 }
 //-----------------------------------------------------------------------------
+void mglBase::curve_plot(size_t num, size_t k0, size_t step)
+{
+	// exclude straight-line parts
+	if(get(MGL_FULL_CURV))	for(size_t i=0;i+1<num;i++)
+		line_plot(k0+i*step,k0+(i+1)*step);
+	else	for(size_t i=0;i+1<num;i++)
+	{
+		const mglPoint p1(GetPntP(k0+i*step)), ps(GetPntP(k0+(i+1)*step));
+		if(mgl_isnan(p1.x) || mgl_isnan(ps.x))	continue;
+		const mglColor c1(GetPntC(k0+i*step));
+		// remove duplicates
+		for(;i+1<num;i++)
+		{
+			size_t ii = k0+(i+1)*step;
+			const mglPoint pp(GetPntP(ii));
+			if(p1!=pp || mgl_isnan(pp.x))	break;
+		}
+		if(i+1>=num)	break;
+
+		float t1=-100, t2=100;		// XY angle boundary
+		float rg1=-100, rg2=100;	// RG angle boundary
+		float gb1=-100, gb2=100;	// GB angle boundary
+		size_t k;
+		for(k=i+1;k<num;k++)
+		{
+			const mglPoint p2(GetPntP(k0+k*step)-p1);
+			if(mgl_isnan(p2.x))	break;
+			float dd=p2.x*p2.x+p2.y*p2.y+p2.z*p2.z;
+			if(dd<=0)	continue;	// the same point (micro-loop? :) )
+			float t = atan2(p2.y,p2.x), d = atan(0.03/dd);
+			if(t1 > t+d || t2 < t-d)	break;		// too curved
+			const mglColor c2(GetPntC(k0+(k-1)*step)-c1);	dd = c2.NormS();
+			if(dd>0)	// color are different
+			{
+				float rg = atan2(c2.r,c2.g), gb = atan2(c2.g,c2.b);	d = atan(1e-4/dd);
+				if(rg1 > rg+d || rg2 < rg-d || gb1 > gb+d || gb2 < gb-d)	break;		// too curved
+				rg1 = rg1<rg-d?rg-d:rg1;	rg2 = rg2>rg+d?rg+d:rg2;	// new RG range
+				gb1 = gb1<gb-d?gb-d:gb1;	gb2 = gb2>gb+d?gb+d:gb2;	// new GB range
+			}
+			t1 = t1<t-d?t-d:t1;	t2 = t2>t+d?t+d:t2;	// new range
+		}
+		k--;	line_plot(k0+i*step,k0+k*step);	i = k-1;
+	}
+}
+//-----------------------------------------------------------------------------
