@@ -247,7 +247,7 @@ GifFileType *gif;*/
 	SetPenPal("k-1");		Alpha(false);
 	stack.clear();	Restore();	DefColor('k');
 	SetPlotFactor(0);	Sub.clear();
-	InPlot(0,1,0,1,false);
+	InPlot(0,1,0,1,false);	clr(MGL_FULL_CURV);
 	SetTickLen(0);	SetCut(true);
 	AdjustTicks("xyzc",true);	Clf('w');
 
@@ -374,7 +374,7 @@ void mglCanvas::mark_plot(long p, char type, mreal size)
 void mglCanvas::line_plot(long p1, long p2)
 {
 	if(PDef==0)	return;
-	if(p1<0 || p2<0 || mgl_isnan(Pnt[p1].x) || mgl_isnan(Pnt[p2].x) || SamePnt(p1,p2))	return;
+	if(SamePnt(p1,p2))	return;
 	if(p1>p2)	{	long kk=p1;	p1=p2;	p2=kk;	}	// rearrange start/end for proper dashing
 	long pp1=p1,pp2=p2;
 	mreal pw = fabs(PenWidth)*sqrt(font_factor/400), d=0;
@@ -384,14 +384,14 @@ void mglCanvas::line_plot(long p1, long p2)
 		{
 			d += hypot(Pnt[p1].x-Pnt[p2].x, Pnt[p1].y-Pnt[p2].y);
 			MGL_LINE_PLOT
-			pPos = fmod(pPos+d/pw/1.5, 16);
+			pPos = fmod(pPos+d/pw, 16);
 		}
 	}
 	else
 	{
 		d = hypot(Pnt[p1].x-Pnt[p2].x, Pnt[p1].y-Pnt[p2].y);
 		MGL_LINE_PLOT
-		pPos = fmod(pPos+d/pw/1.5, 16);
+		pPos = fmod(pPos+d/pw, 16);
 	}
 }
 //-----------------------------------------------------------------------------
@@ -402,7 +402,6 @@ void mglCanvas::line_plot(long p1, long p2)
 							a.m=mask;	a.angl=MaskAn;	a.w = pw;	add_prim(a);}
 void mglCanvas::trig_plot(long p1, long p2, long p3)
 {
-	if(p1<0 || p2<0 || p3<0 || mgl_isnan(Pnt[p1].x) || mgl_isnan(Pnt[p2].x) || mgl_isnan(Pnt[p3].x))	return;
 	if(SamePnt(p1,p2) || SamePnt(p1,p3))	return;
 	long pp1=p1,pp2=p2,pp3=p3;
 	mreal pw = fabs(PenWidth)*sqrt(font_factor/400);
@@ -419,10 +418,10 @@ void mglCanvas::trig_plot(long p1, long p2, long p3)
 							a.m=mask;	a.angl=MaskAn;	a.w = pw;	add_prim(a);	}
 void mglCanvas::quad_plot(long p1, long p2, long p3, long p4)
 {
-	if(p1<0 || mgl_isnan(Pnt[p1].x) || SamePnt(p1,p2))	{	trig_plot(p4,p2,p3);	return;	}
-	if(p2<0 || mgl_isnan(Pnt[p2].x) || SamePnt(p2,p4))	{	trig_plot(p1,p4,p3);	return;	}
-	if(p3<0 || mgl_isnan(Pnt[p3].x) || SamePnt(p1,p3))	{	trig_plot(p1,p2,p4);	return;	}
-	if(p4<0 || mgl_isnan(Pnt[p4].x) || SamePnt(p3,p4))	{	trig_plot(p1,p2,p3);	return;	}
+	if(SamePnt(p1,p2))	{	trig_plot(p4,p2,p3);	return;	}
+	if(SamePnt(p2,p4))	{	trig_plot(p1,p4,p3);	return;	}
+	if(SamePnt(p1,p3))	{	trig_plot(p1,p2,p4);	return;	}
+	if(SamePnt(p3,p4))	{	trig_plot(p1,p2,p3);	return;	}
 	long pp1=p1,pp2=p2,pp3=p3,pp4=p4;
 	mreal pw = fabs(PenWidth)*sqrt(font_factor/400);
 	if(TernAxis&12) for(int i=0;i<4;i++)
@@ -482,7 +481,7 @@ pthread_mutex_lock(&mutexPtx);
 		}
 
 		int align;
-		mreal col1=col, col2=col;
+		float col1=col, col2=col;
 		if(mglGetStyle(font,0,&align))
 		{
 			col1 = AddTexture(font);
@@ -514,17 +513,17 @@ pthread_mutex_lock(&mutexPtx);
 			add_prim(a);
 		}
 
-		q.c=col1;	q.t=0;	Txt[long(col1)].GetC(col1,0,q);
-		q.u = q.v = NAN;	q.a=q.t=q.ta=1;
-		memset(Bt.b,0,9*sizeof(mreal));
+		q.c=col1;	q.ta=0;	Txt[long(col1)].GetC(col1,0,q);
+		q.u = q.v = NAN;	q.a=q.ta=1;
+		memset(Bt.b,0,9*sizeof(float));
 		Bt.b[0] = Bt.b[4] = Bt.b[8] = fscl;
-		mreal opf = Bt.pf;
+		float opf = Bt.pf;
 		Bt.RotateN(ftet,0,0,1);	Bt.pf = Bt.norot?1.55:opf;
 		if(strchr(font,'@'))	// draw box around text
 		{
 			long k1,k2,k3,k4;	mglPnt pt;	mglPoint pp;
 			w = fnt->Width(text,font);	h = fnt->Height(font);
-			mreal d=-w*align/2.-h*0.2;	w+=h*0.4;
+			float d=-w*align/2.-h*0.2;	w+=h*0.4;
 			pt = q;	pp.Set(d,-h*0.4);		PostScale(&Bt,pp);
 			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
 #pragma omp critical(pnt)
@@ -584,7 +583,7 @@ void mglCanvas::Glyph(mreal x, mreal y, mreal f, int s, long j, mreal col)
 						else	add_prim(a);
 void mglCanvas::smbl_plot(long p1, char id, double size)
 {
-	if(p1<0)	return;
+	if(p1<0 || mgl_isnan(Pnt[p1].x))	return;
 	mglPnt q=Pnt[p1];
 	mreal ftet=NAN, ll = q.u*q.u+q.v*q.v;
 	if(mgl_isnan(ll) || !get(MGL_ENABLE_RTEXT))	ftet = 0;
@@ -670,7 +669,7 @@ void mglCanvas::InPlot(mglMatrix &M,mreal x1,mreal x2,mreal y1,mreal y2, bool re
 	}
 	inW=M.b[0];	inH=M.b[4];	ZMin=1;
 	inX=Width*x1;	inY=Height*y1;
-	font_factor = M.b[0] < M.b[4] ? M.b[0] : M.b[4];
+	if(!rel || !get(MGL_NO_SCALE_REL))	font_factor = M.b[0] < M.b[4] ? M.b[0] : M.b[4];
 
 	mglBlock p;	p.AmbBr = AmbBr;	p.DifBr = DifBr;	p.B = M;
 	for(int i=0;i<10;i++)	p.light[i] = light[i];
@@ -705,8 +704,8 @@ void mglCanvas::Rotate(mreal tetz,mreal tetx,mreal tety)
 	B.Rotate(tetz,tetx,tety);
 	if(get(MGL_AUTO_FACTOR))
 	{
-		mreal w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
-		mreal h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
+		float w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
+		float h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
 		B.pf = 1.55+0.6147*(w<h ? (h-1):(w-1));
 	}
 	size_t n = Sub.size();	if(n>0)	Sub[n-1].B = B;
@@ -717,12 +716,12 @@ void mglMatrix::Rotate(mreal tetz,mreal tetx,mreal tety)
 //	RotateN(TetX,1.,0.,0.);
 //	RotateN(TetY,0.,1.,0.);
 //	RotateN(TetZ,0.,0.,1.);
-	mreal R[9], O[9];
-	mreal cx=cos(tetx*M_PI/180), sx=-sin(tetx*M_PI/180), cy=cos(tety*M_PI/180), sy=-sin(tety*M_PI/180), cz=cos(tetz*M_PI/180), sz=-sin(tetz*M_PI/180);
+	float R[9], O[9];
+	float cx=cos(tetx*M_PI/180), sx=-sin(tetx*M_PI/180), cy=cos(tety*M_PI/180), sy=-sin(tety*M_PI/180), cz=cos(tetz*M_PI/180), sz=-sin(tetz*M_PI/180);
 	R[0] = cx*cy;			R[1] = -cy*sx;			R[2] = sy;
 	R[3] = cx*sy*sz+cz*sx;	R[4] = cx*cz-sx*sy*sz;	R[5] =-cy*sz;
 	R[6] = sx*sz-cx*cz*sy;	R[7] = cx*sz+cz*sx*sy;	R[8] = cy*cz;
-	memcpy(O,b,9*sizeof(mreal));
+	memcpy(O,b,9*sizeof(float));
 	b[0] = R[0]*O[0] + R[3]*O[1] + R[6]*O[2];
 	b[1] = R[1]*O[0] + R[4]*O[1] + R[7]*O[2];
 	b[2] = R[2]*O[0] + R[5]*O[1] + R[8]*O[2];
@@ -739,8 +738,8 @@ void mglCanvas::RotateN(mreal Tet,mreal x,mreal y,mreal z)
 	B.RotateN(Tet,x,y,z);
 	if(get(MGL_AUTO_FACTOR))
 	{
-		mreal w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
-		mreal h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
+		float w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
+		float h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
 		B.pf = 1.55+0.6147*(w<h ? (h-1):(w-1));
 	}
 	size_t n = Sub.size();	if(n>0)	Sub[n-1].B = B;
@@ -748,12 +747,12 @@ void mglCanvas::RotateN(mreal Tet,mreal x,mreal y,mreal z)
 //-----------------------------------------------------------------------------
 void mglMatrix::RotateN(mreal Tet,mreal vx,mreal vy,mreal vz)
 {
-	mreal R[9],T[9],c=cos(Tet*M_PI/180),s=-sin(Tet*M_PI/180),r=1-c,n=sqrt(vx*vx+vy*vy+vz*vz);
+	float R[9],T[9],c=cos(Tet*M_PI/180),s=-sin(Tet*M_PI/180),r=1-c,n=sqrt(vx*vx+vy*vy+vz*vz);
 	vx/=n;	vy/=n;	vz/=n;
 	T[0] = vx*vx*r+c;		T[1] = vx*vy*r-vz*s;	T[2] = vx*vz*r+vy*s;
 	T[3] = vx*vy*r+vz*s;	T[4] = vy*vy*r+c;		T[5] = vy*vz*r-vx*s;
 	T[6] = vx*vz*r-vy*s;	T[7] = vy*vz*r+vx*s;	T[8] = vz*vz*r+c;
-	memcpy(R,b,9*sizeof(mreal));
+	memcpy(R,b,9*sizeof(float));
 	b[0] = T[0]*R[0] + T[3]*R[1] + T[6]*R[2];
 	b[1] = T[1]*R[0] + T[4]*R[1] + T[7]*R[2];
 	b[2] = T[2]*R[0] + T[5]*R[1] + T[8]*R[2];
@@ -815,8 +814,8 @@ void mglCanvas::Aspect(mreal Ax,mreal Ay,mreal Az)
 //-----------------------------------------------------------------------------
 void mglCanvas::Shear(mreal Sx,mreal Sy)
 {
-	mreal R[6], Fx=1+fabs(Sx)*inH/inW, Fy=1+fabs(Sy)*inW/inH;
-	memcpy(R,B.b,6*sizeof(mreal));
+	float Fx=1+fabs(Sx)*inH/inW, Fy=1+fabs(Sy)*inW/inH;
+	const float R[6]={B.b[0],B.b[1],B.b[2],B.b[3],B.b[4],B.b[5]};
 	B.b[0] = (R[0]+Sx*R[3])/Fx;	B.b[1] = (R[1]+Sx*R[4])/Fx;	B.b[2] = (R[2]+Sx*R[5])/Fx;
 	B.b[3] = (R[3]+Sy*R[0])/Fy;	B.b[4] = (R[4]+Sy*R[1])/Fy;	B.b[5] = (R[5]+Sy*R[2])/Fy;
 	size_t n = Sub.size();	if(n>0)	Sub[n-1].B = B;
@@ -919,8 +918,9 @@ std::wstring MGL_EXPORT mgl_ftoa(double v, const char *fmt)
 	memmove(se+i+1,se+2+st+dig,le-dig);	le=strlen(se);
 	// add '+' sign if required
 	if(plus && !strchr("-0niNI",se[0]))
-	{	memmove(se+1,se,le+1);	se[0]='+';
-		memmove(sf+1,sf,lf+1);	sf[0]='+';	}
+	{	for(size_t i=le+1;i>0;i--)	se[i]=se[i-1];
+		for(size_t i=lf+1;i>0;i--)	sf[i]=sf[i-1];
+		se[0] = sf[0] = '+';	}
 	if((lf>le && !mglchr(fmt,'f')) || !strcmp(sf,"0") || !strcmp(sf,"-0"))	strcpy(sf,se);
 	lf = strlen(sf);
 	std::wstring res;	res.reserve(lf+8);
@@ -1145,8 +1145,8 @@ void mglCanvas::Title(const wchar_t *title,const char *stl,mreal size)
 	int align;
 	bool box=mglchr(stl,'#'), col = mglGetStyle(stl,0,&align);
 	align = align&3;
-	mreal y=inY+inH-h;
-	mglPoint p(inX + inW*align/2.,y,3*Depth),q(NAN,NAN,NAN);
+	mreal y=inY+inH-h, zpos = 0;//3*Depth;
+	mglPoint p(inX + inW*align/2.,y,zpos),q(NAN,NAN,NAN);
 	mglMatrix M=B;	M.norot=true;
 	if(title)	text_plot(AddPnt(&M,p,-1,q,-1,0),title,stl,size);
 	if(box)	//	draw boungind box
@@ -1155,10 +1155,10 @@ void mglCanvas::Title(const wchar_t *title,const char *stl,mreal size)
 		if((Flag&3)==2 && !col)	{	mreal cc=c1;	c2=c1;	c1=cc;	}
 		else if((Flag&3)==2)	c1=AddTexture('k');
 		long k1,k2,k3,k4;
-		k1=AddPnt(&M,mglPoint(inX,y-h*0.4,3*Depth),c1,q,-1,0);
-		k2=AddPnt(&M,mglPoint(inX+inW,y-h*0.4,3*Depth),c1,q,-1,0);
-		k3=AddPnt(&M,mglPoint(inX,y+h,3*Depth),c1,q,-1,0);
-		k4=AddPnt(&M,mglPoint(inX+inW,y+h,3*Depth),c1,q,-1,0);
+		k1=AddPnt(&M,mglPoint(inX,y-h*0.4,zpos),c1,q,-1,0);
+		k2=AddPnt(&M,mglPoint(inX+inW,y-h*0.4,zpos),c1,q,-1,0);
+		k3=AddPnt(&M,mglPoint(inX,y+h,zpos),c1,q,-1,0);
+		k4=AddPnt(&M,mglPoint(inX+inW,y+h,zpos),c1,q,-1,0);
 		quad_plot(k1,k2,k3,k4);
 		k1=CopyNtoC(k1,c2);	k2=CopyNtoC(k2,c2);
 		k3=CopyNtoC(k3,c2);	k4=CopyNtoC(k4,c2);
@@ -1217,11 +1217,11 @@ void mglCanvas::Push()
 //-----------------------------------------------------------------------------
 void mglCanvas::Pop()
 {
-	B = stack.back(); 
+	B = stack.back();
 #if MGL_HAVE_PTHREAD
-	pthread_mutex_lock(&m);
+	pthread_mutex_lock(&mutexStk);
 	stack.pop_back();
-	pthread_mutex_unlock(&m);
+	pthread_mutex_unlock(&mutexStk);
 #else
 #pragma omp critical(stk)
 	stack.pop_back();
