@@ -22,6 +22,7 @@
 #include "mgl2/evalc.h"
 #include "mgl2/thread.h"
 #include "interp.hpp"
+void MGL_NO_EXPORT mgl_txt_funcC(const mreal *x, mreal *dx, void *par);
 HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 HADT MGL_EXPORT mgl_datac_trace(HCDT d)
@@ -190,7 +191,7 @@ uintptr_t MGL_EXPORT mgl_datac_subdata_(uintptr_t *d, int *xx,int *yy,int *zz)
 uintptr_t MGL_EXPORT mgl_datac_subdata_ext_(uintptr_t *d, uintptr_t *xx, uintptr_t *yy, uintptr_t *zz)
 {	return uintptr_t(mgl_datac_subdata_ext(_DC_,_DA_(xx),_DA_(yy),_DA_(zz)));	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_cresize(void *par)
+static void *mgl_cresize(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
 	long nx=t->p[0]+0.1, ny=t->p[1]+0.1;
@@ -264,7 +265,7 @@ HADT MGL_EXPORT mgl_datac_combine(HCDT d1, HCDT d2)
 uintptr_t MGL_EXPORT mgl_datac_combine_(uintptr_t *a, uintptr_t *b)
 {	return uintptr_t(mgl_datac_combine(_DA_(a),_DA_(b)));	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_sumc_z(void *par)
+static void *mgl_sumc_z(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
 	long nz=t->p[2], nn=t->n;
@@ -281,7 +282,7 @@ MGL_NO_EXPORT void *mgl_sumc_z(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_sumc_y(void *par)
+static void *mgl_sumc_y(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
 	long nx=t->p[0], ny=t->p[1], nn=t->n;
@@ -298,7 +299,7 @@ MGL_NO_EXPORT void *mgl_sumc_y(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_sumc_x(void *par)
+static void *mgl_sumc_x(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
 	long nx=t->p[0], nn=t->n;
@@ -357,10 +358,10 @@ HADT MGL_EXPORT mgl_datac_momentum(HCDT dat, char dir, const char *how)
 {
 	if(!how || !(*how) || !strchr("xyz",dir))	return 0;
 	long nx=dat->GetNx(),ny=dat->GetNy(),nz=dat->GetNz();
-	mglDataV x(nx,ny,nz, 0,1,'x');	x.s=L"x";
-	mglDataV y(nx,ny,nz, 0,1,'y');	y.s=L"y";
-	mglDataV z(nx,ny,nz, 0,1,'z');	z.s=L"z";
-	mglDataC u(dat);	u.s=L"u";	// NOTE slow !!!
+	mglDataV x(nx,ny,nz, 0,1,'x');	x.Name(L"x");
+	mglDataV y(nx,ny,nz, 0,1,'y');	y.Name(L"y");
+	mglDataV z(nx,ny,nz, 0,1,'z');	z.Name(L"z");
+	mglDataC u(dat);	u.Name(L"u");	// NOTE slow !!!
 	std::vector<mglDataA*> list;
 	list.push_back(&x);	list.push_back(&y);	list.push_back(&z);	list.push_back(&u);
 	HADT res=mglFormulaCalcC(how,list), b=0;
@@ -454,24 +455,18 @@ uintptr_t MGL_EXPORT mgl_datac_evaluate_(uintptr_t *d, uintptr_t *idat, uintptr_
 //-----------------------------------------------------------------------------
 HADT MGL_EXPORT mgl_datac_column(HCDT dat, const char *eq)
 {
-	const mglData *dd=dynamic_cast<const mglData *>(dat);
 	std::vector<mglDataA*> list;
-	if(dd && dd->id.length()>0)	for(size_t i=0;i<dd->id.length();i++)
+	const char *id = dat->GetColumnId();
+	size_t len = strlen(id);
+	for(size_t i=0;i<len;i++)
 	{
 		mglDataT *col = new mglDataT(*dat);
-		col->SetInd(i,dd->id[i]);
-		list.push_back(col);
-	}
-	const mglDataC *dc=dynamic_cast<const mglDataC *>(dat);
-	if(dc && dc->id.length()>0)	for(size_t i=0;i<dc->id.length();i++)
-	{
-		mglDataT *col = new mglDataT(*dat);
-		col->SetInd(i,dc->id[i]);
+		col->SetInd(i,id[i]);
 		list.push_back(col);
 	}
 	if(list.size()==0)	return 0;	// no named columns
 	mglDataV *t = new mglDataV(dat->GetNy(),dat->GetNz());
-	t->s=L"#$mgl";	list.push_back(t);
+	t->Name(L"#$mgl");	list.push_back(t);
 	HADT r = mglFormulaCalcC(eq,list);
 	for(size_t i=0;i<list.size();i++)	delete list[i];
 	return r;
@@ -704,3 +699,27 @@ uintptr_t MGL_EXPORT mgl_datac_section_(uintptr_t *d, uintptr_t *ids, const char
 uintptr_t MGL_EXPORT mgl_datac_section_val_(uintptr_t *d, int *id, const char *dir, mreal *val,int)
 {	return uintptr_t(mgl_datac_section_val(_DT_,*id,dir[0],*val));	}
 //-----------------------------------------------------------------------------
+HADT MGL_EXPORT mgl_find_roots_txt_c(const char *func, const char *vars, HCDT ini)
+{
+	if(!vars || !(*vars) || !func || !ini)	return 0;
+	mglEqTxT par;
+	par.var=vars;	par.FillCmplx(func);
+	size_t n = par.str.size();
+	if(ini->GetNx()!=long(n))	return 0;
+	mreal *xx = new mreal[2*n];
+	mglDataC *res = new mglDataC(ini);
+	for(long j=0;j<ini->GetNy()*ini->GetNz();j++)
+	{
+		for(size_t i=0;i<n;i++)
+		{	dual c = ini->vcthr(i+n*j);	xx[2*i] = real(c);	xx[2*i+1] = imag(c);	}
+		bool ok=mgl_find_roots(2*n,mgl_txt_funcC,xx,&par);
+		for(size_t i=0;i<n;i++)	res->a[i+n*j] = ok?dual(xx[2*i],xx[2*i+1]):NAN;
+	}
+	delete []xx;	return res;
+}
+uintptr_t MGL_EXPORT mgl_find_roots_txt_c_(const char *func, const char *vars, uintptr_t *ini,int l,int m)
+{	char *s=new char[l+1];	memcpy(s,func,l);	s[l]=0;
+	char *v=new char[m+1];	memcpy(v,vars,m);	v[m]=0;
+	uintptr_t r = uintptr_t(mgl_find_roots_txt_c(s,v,_DA_(ini)));
+	delete []s;	delete []v;	return r;	}
+//-----------------------------------------------------------------------------
