@@ -44,7 +44,7 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,cons
 	if(!font)	font="";
 	int pos = strchr(font,'T') ? 1:-1, align;
 	bool cc=mglGetStyle(font,0,&align);		align = align&3;
-	mreal h=gr->TextHeight(font,size)/2;
+	mreal h=gr->TextHeight(font,size)/2, g = 1.1*h;
 	wchar_t L[2]=L"a";
 
 	if(align==1)	// TODO divide curve by 2
@@ -59,7 +59,7 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,cons
 	for(long i=nn[f];i>=0 && i!=f;i=nn[i])	// find second real point
 		if(ff[i]>=0)	{	s=gr->GetPntP(ff[i]);	break;	}
 	mglPoint l=!(s-q), t=l;
-	qa.push_back(q+l*h);	qb.push_back(q-l*h);
+	qa.push_back(q+l*g);	qb.push_back(q-l*h);
 	for(long i=nn[f];i>=0 && i!=f;i=nn[i])	// construct curves
 	{
 		p=q;	q=s;	l=t;
@@ -67,11 +67,11 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,cons
 		mreal tet = t.x*l.y-t.y*l.x;
 		mreal tt = 1+fabs(t.x*l.x+t.y*l.y);
 		if(tet>0)
-		{	qa.push_back(q+l*h);	qa.push_back(q+t*h);	qb.push_back(q-(l+t)*(h/tt));	}
+		{	qa.push_back(q+l*g);	qa.push_back(q+t*g);	qb.push_back(q-(l+t)*(h/tt));	}
 		else if(tet<0)
-		{	qb.push_back(q-l*h);	qb.push_back(q-t*h);	qa.push_back(q+(l+t)*(h/tt));	}
+		{	qb.push_back(q-l*h);	qb.push_back(q-t*h);	qa.push_back(q+(l+t)*(g/tt));	}
 		else
-		{	qa.push_back(q+l*h);	qb.push_back(q-l*h);	}
+		{	qa.push_back(q+l*g);	qb.push_back(q-l*h);	}
 	}
 	if(pos>0)	qa=qb;
 	// adjust text direction
@@ -79,12 +79,12 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,cons
 	const char *ffont=mglchr(font,':');
 	char *fnt = new char[strlen(font)+5];
 	if(ffont) strcpy(fnt,ffont);	else *fnt=0;
-	if(qa[0].x>qa[1].x)
+/*	if(qa[0].x>qa[1].x)
 	{
 		if(align==0){	strcat(fnt,":R");	align=2;	}
 		else if(align==1)	rev = true;
 		else		{	strcat(fnt,":L");	align=0;	}
-	}
+	}*/
 	if(mglchr(font,'T'))	strcat(fnt,":T");
 	if(rev)	reverse(qa.begin(),qa.end());
 	long len = mgl_wcslen(text);
@@ -99,7 +99,8 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,cons
 	mreal t1,t2, tt=0;
 	for(long j=0;j<len;j++)
 	{
-		mreal w = align==1 ? wdt[j] : (wdt[j]+wdt[j+1])/2;	p = pt[j];
+		mreal w = align==1 ? wdt[j] : (wdt[j]+wdt[j+1])/2;
+		p = pt[j];	w *= 1.03;
 		for(k=i+1;k<m;k++)	if((p-qa[k]).norm()>w)	break;
 		if(k>i+1 && k<m)	tt=-1;
 		i = k<m ? k-1 : m-2;		// check if end of curve
@@ -129,14 +130,15 @@ void MGL_EXPORT mgl_textw_xyz(HMGL gr, HCDT x, HCDT y, HCDT z,const wchar_t *tex
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("TextC",cgid++);
 
+	long kq = gr->AllocPnts(n);
 	long *nn = new long[n], *ff = new long[n];
+#pragma omp parallel for
 	for(long i=0;i<n;i++)
-		ff[i] = gr->AddPnt(mglPoint(x->v(i),y->v(i),z->v(i)),-1);
-	for(long i=1;i<n;i++)	nn[i-1] = i;
+	{	ff[i] = kq+i;	nn[i] = i+1;
+		gr->AddPntQ(kq+i,mglPoint(x->v(i),y->v(i),z->v(i)),-1);	}
 	nn[n-1]=-1;
 	mgl_string_curve(gr,0,n,ff,nn,text,font,-1);
-	gr->EndGroup();
-	delete []ff;	delete []nn;
+	delete []ff;	delete []nn;	gr->EndGroup();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textw_xy(HMGL gr, HCDT x, HCDT y, const wchar_t *text, const char *font, const char *opt)
@@ -349,22 +351,28 @@ void MGL_NO_EXPORT mgl_draw_curvs(HMGL gr, mreal val, mreal c, int text, const s
 		mglprintf(wcs,64,L"%4.3g",val);
 		mreal del = 2*gr->TextWidth(wcs,"",-0.5);
 		// find width and height of drawing area
-		mreal ar=gr->GetRatio(), w=gr->FontFactor(), h;
-		if(del<w/5)	del = w/5;
-		if(ar<1) h=w/ar;	else {	h=w;	w*=ar;	}
-		long m=long(2*w/del)+1, n=long(2*h/del)+1;
+		mreal ar=gr->GetRatio(), w=gr->GetWidth(), h = gr->GetHeight();
+		ar = (ar>1?1/ar:1)*gr->FontFactor();
+		if(del<ar/2)	del = ar/2;
+
+		long m=long(2*w/del)+3, n=long(2*h/del)+3;
 		long *oo=new long[n*m];
 		mreal *rr=new mreal[n*m];
-		for(long i=0;i<n*m;i++)	{	oo[i]=-1;	rr[i]=del*del/4;	}
+		for(long i=0;i<n*m;i++)	{	oo[i]=-1;	rr[i]=del*del;	}
+		int ii1 = (1664525*pc+1013904223)&0xffff, ii2 = (1664525*ii1+1013904223)&0xffff;
+		mreal x0 = (del*ii1)/0xffff, y0 = (del*ii2)/0xffff;
 		for(long k=0;k<pc;k++)	// print label several times if possible
 		{
 			if(nn[k]<0)	continue;
-			mglPoint t = gr->GetPntP(ff[k]);
-			long i = long(t.x/del);	t.x -= i*del;
-			long j = long(t.y/del);	t.y -= j*del;
-			if(i<0 || i>=m || j<0 || j>=n)	continue;	// never should be here!
-			mreal xx = t.x*t.x+t.y*t.y;	i += m*j;
-			if(rr[i]>xx)	{	rr[i]=xx;	oo[i]=k;	}
+			const mglPoint t = gr->GetPntP(ff[k]);
+			mreal tx = t.x+x0, ty = t.y+y0;		// quasi-random shift
+			long i = long(tx/del);	tx -= i*del;
+			long j = long(ty/del);	ty -= j*del;
+			if(i>=0 && i<m && j>=0 && j<n)
+			{
+				tx = tx*tx+ty*ty;	i += m*j;
+				if(rr[i]>tx)	{	rr[i]=tx;	oo[i]=k;	}
+			}
 		}
 		for(long i=0;i<n*m;i++)	if(oo[i]>=0)
 			mgl_string_curve(gr,oo[i],pc,ff,nn,wcs,text==1?"t:C":"T:C",-0.5);
@@ -443,24 +451,92 @@ void MGL_EXPORT mgl_cont_val(HMGL gr, HCDT v, HCDT z, const char *sch, const cha
 	mgl_cont_xy_val(gr,v,&x,&y,z,sch,0);
 }
 //-----------------------------------------------------------------------------
+#define norm(x,y)	((x)*(x)+(y)*(y))
+std::vector<mreal> MGL_NO_EXPORT mgl_find_saddle_val(HCDT z)
+{
+	long nx=z->GetNx(), ny=z->GetNy();
+	std::vector<mreal> v;
+	for(long i=1;i<nx-1;i++)
+	{
+		mreal dd=z->vthr(i), x1=z->vthr(i+1), x2=z->vthr(i-1), dp=z->vthr(i+nx);
+		if(dd<=x1 && dd<=x2 && dd>=dp)	v.push_back(z->vthr(i));
+		if(dd>=x1 && dd>=x2 && dd<=dp)	v.push_back(z->vthr(i));
+		long i0 = i+nx*(ny-1);
+		dd=z->vthr(i0);	x1=z->vthr(i0+1);	x2=z->vthr(i0-1);	dp=z->vthr(i0-nx);
+		if(dd<=x1 && dd<=x2 && dd>=dp)	v.push_back(z->vthr(i0));
+		if(dd>=x1 && dd>=x2 && dd<=dp)	v.push_back(z->vthr(i0));
+	}
+	for(long j=1;j<ny-1;j++)
+	{
+		long i0 = nx*j;
+		mreal dd=z->vthr(i0), dp=z->vthr(i0+1), y1=z->vthr(i0+nx), y2=z->vthr(i0-nx);
+		if(dd<=dp && dd>=y1 && dd>=y2)	v.push_back(z->vthr(i0));
+		if(dd>=dp && dd<=y1 && dd<=y2)	v.push_back(z->vthr(i0));
+		i0 = nx*j+nx-1;
+		dd=z->vthr(i0);	dp=z->vthr(i0-1);	y1=z->vthr(i0+nx);	y2=z->vthr(i0-nx);
+		if(dd<=dp && dd>=y1 && dd>=y2)	v.push_back(z->vthr(i0));
+		if(dd>=dp && dd<=y1 && dd<=y2)	v.push_back(z->vthr(i0));
+	}
+	for(long j=1;j<ny-1;j++)	for(long i=1;i<nx-1;i++)
+	{
+		long i0 = i+nx*j;	bool ok=false;
+		mreal dd=z->vthr(i0),x1=z->vthr(i0+1),x2=z->vthr(i0-1),y1=z->vthr(i0+nx),y2=z->vthr(i0-nx);
+		if(dd<=x1 && dd<=x2 && dd>=y1 && dd>=y2)	ok=true;
+		if(dd>=x1 && dd>=x2 && dd<=y1 && dd<=y2)	ok=true;
+		x1=z->vthr(i0+1+nx);	x2=z->vthr(i0-1-nx);
+		y1=z->vthr(i0-1+nx);	y2=z->vthr(i0+1-nx);
+		if(dd<=x1 && dd<=x2 && dd>=y1 && dd>=y2)	ok=true;
+		if(dd>=x1 && dd>=x2 && dd<=y1 && dd<=y2)	ok=true;
+		if(ok)	v.push_back(z->vthr(i0));
+	}
+	return v;
+}
 void MGL_EXPORT mgl_cont_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
-	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
-	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
-	mgl_cont_xy_val(gr,&v,x,y,z,sch,0);
+	if(mglchr(sch,'.'))
+	{
+		mglDataS v;	v.dat = mgl_find_saddle_val(z);
+		if(v.dat.size()>0)
+		{
+			std::sort( v.dat.begin(), v.dat.end() );
+			v.dat.erase( std::unique( v.dat.begin(), v.dat.end() ), v.dat.end() );
+			mgl_cont_xy_val(gr,&v,x,y,z,sch,0);
+		}
+		else	gr->SetWarn(mglWarnCnt,"Cont");
+	}
+	else
+	{
+		long Num = mgl_isnan(r)?7:long(r+0.5);
+		if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
+		mglData v(Num);
+		for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
+		mgl_cont_xy_val(gr,&v,x,y,z,sch,0);
+	}
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
-	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
-	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
-	mgl_cont_val(gr,&v,z,sch,0);
+	if(mglchr(sch,'.'))
+	{
+		mglDataS v;	v.dat = mgl_find_saddle_val(z);
+		if(v.dat.size()>0)
+		{
+			std::sort( v.dat.begin(), v.dat.end() );
+			v.dat.erase( std::unique( v.dat.begin(), v.dat.end() ), v.dat.end() );
+			mgl_cont_val(gr,&v,z,sch,0);
+		}
+		else	gr->SetWarn(mglWarnCnt,"Cont");
+	}
+	else
+	{
+		long Num = mgl_isnan(r)?7:long(r+0.5);
+		if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
+		mglData v(Num);
+		for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
+		mgl_cont_val(gr,&v,z,sch,0);
+	}
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -488,7 +564,7 @@ mgl_cont(_GR_, _DA_(a), s, o);	delete []o;	delete []s;	}
 //	ContF series
 //
 //-----------------------------------------------------------------------------
-long MGL_NO_EXPORT mgl_add_pnt(HMGL gr, mreal d, HCDT x, HCDT y, HCDT z, long i1, long j1, long i2, long j2, mreal c, bool edge)
+long static mgl_add_pnt(HMGL gr, mreal d, HCDT x, HCDT y, HCDT z, long i1, long j1, long i2, long j2, mreal c, bool edge)
 {
 	long res=-1;
 	if(edge || (d>0 && d<1))
@@ -507,7 +583,7 @@ long MGL_NO_EXPORT mgl_add_pnt(HMGL gr, mreal d, HCDT x, HCDT y, HCDT z, long i1
 	return res;
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_add_range(HMGL gr, HCDT a, HCDT x, HCDT y, HCDT z, long i1, long j1, long di, long dj, mreal c, long &u1, long &u2, long ak, mreal v1, mreal v2)
+void static mgl_add_range(HMGL gr, HCDT a, HCDT x, HCDT y, HCDT z, long i1, long j1, long di, long dj, mreal c, long &u1, long &u2, long ak, mreal v1, mreal v2)
 {
 	long i2=i1+di, j2=j1+dj;
 	mreal f1 = a->v(i1,j1,ak),	f2 = a->v(i2,j2,ak), d1, d2;
@@ -519,7 +595,7 @@ void MGL_NO_EXPORT mgl_add_range(HMGL gr, HCDT a, HCDT x, HCDT y, HCDT z, long i
 	if(u1<0)	{	u1=u2;	u2=-1;	}
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_add_edges(HMGL gr, HCDT a, HCDT x, HCDT y, HCDT z, long i1, long j1, long di, long dj, mreal c, long &u1, long &u2, long ak, mreal v1, mreal v2)
+void static mgl_add_edges(HMGL gr, HCDT a, HCDT x, HCDT y, HCDT z, long i1, long j1, long di, long dj, mreal c, long &u1, long &u2, long ak, mreal v1, mreal v2)
 {
 	long i2=i1+di, j2=j1+dj;
 	u1 = u2 = -1;
@@ -830,7 +906,7 @@ void MGL_EXPORT mgl_contp_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t
 //	ContD series
 //
 //-----------------------------------------------------------------------------
-int MGL_NO_EXPORT mgl_get_ncol(const char *sch, char *res)
+int static mgl_get_ncol(const char *sch, char *res)
 {
 	long j=0;
 	if(sch)	for(long i=0;sch[i]&&sch[i]!=':';i++)	if(strchr(MGL_COLORS,sch[i]))
@@ -1386,14 +1462,13 @@ long MGL_LOCAL_PURE mgl_find_prev(long i, long pc, long *nn)
 	for(long k=0;k<pc;k++)	if(nn[k]==i)	return k;
 	return -1;
 }
-void MGL_NO_EXPORT mgl_axial_plot(mglBase *gr,long pc, mglPoint *ff, long *nn,char dir,mreal cc,int wire)
+void static mgl_axial_plot(mglBase *gr,long pc, mglPoint *ff, long *nn,char dir,mreal cc,int wire)
 {
-	mglPoint a(0,0,1),b,c,p,q1,q2;
+	mglPoint a(0,0,1),b,c,q1,q2;
 	if(dir=='x')	a.Set(1,0,0);
 	if(dir=='y')	a.Set(0,1,0);
 	b = !a;	c = a^b;
 
-	long p1,p2,p3,p4;
 	gr->Reserve(pc*82);
 	for(long i=0;i<pc;i++)
 	{
@@ -1402,28 +1477,35 @@ void MGL_NO_EXPORT mgl_axial_plot(mglBase *gr,long pc, mglPoint *ff, long *nn,ch
 		q1 = k<0 ? ff[nn[i]]-ff[i]  : (ff[nn[i]]-ff[k])*0.5;
 		q2 = nn[nn[i]]<0 ? ff[nn[i]]-ff[i]  : (ff[nn[nn[i]]]-ff[i])*0.5;
 
-		p = a*ff[i].y + c*ff[i].x;
-		p1 = wire ? gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q1.y + c*q1.x)^b);
-		p = a*ff[nn[i]].y + c*ff[nn[i]].x;
-		p2 = wire ? gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q2.y + c*q2.x)^b);
-		if(wire==1)	gr->line_plot(p1,p2);
-		else if(wire)	{	gr->mark_plot(p1,'.');	gr->mark_plot(p2,'.');	}
-
-		for(long j=1;j<41;j++)
+		long kq = gr->AllocPnts(41*2);
+#pragma omp parallel for
+		for(long j=0;j<41;j++)
 		{
-			p3 = p1;	p4 = p2;
 			float co = mgl_cos[(j*18)%360], si = mgl_cos[(270+j*18)%360];
 //			fi = j*M_PI/20;		si = sin(fi);	co = cos(fi);
-			p = a*ff[i].y + b*(si*ff[i].x) +  c*(co*ff[i].x);
-			p1 = wire ?	gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q1.y + b*(si*q1.x) +  c*(co*q1.x))^(b*co-c*si));
-			p = a*ff[nn[i]].y + b*(si*ff[nn[i]].x) +  c*(co*ff[nn[i]].x);
-			p2 = wire ?	gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q2.y + b*(si*q2.x) +  c*(co*q2.x))^(b*co-c*si));
-			if(wire==1)
-			{	gr->line_plot(p1,p2);	gr->line_plot(p1,p3);
-			gr->line_plot(p4,p2);	gr->line_plot(p4,p3);	}
-			else if(wire)	{	gr->mark_plot(p1,'.');	gr->mark_plot(p2,'.');	}
-			else	gr->quad_plot(p3,p4,p1,p2);
+			mglPoint p1 = a*ff[i].y + b*(si*ff[i].x) +  c*(co*ff[i].x);
+			mglPoint p2 = a*ff[nn[i]].y + b*(si*ff[nn[i]].x) +  c*(co*ff[nn[i]].x);
+			if(wire)
+			{	gr->AddPntQ(kq+2*j,p1,cc);	gr->AddPntQ(kq+2*j+1,p2,cc);	}
+			else
+			{
+				gr->AddPntQ(kq+2*j, p1,cc,(a*q1.y + b*(si*q1.x) +  c*(co*q1.x))^(b*co-c*si));
+				gr->AddPntQ(kq+2*j+1,p2,cc,(a*q2.y + b*(si*q2.x) +  c*(co*q2.x))^(b*co-c*si));
+			}
 		}
+		if(wire==1)
+		{
+			gr->line_plot(kq,kq+1);
+			for(long j=1;j<41;j++)
+			{
+				gr->line_plot(kq+2*j,kq+2*j+1);		gr->line_plot(kq+2*j,kq+2*j-2);
+				gr->line_plot(kq+2*j-1,kq+2*j+1);	gr->line_plot(kq+2*j-1,kq+2*j-2);
+			}
+		}
+		else if(wire)	for(long j=0;j<41;j++)
+		{	gr->mark_plot(kq+2*j,'.');	gr->mark_plot(kq+2*j+1,'.');	}
+		else	for(long j=1;j<41;j++)
+		{	long i0 = kq+2*j;	gr->quad_plot(i0-2,i0-1,i0,i0+1);	}
 	}
 }
 //-----------------------------------------------------------------------------
