@@ -21,6 +21,8 @@
 #include "mgl2/data.h"
 #include "mgl2/eval.h"
 #include "mgl2/thread.h"
+#include "interp.hpp"
+void MGL_NO_EXPORT mgl_txt_func(const mreal *x, mreal *dx, void *par);
 HMDT MGL_NO_EXPORT mglFormulaCalc(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_data_trace(HCDT d)
@@ -154,7 +156,7 @@ uintptr_t MGL_EXPORT mgl_data_subdata_(uintptr_t *d, int *xx,int *yy,int *zz)
 uintptr_t MGL_EXPORT mgl_data_subdata_ext_(uintptr_t *d, uintptr_t *xx, uintptr_t *yy, uintptr_t *zz)
 {	return uintptr_t(mgl_data_subdata_ext(_DT_,_DA_(xx),_DA_(yy),_DA_(zz)));	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_resize(void *par)
+static void *mgl_resize(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0]+0.1, ny=t->p[1]+0.1;
@@ -209,7 +211,7 @@ HMDT MGL_EXPORT mgl_data_combine(HCDT d1, HCDT d2)
 uintptr_t MGL_EXPORT mgl_data_combine_(uintptr_t *a, uintptr_t *b)
 {	return uintptr_t(mgl_data_combine(_DA_(a),_DA_(b)));	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_sum_z(void *par)
+static void *mgl_sum_z(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nz=t->p[2], nn=t->n;
@@ -226,7 +228,7 @@ MGL_NO_EXPORT void *mgl_sum_z(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_sum_y(void *par)
+static void *mgl_sum_y(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], ny=t->p[1], nn=t->n;
@@ -243,7 +245,7 @@ MGL_NO_EXPORT void *mgl_sum_y(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_sum_x(void *par)
+static void *mgl_sum_x(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], nn=t->n;
@@ -298,7 +300,7 @@ uintptr_t MGL_EXPORT mgl_data_sum_(uintptr_t *d, const char *dir,int l)
 {	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
 	uintptr_t r=uintptr_t(mgl_data_sum(_DT_,s));	delete []s;	return r;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_max_z(void *par)
+static void *mgl_max_z(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nz=t->p[2], nn=t->n;
@@ -314,7 +316,7 @@ MGL_NO_EXPORT void *mgl_max_z(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_max_y(void *par)
+static void *mgl_max_y(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], ny=t->p[1], nn=t->n;
@@ -330,7 +332,7 @@ MGL_NO_EXPORT void *mgl_max_y(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_max_x(void *par)
+static void *mgl_max_x(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], nn=t->n;
@@ -384,7 +386,7 @@ uintptr_t MGL_EXPORT mgl_data_max_dir_(uintptr_t *d, const char *dir,int l)
 {	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
 	uintptr_t r=uintptr_t(mgl_data_max_dir(_DT_,s));	delete []s;	return r;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_min_z(void *par)
+static void *mgl_min_z(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nz=t->p[2], nn=t->n;
@@ -400,7 +402,7 @@ MGL_NO_EXPORT void *mgl_min_z(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_min_y(void *par)
+static void *mgl_min_y(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], ny=t->p[1], nn=t->n;
@@ -416,7 +418,7 @@ MGL_NO_EXPORT void *mgl_min_y(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_min_x(void *par)
+static void *mgl_min_x(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], nn=t->n;
@@ -474,10 +476,10 @@ HMDT MGL_EXPORT mgl_data_momentum(HCDT dat, char dir, const char *how)
 {
 	if(!how || !(*how) || !strchr("xyz",dir))	return 0;
 	long nx=dat->GetNx(),ny=dat->GetNy(),nz=dat->GetNz();
-	mglDataV x(nx,ny,nz, 0,1,'x');	x.s=L"x";
-	mglDataV y(nx,ny,nz, 0,1,'y');	y.s=L"y";
-	mglDataV z(nx,ny,nz, 0,1,'z');	z.s=L"z";
-	mglData u(dat);	u.s=L"u";	// NOTE slow !!!
+	mglDataV x(nx,ny,nz, 0,1,'x');	x.Name(L"x");
+	mglDataV y(nx,ny,nz, 0,1,'y');	y.Name(L"y");
+	mglDataV z(nx,ny,nz, 0,1,'z');	z.Name(L"z");
+	mglData u(dat);	u.Name(L"u");	// NOTE slow !!!
 	std::vector<mglDataA*> list;
 	list.push_back(&x);	list.push_back(&y);	list.push_back(&z);	list.push_back(&u);
 	HMDT res=mglFormulaCalc(how,list), b=0;
@@ -659,7 +661,7 @@ void MGL_EXPORT mgl_data_div_num_(uintptr_t *d, mreal *b)		{	mgl_data_div_num(_D
 void MGL_EXPORT mgl_data_add_num_(uintptr_t *d, mreal *b)		{	mgl_data_add_num(_DT_, *b);	}
 void MGL_EXPORT mgl_data_sub_num_(uintptr_t *d, mreal *b)		{	mgl_data_sub_num(_DT_, *b);	}
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_hist_p(mglThreadD *t,mreal *a)
+void static mgl_hist_p(mglThreadD *t,mreal *a)
 {
 	long n=t[0].p[0];
 	memset(a,0,n*sizeof(mreal));
@@ -673,7 +675,7 @@ void MGL_NO_EXPORT mgl_hist_p(mglThreadD *t,mreal *a)
 		delete []b;
 	}
 }
-MGL_NO_EXPORT void *mgl_hist_1(void *par)
+static void *mgl_hist_1(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nn=t->n, n = t->p[0];
@@ -695,7 +697,7 @@ MGL_NO_EXPORT void *mgl_hist_1(void *par)
 	}
 	t->a = b;	return 0;
 }
-MGL_NO_EXPORT void *mgl_hist_2(void *par)
+static void *mgl_hist_2(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nn=t->n, n = t->p[0];
@@ -773,6 +775,85 @@ void MGL_EXPORT mgl_data_sort(HMDT dat, long idx, long idy)
 void MGL_EXPORT mgl_data_sort_(uintptr_t *d, int *idx, int *idy)
 {	mgl_data_sort(_DT_,*idx,*idy);	}
 //-----------------------------------------------------------------------------
+//
+//	Find roots series
+//
+//
+//-----------------------------------------------------------------------------
+#if MGL_HAVE_GSL
+#include <gsl/gsl_multiroots.h>
+struct mglRoots
+{
+	mreal *x, *f;
+	size_t n;
+	void (*func)(const mreal *x, mreal *f, void *par);
+	void *par;
+};
+int static mgl_root(const gsl_vector *x, void *params, gsl_vector *f)
+{
+	mglRoots *p = (mglRoots *)params;
+	for(size_t i=0;i<p->n;i++)	p->x[i] = gsl_vector_get (x, i);
+	p->func(p->x,p->f,p->par);
+	bool ok = true;
+	for(size_t i=0;i<p->n;i++)
+	{	gsl_vector_set (f, i, p->f[i]);
+		if(mgl_isbad(p->f[i]))	ok=false;	}
+	return ok?GSL_SUCCESS:GSL_FAILURE;
+}
+bool MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f, void *par), mreal *x0, void *par)
+{
+	for(size_t i=0;i<n;i++)	if(mgl_isbad(x0[i]))	return false;
+	mglRoots pp;
+	pp.x=x0;	pp.func=func;	pp.n=n;	pp.par=par;
+	mreal *y = new mreal[n];	pp.f=y;
+
+	gsl_multiroot_function f = {&mgl_root, n, &pp};
+	gsl_vector *x = gsl_vector_alloc(n);
+	for(size_t i=0;i<n;i++)	gsl_vector_set(x, i, x0[i]);
+
+	gsl_multiroot_fsolver *s = gsl_multiroot_fsolver_alloc(gsl_multiroot_fsolver_hybrids, n);
+	gsl_multiroot_fsolver_set(s, &f, x);
+
+	int status = GSL_CONTINUE;
+	for(size_t i=0;i<1000 && status==GSL_CONTINUE;i++)
+	{
+	  status = gsl_multiroot_fsolver_iterate(s);
+	  if(status)	break;
+	  status = gsl_multiroot_test_residual(s->f, 1e-7);
+	}
+	for(size_t i=0;i<n;i++)	x0[i] = gsl_vector_get(s->x, i);
+	gsl_multiroot_fsolver_free(s);
+	gsl_vector_free(x);	delete []y;
+	return status==GSL_SUCCESS;
+}
+#else
+bool MGL_EXPORT mgl_find_roots(size_t , void (*)(const mreal *, mreal *, void *), mreal *, void *)
+{	return false;	}
+#endif // MGL_HAVE_GSL
+//-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_find_roots_txt(const char *func, const char *vars, HCDT ini)
+{
+	if(!vars || !(*vars) || !func || !ini)	return 0;
+	mglEqTxT par;
+	par.var=vars;	par.FillReal(func);
+	size_t n = par.str.size();
+	if(ini->GetNx()!=n)	return 0;
+	mreal *xx = new mreal[n];
+	mglData *res = new mglData(ini);
+	for(long j=0;j<ini->GetNy()*ini->GetNz();j++)
+	{
+		for(size_t i=0;i<n;i++)	xx[i] = ini->vthr(i+n*j);
+		bool ok=mgl_find_roots(n,mgl_txt_func,xx,&par);
+		for(size_t i=0;i<n;i++)	res->a[i+n*j] = ok?xx[i]:NAN;
+	}
+	delete []xx;	return res;
+}
+uintptr_t MGL_EXPORT mgl_find_roots_txt_(const char *func, const char *vars, uintptr_t *ini,int l,int m)
+{	char *s=new char[l+1];	memcpy(s,func,l);	s[l]=0;
+	char *v=new char[m+1];	memcpy(v,vars,m);	v[m]=0;
+	uintptr_t r = uintptr_t(mgl_find_roots_txt(s,v,_DA_(ini)));
+	delete []s;	delete []v;	return r;	}
+//-----------------------------------------------------------------------------
 mreal MGL_EXPORT mgl_find_root(mreal (*func)(mreal x, void *par), mreal x0, void *par)
 {
 	mreal x1=x0+1e-2*(x0?x0:1), f0=func(x0,par), f1=func(x1,par), x, f;
@@ -794,7 +875,7 @@ mreal MGL_EXPORT mgl_find_root(mreal (*func)(mreal x, void *par), mreal x0, void
 }
 //-----------------------------------------------------------------------------
 struct MGL_NO_EXPORT mglFuncV	{	mglFormula *eq;	char var;	};
-mreal MGL_NO_EXPORT mgl_funcv(mreal v, void *par)
+mreal static mgl_funcv(mreal v, void *par)
 {
 	mglFuncV *f = (mglFuncV *)par;
 	mreal var[MGL_VS];	memset(var,0,MGL_VS*sizeof(mreal));
@@ -831,7 +912,7 @@ mreal MGL_EXPORT mgl_find_root_txt_(const char *func, mreal *ini, const char *va
 	mreal r = mgl_find_root_txt(s,*ini,*var);
 	delete []s;	return r;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_pulse_z(void *par)
+static void *mgl_pulse_z(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nz=t->p[2], nn=t->n;
@@ -871,7 +952,7 @@ MGL_NO_EXPORT void *mgl_pulse_z(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_pulse_y(void *par)
+static void *mgl_pulse_y(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], ny=t->p[1], nn=t->n;
@@ -912,7 +993,7 @@ MGL_NO_EXPORT void *mgl_pulse_y(void *par)
 	}
 	return 0;
 }
-MGL_NO_EXPORT void *mgl_pulse_x(void *par)
+static void *mgl_pulse_x(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0], nn=t->n;
