@@ -35,7 +35,7 @@
 #endif
 #if MGL_HAVE_HDF4
 #define intf hdf4_intf
-#include <hdf/mfhdf.h>
+#include <mfhdf.h>
 #undef intf
 #endif
 
@@ -64,6 +64,7 @@ void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)
 	const std::string loc = setlocale(LC_NUMERIC, "C");
 	std::vector<char *> lines;
 	std::vector<std::vector<mreal> > numbs;
+	while(*buf && *buf<=' ')	buf++;
 	lines.push_back(buf);
 	for(char *s=buf; *s; s++)	if(isn(*s))
 	{	lines.push_back(s+1);	*s = 0;	s++;	}
@@ -80,8 +81,10 @@ void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)
 			if(j>=nb)	break;
 			if(b[j]=='#')
 			{
+				std::string id;
 				if(j<nb-1 && b[j+1]=='#')	for(long i=j+2;i<nb;i++)
-					if(b[i]>='a' && b[i]<='z')	d->id.push_back(b[i]);
+					if(b[i]>='a' && b[i]<='z')	id.push_back(b[i]);
+				d->SetColumnId(id.c_str());
 				break;
 			}
 			const char *s=b+j;
@@ -261,15 +264,18 @@ void MGL_EXPORT mgl_data_rearrange(HMDT d, long mx,long my,long mz)
 void MGL_EXPORT mgl_data_rearrange_(uintptr_t *d, int *mx, int *my, int *mz)
 {	mgl_data_rearrange(_DT_,*mx,*my,*mz);	}
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_data_set_id(HMDT d, const char *ids)	{	d->id = ids;	}
+MGL_EXPORT const char *mgl_data_get_id(HCDT d)	{	return d->id.s;	}
+void MGL_EXPORT mgl_data_set_id(mglDataA *d, const char *ids)	{	d->id = ids;	}
 void MGL_EXPORT mgl_data_set_id_(uintptr_t *d, const char *eq,int l)
 {	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
 	mgl_data_set_id(_DT_, s);	delete []s;	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_set_name_w(mglDataA *dat, const wchar_t *name)
 {	dat->s = name;	}
+MGL_EXPORT const wchar_t *mgl_data_get_name_w(HCDT dat)
+{	return dat->s.w;	}
 void MGL_EXPORT mgl_data_set_name(mglDataA *dat, const char *name)
-{	MGL_TO_WCS(name,dat->s = wcs);	}
+{	MGL_TO_WCS(name,dat->Name(wcs));	}
 void MGL_EXPORT mgl_data_set_name_(uintptr_t *d, const char *name,int l)
 {	char *s=new char[l+1];	memcpy(s,name,l);	s[l]=0;
 	mgl_data_set_name(_DT_,s);		delete []s;	}
@@ -325,8 +331,8 @@ std::string MGL_EXPORT mgl_data_to_string(HCDT d, long ns)
 	std::string out;
 	if(ns<0 || (ns>=nz && nz>1))	for(long k=0;k<nz;k++)
 	{	// save whole data
-		const mglData *dr = dynamic_cast<const mglData *>(d);
-		if(dr && !dr->id.empty())	out += "## "+dr->id+'\n';
+		std::string id = d->GetColumnId();
+		if(!id.empty())	out += "## "+id+'\n';
 		for(long i=0;i<ny;i++)
 		{
 			for(long j=0;j<nx-1;j++)	out += mgl_str_num(d->v(j,i,k))+'\t';
@@ -377,7 +383,8 @@ int MGL_EXPORT mgl_data_read(HMDT d, const char *fname)
 		if(!d->a)	mgl_data_create(d, 1,1,1);
 		return	0;
 	}
-	char *buf = mgl_read_gz(fp);
+	char *buf = mgl_read_gz(fp), *tbuf=buf;
+	while(*buf && *buf<=' ')	buf++;	// remove leading spaces
 	long nb = strlen(buf);	gzclose(fp);
 
 	bool first=false;
@@ -415,7 +422,7 @@ int MGL_EXPORT mgl_data_read(HMDT d, const char *fname)
 	}
 	else	for(i=0;i<nb-1;i++)	if(buf[i]=='\f')	l++;
 	mglFromStr(d,buf,k,m,l);
-	free(buf);	return 1;
+	free(tbuf);	return 1;
 }
 int MGL_EXPORT mgl_data_read_(uintptr_t *d, const char *fname,int l)
 {	char *s=new char[l+1];		memcpy(s,fname,l);	s[l]=0;
@@ -485,7 +492,7 @@ void MGL_EXPORT mgl_data_create(HMDT d,long mx,long my,long mz)
 	d->nx = mx>0 ? mx:1;	d->ny = my>0 ? my:1;	d->nz = mz>0 ? mz:1;
 	if(d->a && !d->link)	delete [](d->a);
 	d->a = new mreal[d->nx*d->ny*d->nz];
-	d->id.clear();	d->link=false;
+	d->NewId();	d->link=false;
 	memset(d->a,0,d->nx*d->ny*d->nz*sizeof(mreal));
 }
 void MGL_EXPORT mgl_data_create_(uintptr_t *d, int *nx,int *ny,int *nz)
@@ -982,7 +989,7 @@ void MGL_EXPORT mgl_data_transpose_(uintptr_t *d, const char *dim,int l)
 {	char *s=new char[l+1];	memcpy(s,dim,l);	s[l]=0;
 	mgl_data_transpose(_DT_,s);	delete []s;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_modify(void *par)
+static void *mgl_modify(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	const mglFormula *f = (const mglFormula *)(t->v);
@@ -1023,20 +1030,20 @@ void MGL_EXPORT mgl_data_modify_(uintptr_t *d, const char *eq,int *dim,int l)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_modify_vw(HMDT d, const char *eq,HCDT vdat,HCDT wdat)
 {
-	std::wstring s = d->s;	d->s = L"u";
-	mglDataV x(d->nx,d->ny,d->nz, 0,1,'x');	x.s=L"x";
-	mglDataV y(d->nx,d->ny,d->nz, 0,1,'y');	y.s=L"y";
-	mglDataV z(d->nx,d->ny,d->nz, 0,1,'z');	z.s=L"z";
-	mglDataV i(d->nx,d->ny,d->nz, 0,d->nx-1,'x');	i.s=L"i";
-	mglDataV j(d->nx,d->ny,d->nz, 0,d->ny-1,'y');	j.s=L"j";
-	mglDataV k(d->nx,d->ny,d->nz, 0,d->nz-1,'z');	k.s=L"k";
-	mglDataV r(d->nx,d->ny,d->nz);	r.s=L"#$mgl";
-	mglData v(vdat), w(wdat);	v.s = L"v";	w.s = L"w";
+	std::wstring s=d->Name();	d->Name(L"u");
+	mglDataV x(d->nx,d->ny,d->nz, 0,1,'x');	x.Name(L"x");
+	mglDataV y(d->nx,d->ny,d->nz, 0,1,'y');	y.Name(L"y");
+	mglDataV z(d->nx,d->ny,d->nz, 0,1,'z');	z.Name(L"z");
+	mglDataV i(d->nx,d->ny,d->nz, 0,d->nx-1,'x');	i.Name(L"i");
+	mglDataV j(d->nx,d->ny,d->nz, 0,d->ny-1,'y');	j.Name(L"j");
+	mglDataV k(d->nx,d->ny,d->nz, 0,d->nz-1,'z');	k.Name(L"k");
+	mglDataV r(d->nx,d->ny,d->nz);	r.Name(L"#$mgl");
+	mglData v(vdat), w(wdat);	v.Name(L"v");	w.Name(L"w");
 	std::vector<mglDataA*> list;
 	list.push_back(&x);	list.push_back(&y);	list.push_back(&z);	list.push_back(d);
 	list.push_back(&v);	list.push_back(&w);	list.push_back(&r);
 	list.push_back(&i);	list.push_back(&j);	list.push_back(&k);
-	d->Move(mglFormulaCalc(eq,list));	d->s = s;
+	d->Move(mglFormulaCalc(eq,list));	d->Name(s.c_str());
 }
 void MGL_EXPORT mgl_data_modify_vw_(uintptr_t *d, const char *eq, uintptr_t *v, uintptr_t *w,int l)
 {	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
@@ -1311,24 +1318,18 @@ int MGL_EXPORT mgl_data_read_all_(uintptr_t *d, const char *fname, int *as_slice
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_data_column(HCDT dat, const char *eq)
 {
-	const mglData *dd=dynamic_cast<const mglData *>(dat);
 	std::vector<mglDataA*> list;
-	if(dd && dd->id.length()>0)	for(size_t i=0;i<dd->id.length();i++)
+	const char *id = dat->GetColumnId();
+	size_t len = strlen(id);
+	for(size_t i=0;i<len;i++)
 	{
 		mglDataT *col = new mglDataT(*dat);
-		col->SetInd(i,dd->id[i]);
-		list.push_back(col);
-	}
-	const mglDataC *dc=dynamic_cast<const mglDataC *>(dat);
-	if(dc && dc->id.length()>0)	for(size_t i=0;i<dc->id.length();i++)
-	{
-		mglDataT *col = new mglDataT(*dat);
-		col->SetInd(i,dc->id[i]);
+		col->SetInd(i,id[i]);
 		list.push_back(col);
 	}
 	if(list.size()==0)	return 0;	// no named columns
 	mglDataV *t = new mglDataV(dat->GetNy(),dat->GetNz());
-	t->s=L"#$mgl";	list.push_back(t);
+	t->Name(L"#$mgl");	list.push_back(t);
 	HMDT r = mglFormulaCalc(eq,list);
 	for(size_t i=0;i<list.size();i++)	delete list[i];
 	return r;
@@ -1342,13 +1343,37 @@ void MGL_EXPORT mgl_data_limit(HMDT d, mreal v)
 {
 	long n = d->GetNN();
 	mreal *a = d->a;
-	#pragma omp parallel for
+#pragma omp parallel for
 	for(long i=0;i<n;i++)
 	{	mreal b = fabs(a[i]);	if(b>v)	a[i] *= v/b;	}
 }
 void MGL_EXPORT mgl_data_limit_(uintptr_t *d, mreal *v)
 {	mgl_data_limit(_DT_, *v);	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_coil(HMDT d, mreal v1, mreal v2, int sep)
+{
+	long n = d->GetNN();
+	if(mgl_isnan(v2))	v2=-v1;
+	if(v2<v1)	{	mreal tmp=v1;	v1=v2;	v2=tmp;	}
+	mreal *a = d->a, dv = v2-v1;
+	if(dv==0)	return;
+	long *kk=new long[n];
+#pragma omp parallel for
+	for(long i=0;i<n;i++)
+	{
+		kk[i] = mgl_int((a[i]-v1)/dv-0.5);
+		a[i] -= kk[i]*dv;
+	}
+	if(sep)
+	{
+#pragma omp parallel for
+		for(long i=1;i<n;i++)	if(kk[i]!=kk[i-1])	a[i] = NAN;
+	}
+	delete []kk;
+}
+void MGL_EXPORT mgl_data_coil_(uintptr_t *d, mreal *v1, mreal *v2, int *sep)
+{	mgl_data_coil(_DT_, *v1, *v2, *sep);	}
+//-----------------------------------------------------------------------------
 /// Read binary data and swap big-endian to little-endian if swap=true
 size_t MGL_EXPORT mgl_fread(FILE *fp, void *vals, size_t size, size_t num, int swap)
 {
@@ -1391,7 +1416,7 @@ int MGL_EXPORT mgl_data_read_wfm(HMDT d,const char *fname, long num, long step/*
 	unsigned short byte_order;
 	fread(&byte_order,2,1,fp);
 	bool byteorder;	// TODO
-*/	return 0;	
+*/	return 0;
 }
 int MGL_EXPORT mgl_data_read_wfm_(uintptr_t *d, const char *fname, long *num, long *step, long *start,int l)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
