@@ -315,7 +315,7 @@ void MGL_EXPORT mgl_wcstombs(char *dst, const wchar_t *src, int size)
 MGL_LOCAL_PURE const mglDataA *FindVar(const std::vector<mglDataA*> &head, const std::wstring &name)
 {
 	for(size_t i=0;i<head.size();i++)
-		if(head[i] && head[i]->s==name)	return head[i];
+		if(head[i] && head[i]->Name()==name)	return head[i];
 	return 0;
 }
 //-----------------------------------------------------------------------------
@@ -418,9 +418,11 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 	n=mglFindInText(str,"+-");	// normal priority -- additions
 	if(n>=0 && (n<2 || str[n-1]!='e' || (str[n-2]!='.' && !isdigit(str[n-2])) ))
 		return str[n]=='+'? mglApplyOperAdd(str.substr(0,n),str.substr(n+1),arg, head) : mglApplyOperSub(str.substr(0,n),str.substr(n+1),arg, head);
-	n=mglFindInText(str,"*/");	// high priority -- multiplications
+	n=mglFindInText(str,"*/%");	// high priority -- multiplications
 	if(n>=0)
-		return str[n]=='*'? mglApplyOperMul(str.substr(0,n),str.substr(n+1),arg, head) : mglApplyOperDiv(str.substr(0,n),str.substr(n+1),arg, head);
+		return str[n]=='*'? mglApplyOperMul(str.substr(0,n),str.substr(n+1),arg, head) : 
+			(str[n]=='/'? mglApplyOperDiv(str.substr(0,n),str.substr(n+1),arg, head) :
+				mglApplyOper(str.substr(0,n),str.substr(n+1),arg, head, fmod));
 	n=mglFindInText(str,"@");	// high priority -- combine
 	if(n>=0)
 	{
@@ -428,7 +430,7 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		HMDT a2 = mglFormulaCalc(str.substr(n+1),arg, head);
 		HMDT res = mgl_data_combine(a1,a2);
 		mgl_delete_data(a1);	mgl_delete_data(a2);
-		return res;
+		return res?res:new mglData;
 	}
 	n=mglFindInText(str,"^");	// highest priority -- power
 	if(n>=0)	return mglApplyOper(str.substr(0,n),str.substr(n+1),arg, head, ipw);
@@ -510,6 +512,11 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		if(mgl_isfin(v))
 		{	HMDT res = new mglData;	res->a[0]=v;	return res;	}
 	}
+	if(str[0]=='`')
+	{
+		HMDT res = mglFormulaCalc(str.substr(1), arg, head);
+		res->Transpose();	return res;
+	}
 	for(n=0;n<len;n++)	if(str[n]=='(')	break;
 	if(n>=len)		// this is number or variable
 	{
@@ -541,8 +548,16 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		std::wstring nm = str.substr(0,n);
 		str = str.substr(n+1,len-n-2);	len -= n+2;
 		HCDT v = FindVar(head, nm);
+		HMDT tmp = 0;
 //		mglVar *v = arg->FindVar(nm.c_str());
 		if(!v && !nm.compare(0,7,L"jacobi_"))	nm = nm.substr(7);
+		if(!v && nm.empty())	
+		{
+			long m=mglFindInText(str,")");
+			if(m>1)
+			{	nm = str.substr(0,m);	str = str.substr(m+2);
+				tmp = mglFormulaCalc(nm,arg,head);	v = tmp;	}
+		}
 		n = mglFindInText(str,",");
 		if(v)	// subdata
 		{
@@ -551,7 +566,8 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 				char *buf = new char[len];
 				mgl_wcstombs(buf, str.substr(1).c_str(), len-1);	buf[len-1]=0;
 				HMDT res = mgl_data_column(v,buf);
-				delete []buf;	return res;
+				if(tmp)	mgl_delete_data(tmp);
+				delete []buf;	return res?res:new mglData;
 			}
 			else
 			{
@@ -574,9 +590,11 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 				}
 				else	a1 = mglFormulaCalc(str, arg, head);
 				HMDT res = mgl_data_subdata_ext(v,a1,a2,a3);
+				if(tmp)	mgl_delete_data(tmp);
 				mgl_delete_data(a1);	mgl_delete_data(a2);
-				mgl_delete_data(a3);	return res;
+				mgl_delete_data(a3);	return res?res:new mglData;
 			}
+			if(tmp)	mgl_delete_data(tmp);
 		}
 		else if(nm[0]=='a')	// function
 		{
@@ -806,7 +824,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 	mreal tval = mgl_gettime(str);
 	if(mgl_isnum(tval))
 	{	mglDataC *r=new mglDataC;	r->a[0] = tval;	return r;	}
-	
+
 	long n,len=str.length();
 	if(str[0]=='(' && mglCheck(str.substr(1,len-2)))	// remove braces
 	{	str = str.substr(1,len-2);	len-=2;	}
@@ -869,7 +887,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 		HADT a2 = mglFormulaCalcC(str.substr(n+1),arg, head);
 		HADT res = mgl_datac_combine(a1,a2);
 		mgl_delete_datac(a1);	mgl_delete_datac(a2);
-		return res;
+		return res?res:new mglDataC;
 	}
 	n=mglFindInText(str,"^");				// highest priority -- power
 	if(n>=0)
@@ -944,6 +962,11 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 		if(mgl_isfin(v))
 		{	HADT res = new mglDataC;	res->a[0]=v;	return res;	}
 	}
+	if(str[0]=='`')
+	{
+		HADT res = mglFormulaCalcC(str.substr(1), arg, head);
+		res->Transpose();	return res;
+	}
 	for(n=0;n<len;n++)	if(str[n]=='(')	break;
 	if(n>=len)		// this is number or variable
 	{
@@ -966,7 +989,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			if(ch<':')	// this is real number
 				res->a[0] = (str[str.length()-1]=='i') ? dual(0,wcstod(str.c_str(),0)) :  mreal(wcstod(str.c_str(),0));
 			else if(ch=='i')	// this is imaginary number
-				res->a[0] = dual(0,str[1]>' '?wcstod(str.c_str()+1,0):1);
+				res->a[0] = dual(0,(str[1]>='0' && str[1]<='9')?wcstod(str.c_str()+1,0):1);
 			else if(!str.compare(L"pi"))	res->a[0] = M_PI;
 			else if(ch==':')	res->a[0] = -1;
 			else if(!str.compare(L"nan"))	res->a[0] = NAN;
@@ -979,8 +1002,16 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 		std::wstring nm = str.substr(0,n);
 		str = str.substr(n+1,len-n-2);	len -= n+2;
 		HCDT v = FindVar(head, nm);
+		HADT tmp = 0;
 //		mglVar *v = arg->FindVar(nm.c_str());
 		if(!v && !nm.compare(0,7,L"jacobi_"))	nm = nm.substr(7);
+		if(!v && nm.empty())	
+		{
+			long m=mglFindInText(nm,")");
+			if(m>1)
+			{	nm = str.substr(0,m);	str = str.substr(m+2);
+				tmp = mglFormulaCalcC(nm,arg,head);	v = tmp;	}
+		}
 		n = mglFindInText(str,",");
 		if(v)	// subdata
 		{
@@ -989,7 +1020,8 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 				char *buf = new char[len];
 				mgl_wcstombs(buf, str.substr(1).c_str(), len-1);	buf[len-1]=0;
 				HADT res = mgl_datac_column(v,buf);
-				delete []buf;	return res;
+				if(tmp)	mgl_delete_datac(tmp);
+				delete []buf;	return res?res:new mglDataC;
 			}
 			else
 			{
@@ -1012,9 +1044,11 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 				}
 				else	a1 = mglFormulaCalc(str, arg, head);
 				HADT res = mgl_datac_subdata_ext(v,a1,a2,a3);
+				if(tmp)	mgl_delete_datac(tmp);
 				mgl_delete_data(a1);	mgl_delete_data(a2);
-				mgl_delete_data(a3);	return res;
+				mgl_delete_data(a3);	return res?res:new mglDataC;
 			}
+				if(tmp)	mgl_delete_datac(tmp);
 		}
 		else if(nm[0]=='a')	// function
 		{
