@@ -20,7 +20,12 @@
 #include <locale.h>
 #include <ctype.h>
 #include <wctype.h>
+#ifdef WIN32
+#include <io.h>
+#include <direct.h>
+#else
 #include <unistd.h>
+#endif
 
 #if !defined(__BORLANDC__) || (__CODEGEARC__ >=  0x0630)
 #include <algorithm>
@@ -263,8 +268,8 @@ unsigned mglFont::Parse(const wchar_t *s) const
 	else if(!wcscmp(s,L"underset"))	res = unsigned(-9);
 	else if(!wcscmp(s,L"stackr"))	res = unsigned(-10);
 	else if(!wcscmp(s,L"stackl"))	res = unsigned(-11);
-	else if(!wcscmp(s,L"sub"))		res = unsigned(-12);
-	else if(!wcscmp(s,L"sup"))		res = unsigned(-13);
+	else if(!wcscmp(s,L"sub"))		res = unsigned(-9);	//unsigned(-12);
+	else if(!wcscmp(s,L"sup"))		res = unsigned(-8);	//unsigned(-13);
 	else if(!wcscmp(s,L"textsc"))	res = unsigned(-14);	// new
 	else if(!wcscmp(s,L"dfrac"))	res = unsigned(-15);
 	else if(!wcscmp(s,L"b"))		res = MGL_FONT_BOLD;
@@ -410,25 +415,25 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,floa
 			Puts(b2, x+(ww-w2)/2, yy-660*ff/fact[a], ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
 			MGL_CLEAR_STYLE
 		}
-		else if(s==unsigned(-9))	// underset
+		else if(s==unsigned(-9))	// underset or sub
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
-			Puts(b2, x+(ww-w2)/2, yy-150*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
+			Puts(b2, x+(ww-w2)/2, yy-175*ff/fact[a], ff/3, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
 				draw_ouline(st,x,y,f,fact[a],ww,ccol);
 			MGL_CLEAR_STYLE
 		}
-		else if(s==unsigned(-8))	// overset
+		else if(s==unsigned(-8))	// overset or sup
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
-			Puts(b2, x+(ww-w2)/2, yy+375*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
+			Puts(b2, x+(ww-w2)/2, yy+400*ff/fact[a], ff/3, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
 				draw_ouline(st,x,y,f,fact[a],ww,ccol);
 			MGL_CLEAR_STYLE
 		}
-		else if(s==unsigned(-12))	// sub
+/*		else if(s==unsigned(-12))	// sub	// NOTE: unused because is the same as \underset now
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
@@ -436,8 +441,8 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,floa
 			if(gr && !(style&0x10))	// add under-/over- line now
 				draw_ouline(st,x,y,f,fact[a],ww,ccol);
 			MGL_CLEAR_STYLE
-		}
-		else if(s==unsigned(-13))	// sup
+		}*/
+/*		else if(s==unsigned(-13))	// sup	// NOTE: unused because is the same as \overset now
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol,ccol);
@@ -445,7 +450,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,floa
 			if(gr && !(style&0x10))	// add under-/over- line now
 				draw_ouline(st,x,y,f,fact[a],ww,ccol);
 			MGL_CLEAR_STYLE
-		}
+		}*/
 		else if(s==unsigned(-11))	// stackl
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff*0.45, ff*0.45, st);
@@ -557,9 +562,9 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,floa
 			ss = (s/MGL_FONT_UPPER)%4;
 			if(ss)
 			{
-				if(ss==1)		{	ff *=0.6;	yy += 200*ff/fact[a];	}	// =  500*0.4
-				else if(ss==2)	{	ff *=0.6;	yy -=  80*ff/fact[a];	}	// = -500*0.16
-				else if(ss==3)	{	ff *=0.8;	yy +=  0*60*ff/fact[a];	}	// =  500*0.12
+				if(ss==1)		{	ff *=0.6;	yy += 250*ff/fact[a];	}	// =  500*0.4
+				else if(ss==2)	{	ff *=0.6;	yy -=  130*ff/fact[a];	}	// = -500*0.16
+				else if(ss==3)	{	ff *=0.6;	yy +=  60*ff/fact[a];	}	// =  500*0.12
 			}
 		}
 		x += ww;	w += ww;
@@ -737,10 +742,10 @@ bool mglFont::LoadBin(const char *base, const char *path)
 //-----------------------------------------------------------------------------
 bool mglFont::Load(const char *base, const char *path)
 {
-//	base = 0;
-	char *buf=0,sep='/';
 #ifdef WIN32
-	sep='\\';
+	char *buf=0, sep='\\';
+#else
+	char *buf=0, sep='/';
 #endif
 	char str[256];
 	std::string loc = setlocale(LC_NUMERIC,"C");
@@ -887,7 +892,7 @@ pthread_mutex_t mutexRnd;
 #endif
 //-----------------------------------------------------------------------------
 float mgl_cos[360];
-void MGL_NO_EXPORT mgl_init()
+void static mgl_init()
 {
 	mgl_textdomain(NULL,"");
 #if MGL_HAVE_PTHREAD
@@ -926,7 +931,7 @@ void mglFont::Copy(mglFont *f)
 #pragma omp critical(font)
 	{	if(Buf)	delete []Buf;	Buf=0;	}
 	// copy scale factors
-	memcpy(fact,f->fact,4*sizeof(float));
+	fact[0]=f->fact[0];	fact[1]=f->fact[1];	fact[2]=f->fact[2];	fact[3]=f->fact[3];
 	// copy symbols descriptions
 	numb = f->numb;	Buf = new short[numb];	memcpy(Buf, f->Buf, numb*sizeof(short));
 	// copy symbol parameters
@@ -950,7 +955,7 @@ long MGL_EXPORT mgl_check_tex_table()
 	return res;
 }
 //---------------------------------------------------------------------------
-bool MGL_NO_EXPORT test_transl(const char *p)
+bool static test_transl(const char *p)
 {
 	if(!p)	return false;
 #if MGL_USE_GETTEXT
