@@ -22,6 +22,7 @@
 #include "mgl2/eval.h"
 #include "mgl2/thread.h"
 #include "mgl2/base.h"
+#include "interp.hpp"
 const double GAMMA=0.1;	///< value for damping
 HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
@@ -29,7 +30,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*>
 //		Advanced PDE series in 2D case
 //
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_operator_exp(long n, dual *h, dual *a, dual *f)
+void static mgl_operator_exp(long n, const dual *h, dual *a, dual *f)
 {
 	memset(f,0,2*n*sizeof(dual));
 	const long i1=n/2, i2=3*n/2-1;
@@ -74,7 +75,7 @@ void MGL_NO_EXPORT mgl_operator_exp(long n, dual *h, dual *a, dual *f)
 	}
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g, dual *o)
+void static mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g, dual *o, const dual *iexp)
 {
 	memset(f,0,2*n*sizeof(dual));
 	memset(g,0,2*n*sizeof(dual));
@@ -85,10 +86,10 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g,
 		long jp = (j+1)%n;
 		mreal h1=tanh(h[n*j]), g1=(h1+tanh(h[n*jp]))/2;
 		mreal h2=tanh(h[n-1+n*j]), g2=(h2+tanh(h[n-1+n*jp]))/2;
-		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
+		const dual *ie1=iexp+4*n*j, *ie2=iexp+2*n*(2*j+1);
 		for(long i=0;i<i1;i++)
 		{
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			dual e1=ie1[i], e2=ie2[i];
 			f[2*j] += a[i]*h1*e1;	f[2*j+1] += a[i]*g1*e2;
 			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
 		}
@@ -96,13 +97,13 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g,
 		{
 			mreal hh = tanh(h[i-i1+n*j]);
 			mreal gg = (hh+tanh(h[i-i1+n*jp]))/2;
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			dual e1=ie1[i], e2=ie2[i];
 			f[2*j] += a[i]*hh*e1;	f[2*j+1] += a[i]*gg*e2;
 			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
 		}
 		for(long i=i2;i<2*n;i++)
 		{
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
+			dual e1=ie1[i], e2=ie2[i];
 			f[2*j] += a[i]*h2*e1;	f[2*j+1] += a[i]*g2*e2;
 			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
 		}
@@ -114,19 +115,20 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g,
 		long ii=i-i1;
 		if(ii<0)	ii=0;
 		if(ii>n-1)	ii=n-1;
-		double kk=M_PI*2*i/n;
+		const dual *ie1=iexp+2*n*i;
+//		double kk=M_PI*2*i/n;
 		for(long j=0;j<n;j++)
 		{
 			mreal h1 = tanh(h[ii+n*j]);
 			mreal g1 = (h1+tanh(h[ii+n*((j+1)%n)]))/2;
-			dual e1=exp(dual(0,-j*kk)), e2=exp(dual(0,-kk*(j+0.5)));
+			dual e1=conj(ie1[2*j]), e2=conj(ie1[2*j+1]);
 			o[i] += f[2*j]*e1 + f[2*j+1]*e2;
 			o[i] += g[2*j]*h1*e1 + g[2*j+1]*g1*e2;
 		}
 	}
 }
 //-----------------------------------------------------------------------------
-HADT MGL_NO_EXPORT mgl_apde_calc_ham(HMDT hs, bool old, const char *func, std::vector<mglDataA*> list, const mreal dd)
+HADT static mgl_apde_calc_ham(HMDT hs, bool old, const char *func, std::vector<mglDataA*> list, const mreal dd)
 {
 	HADT ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);
 	const long nx = ham->nx;
@@ -178,11 +180,11 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 
 	mglDataC *res=new mglDataC(nx, nt);
 	mglData hIm(nx,nx);	// for advanced damping calculation
-	mglDataC u(nx);	u.s = L"u";
+	mglDataC u(nx);	u.Name(L"u");
 	mglDataV x(nx,nx), y(nx,nx), r(nx,nx);
-	mglDataW p(nx,nx);	p.s = L"p";
+	mglDataW p(nx,nx);	p.Name(L"p");
 	bool old = func[0]==';';	if(old)	func=func+1;
-	x.s = L"x";	y.s = L"y";	r.s=L"#$mgl";
+	x.Name(L"x");	y.Name(L"y");	r.Name(L"#$mgl");
 	const mreal dp = 2*M_PI/(Max.x-Min.x), dd = k0*dt/2;
 	x.Fill(Min.x,Max.x,'x');	p.Freq(dp/k0,'y');
 	std::vector<mglDataA*> list;
@@ -203,6 +205,10 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 	bool have_y = mglchr(func,'y');
 	HADT ham;
 	if(!have_y)		ham = mgl_apde_calc_ham(&hIm, old, func, list, dd);
+	dual *iexp = new dual[4*nx*nx];
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<2*nx;j++)	for(long i=0;i<2*nx;i++)
+		iexp[i+2*nx*j] = exp(dual(0,(M_PI*i*j)/nx));
 	for(long k=0;k<nt;k++)
 	{
 		memcpy(u.a,a+nx/2,nx*sizeof(dual));
@@ -210,14 +216,14 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 		if(have_y)
 		{	y.Fill(k*dt);	ham = mgl_apde_calc_ham(&hIm, old, func, list, dd);	}
 		mgl_operator_exp(nx,ham->a,a,f);
-		mgl_operator_lin(nx,hIm.a,a,f,g,s);
-		mgl_operator_lin(nx,hIm.a,s,f,g,s);
+		mgl_operator_lin(nx,hIm.a,a,f,g,s,iexp);
+		mgl_operator_lin(nx,hIm.a,s,f,g,s,iexp);
 #pragma omp parallel for
 		for(long i=0;i<2*nx;i++)
 			a[i] = (a[i]-s[i]/mreal(8*nx*nx))*mreal(exp(-dmp[i]*dt)/2/nx);
 		if(have_y)	delete ham;
 	}
-	delete []a;	delete []f;	delete []dmp;
+	delete []a;	delete []f;	delete []dmp;	delete []iexp;
 	if(!have_y)	delete ham;
 	gr->LoadState();	return res;
 }
@@ -240,55 +246,6 @@ uintptr_t MGL_EXPORT mgl_pde_adv_(uintptr_t* gr, const char *ham, uintptr_t* ini
 	uintptr_t res = uintptr_t(mgl_pde_adv(_GR_, s, _DA_(ini_re), _DA_(ini_im), *dz, *k0, o));
 	delete []o;	delete []s;	return res;	}
 //-----------------------------------------------------------------------------
-/*HADT MGL_EXPORT mgl_pde_adv_3d_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im, mreal dz, mreal k0, const char *opt)
-{
-	mreal gamma = gr->SaveState(opt);	if(mgl_isnan(gamma))	gamma = GAMMA;
-	mglPoint Min=gr->Min, Max=gr->Max;
-	long nx=ini_re->GetNx(), ny=ini_re->GetNy(), nz = long((Max.z-Min.z)/dz)+1;
-	if(nx<2 || nz<2 || Max.x==Min.x)			// Too small data
-	{	gr->SetWarn(mglWarnLow,"PDE");	return 0;	}
-	if(ini_im->GetNx()*ini_im->GetNy() != nx*ny)// Wrong dimensions
-	{	gr->SetWarn(mglWarnDim,"PDE");	return 0;	}
-	mglDataC *res=new mglDataC(nz, nx, ny);
-
-	mglDataV x(nx,ny), y(nx,ny), z, r(nx,ny);
-	mglDataW p(nx,ny), q(nx,ny);
-	x.s = L"x";	y.s = L"y";	p.s = L"p";	q.s = L"q";	z.s = L"z";	r.s=L"#$mgl";
-	//z.Fill(f->zz);
-	mreal dx = (Max.x-Min.x)/(nx-1), dy = ny>1?(Max.y-Min.y)/(ny-1):0;
-	mreal dp = M_PI/(Max.x-Min.x), dq = M_PI/(Max.y-Min.y);
-	double dd = k0*dz;
-	x.Fill(Min.x,Max.x,'x');	p.Freq(dp/k0,'x');
-	y.Fill(Min.y,Max.y,'y');	q.Freq(dq/k0,'y');
-
-	ddual *a = new ddual[4*nx*ny];	// Add "damping" area
-	ddual *f = new ddual[4*nx*ny];	// Effective "spectrum"
-	memset(a,0,4*nx*ny*sizeof(ddual));
-	memset(f,0,4*nx*ny*sizeof(ddual));
-#pragma omp parallel for collapse(2)
-	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	// Initial conditions
-	{
-		long i0 = i+nx/2+2*nx*(j+ny/2);
-		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
-		res->a[nz*(i+nx*j)] = a[i0];
-	}
-	double *dmp = new double[4*nx*ny];
-	memset(dmp,0,4*nx*ny*sizeof(double));
-#pragma omp parallel for collapse(2)
-	for(long j=0;j<2*ny;j++)	for(long i=0;i<2*nx;i++)	// step 1
-	{
-		long i0 = i+2*nx*j;
-		if(i<nx/2)		dmp[i0] += gamma*mgl_ipow((nx/2-i)/(nx/2.),2);
-		if(i>3*nx/2)	dmp[i0] += gamma*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
-		if(j<ny/2)		dmp[i0] += gamma*mgl_ipow((ny/2-j)/(ny/2.),2);
-		if(j>3*ny/2)	dmp[i0] += gamma*mgl_ipow((j-3*ny/2-1)/(ny/2.),2);
-	}
-	ddual *Hdat = new ddual[nx*nx*ny*ny];
-	for(long k=1;k<nz;k++)
-	{
-	}
-}*/
-//-----------------------------------------------------------------------------
 //
 //		Simplified PDE series
 //
@@ -301,15 +258,15 @@ struct mgl_pde_ham
 	double xx,yy,xs,ys,dx,dy,dq,dp,zz;
 	double dd;
 };
-void MGL_NO_EXPORT mgl_pde_hprep(const mgl_pde_ham *f)
+void static mgl_pde_hprep(const mgl_pde_ham *f)
 {
 	const long nx = f->nx, ny = f->ny;
 	mglDataV x(nx,ny), y(nx,ny), z, r(nx,ny);
 	mglDataW p(nx,ny), q(nx,ny);
-	x.s = L"x";	y.s = L"y";	p.s = L"p";	q.s = L"q";	r.s=L"#$mgl";
-	z.s = L"z";	z.Fill(f->zz);
+	x.Name(L"x");	y.Name(L"y");	p.Name(L"p");	q.Name(L"q");	r.Name(L"#$mgl");
+	z.Name(L"z");	z.Fill(f->zz);
 	dual dd(0,f->dd);
-	mglData u(nx,ny);	u.s = L"u";
+	mglData u(nx,ny);	u.Name(L"u");
 #pragma omp parallel for
 	for(long i=0;i<nx*ny;i++)	u.a[i] = abs(f->a[i]);
 	std::vector<mglDataA*> list;
@@ -418,13 +375,13 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 		else
 #pragma omp parallel for
 			for(long i=0;i<4*nx*ny;i++)	huv[i] -= hh0;
-			// solve equation
+		// solve equation
 		if(ny>1)
 #pragma omp parallel
 		{
 			void *wsx = mgl_fft_alloc_thr(2*nx), *wsy = mgl_fft_alloc_thr(2*ny);
 #pragma omp for
-			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i])*exp(-double(dmp[i]*dz));
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i]-double(dmp[i]*dz));
 #pragma omp for
 			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
 #pragma omp for
@@ -446,7 +403,7 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 		{
 			void *wsx = mgl_fft_alloc_thr(2*nx);
 #pragma omp for
-			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i])*exp(-double(dmp[i]*dz));
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i]-double(dmp[i]*dz));
 #pragma omp for
 			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
 #pragma omp for
@@ -489,79 +446,59 @@ uintptr_t MGL_EXPORT mgl_pde_solve_(uintptr_t* gr, const char *ham, uintptr_t* i
 //		ODE series
 //
 //-----------------------------------------------------------------------------
-struct mglOdeTxt	{	long n;	HMEX *eq;	const char *var;	};
 void MGL_NO_EXPORT mgl_txt_func(const mreal *x, mreal *dx, void *par)
 {
-	mglOdeTxt *p=(mglOdeTxt *)par;
+	mglEqTxT *p=(mglEqTxT *)par;
 	mreal vars[MGL_VS];
-	for(long i=0;i<p->n;i++)
+	size_t n = p->str.size();
+	for(size_t i=0;i<n;i++)
 	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=x[i];	}
 #pragma omp parallel for
-	for(long i=0;i<p->n;i++)
-		dx[i] = mgl_expr_eval_v(p->eq[i], vars);
+	for(long i=0;i<long(n);i++)
+		dx[i] = mgl_expr_eval_v(p->eqR[i], vars);
 }
 HMDT MGL_EXPORT mgl_ode_solve_str(const char *func, const char *var, HCDT x0, mreal dt, mreal tmax)
 {
 	if(!var || !(*var) || !func)	return 0;
-	long len = strlen(func);
-	mglOdeTxt par;	par.var=var;
-	par.n = strlen(var);
-	par.eq = new HMEX[par.n];
-	char *buf = new char[len+1], *f=buf, *g=f;	memcpy(buf,func,len+1);
-	mreal *xx = new mreal[par.n];
-	for(long i=0;i<par.n;i++)
-	{
-		xx[i] = x0?x0->vthr(i):0;
-		for(long k=0;f[k];k++)	if(f[k]==';')
-		{ g = f+k+1;	f[k]=0;	break;	}
-		if(f==g)	g = f+strlen(f);
-		par.eq[i] = mgl_create_expr(f);
-		f = g;
-	}
-	HMDT res = mgl_ode_solve_ex(mgl_txt_func,par.n,xx,dt,tmax,&par,NULL);
-	for(long i=0;i<par.n;i++)	mgl_delete_expr(par.eq[i]);
-	delete []par.eq;	delete []buf;	delete []xx;
-	return res;
+	mglEqTxT par;
+	par.var=var;	par.FillReal(func);
+	size_t n = par.str.size();
+	mreal *xx = new mreal[n];
+	for(size_t i=0;i<n;i++)	xx[i] = x0?x0->vthr(i):0;
+	HMDT res = mgl_ode_solve_ex(mgl_txt_func,n,xx,dt,tmax,&par,NULL);
+	delete []xx;	return res;
 }
 //-----------------------------------------------------------------------------
-struct mglOdeTxtC	{	long n;	HAEX *eq;	const char *var;	};
 void MGL_NO_EXPORT mgl_txt_funcC(const mreal *x, mreal *dx, void *par)
 {
-	mglOdeTxtC *p=(mglOdeTxtC *)par;
+	mglEqTxT *p=(mglEqTxT *)par;
 	dual vars[MGL_VS];
-	for(long i=0;i<p->n;i++)
+	size_t n = p->str.size();
+	for(size_t i=0;i<n;i++)
 	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=dual(x[2*i],x[2*i+1]);	}
 #pragma omp parallel for
-	for(long i=0;i<p->n;i++)
+	for(long i=0;i<long(n);i++)
 	{
-		dual r = mgl_cexpr_eval_v(p->eq[i], vars);
+		dual r = mgl_cexpr_eval_v(p->eqC[i], vars);
 		dx[2*i] = real(r);	dx[2*i+1] = imag(r);
 	}
 }
 HADT MGL_EXPORT mgl_ode_solve_str_c(const char *func, const char *var, HCDT x0, mreal dt, mreal tmax)
 {
 	if(!var || !(*var) || !func)	return 0;
-	long len = strlen(func);
-	mglOdeTxtC par;	par.var=var;
-	par.n = strlen(var);
-	par.eq = new HAEX[par.n];
-	char *buf = new char[len+1], *f=buf, *g=f;	memcpy(buf,func,len+1);
-	mreal *xx = new mreal[2*par.n];
+	mglEqTxT par;	par.var=var;
+	par.var=var;	par.FillCmplx(func);
+	size_t n = par.str.size();
+	mreal *xx = new mreal[2*n];
 	const mglDataC *c = dynamic_cast<const mglDataC *>(x0);
-	for(long i=0;i<par.n;i++)
+	for(size_t i=0;i<n;i++)
 	{
 		if(c)	{	xx[2*i]=real(c->a[i]);	xx[2*i+1]=imag(c->a[i]);	}
 		else	{	xx[2*i] = x0?x0->vthr(i):0;	xx[2*i+1]=0;	}
-		for(long k=0;f[k];k++)	if(f[k]==';')
-		{ g = f+k+1;	f[k]=0;	break;	}
-		if(f==g)	g = f+strlen(f);
-		par.eq[i] = mgl_create_cexpr(f);
-		f = g;
 	}
-	HMDT res = mgl_ode_solve_ex(mgl_txt_funcC,2*par.n,xx,dt,tmax,&par,NULL);
-	for(long i=0;i<par.n;i++)	mgl_delete_cexpr(par.eq[i]);
-	delete []par.eq;	delete []buf;	delete []xx;
-	const long nn=par.n,nt=res->ny;
+	HMDT res = mgl_ode_solve_ex(mgl_txt_funcC,2*n,xx,dt,tmax,&par,NULL);
+	delete []xx;
+	const long nn=n, nt=res->ny;
 	mglDataC *out = new mglDataC(nn, nt);
 #pragma omp parallel for
 	for(long i=0;i<nt*nn;i++)	out->a[i] = dual(res->a[2*i],res->a[2*i+1]);
@@ -579,7 +516,9 @@ HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *p
 	// initial conditions
 	for(long i=0;i<n;i++)	x[i] = res->a[i] = x0[i];
 	// Runge Kutta scheme of 4th order
-	for(long k=1;k<nt;k++)
+	bool good=true;
+	long k;
+	for(k=1;k<nt && good;k++)
 	{
 		func(x,k1,par);
 		for(long i=0;i<n;i++)	v[i] = x[i]+k1[i]*hh;
@@ -590,9 +529,11 @@ HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *p
 		func(v,k2,par);
 		for(long i=0;i<n;i++)	x[i] += (k1[i]+k2[i]+2*k3[i])*dt/6;
 		if(bord)	bord(x,res->a+n*(k-1),par);
-		for(long i=0;i<n;i++)	res->a[i+n*k] = x[i];
+		for(long i=0;i<n;i++)
+		{	res->a[i+n*k] = x[i];	if(mgl_isbad(x[i]))	good=false;	}
 	}
 	delete []x;	delete []k1;	delete []k2;	delete []k3;	delete []v;
+	res->Crop(0,k,'y');
 	return res;
 }
 //-----------------------------------------------------------------------------
@@ -600,7 +541,7 @@ HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *p
 //		Common functions for quasioptical calculations
 //
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_ray3d(const mreal *in, mreal *out, void *par)
+void static mgl_ray3d(const mreal *in, mreal *out, void *par)
 {
 	mglFormula *eqs = (mglFormula *)par;
 	const char *v="xyzpqvt";
@@ -633,7 +574,7 @@ struct mgl_ap
 	mgl_ap()	{	memset(this,0,sizeof(mgl_ap));	}
 };
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_init_ra(long n, int n7, const mreal *r, mgl_ap *ra)	// prepare some intermediate data for QO (3d case)
+void static mgl_init_ra(long n, int n7, const mreal *r, mgl_ap *ra)	// prepare some intermediate data for QO (3d case)
 {
 	double tt = hypot(r[n7]-r[0], r[n7+1]-r[1]);
 	if(tt)
@@ -699,7 +640,7 @@ struct mgl_qo2d_ham
 	void *par;
 };
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_qo2d_hprep(void *par)
+static void *mgl_qo2d_hprep(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo2d_ham *f = (mgl_qo2d_ham *)t->v;
@@ -811,7 +752,7 @@ HMDT MGL_EXPORT mgl_qo2d_func(ddual (*ham)(mreal u, mreal x, mreal y, mreal px,
 	HMDT out = mgl_datac_abs(res);	delete res;	return out;
 }
 //-----------------------------------------------------------------------------
-ddual MGL_NO_EXPORT mgl_ham2d(mreal u, mreal x, mreal y, mreal px, mreal py, void *par)
+ddual static mgl_ham2d(mreal u, mreal x, mreal y, mreal px, mreal py, void *par)
 {
 	mglFormula *h = (mglFormula *)par;
 	mreal var[MGL_VS];	memset(var,0,MGL_VS*sizeof(mreal));
@@ -852,7 +793,7 @@ struct mgl_qo3d_ham
 	void *par;
 };
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_qo3d_hprep(void *par)
+static void *mgl_qo3d_hprep(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
@@ -887,7 +828,7 @@ MGL_NO_EXPORT void *mgl_qo3d_hprep(void *par)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_qo3d_post(void *par)
+static void *mgl_qo3d_post(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
@@ -1034,7 +975,7 @@ HMDT MGL_EXPORT mgl_qo3d_func(ddual (*ham)(mreal u, mreal x, mreal y, mreal z, m
 	HMDT out = mgl_datac_abs(res);	delete res;	return out;
 }
 //-----------------------------------------------------------------------------
-ddual MGL_NO_EXPORT mgl_ham3d(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par)
+ddual static mgl_ham3d(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par)
 {
 	mglFormula *h = (mglFormula *)par;
 	mreal var[MGL_VS];	memset(var,0,MGL_VS*sizeof(mreal));
@@ -1064,7 +1005,7 @@ uintptr_t MGL_EXPORT mgl_qo3d_solve_(const char *ham, uintptr_t* ini_re, uintptr
 //		mglJacobian series
 //
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_jacob2(void *par)
+static void *mgl_jacob2(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	const long nx=t->p[0], ny=t->p[1];
@@ -1111,7 +1052,7 @@ HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 	return r;
 }
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_jacob3(void *par)
+static void *mgl_jacob3(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	const long nx=t->p[0], ny=t->p[1], nz=t->p[2];
@@ -1180,7 +1121,7 @@ uintptr_t MGL_EXPORT mgl_jacobian_3d_(uintptr_t* x, uintptr_t* y, uintptr_t* z)
 //	Progonka
 //
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
+void static mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *uu=dat+2*n;
 	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
@@ -1197,7 +1138,7 @@ void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 	uu[n-1] = bb[n-1];
 	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
 }
-void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
+void static mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
 	mreal a0=A->vthr(i0), b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
@@ -1222,7 +1163,7 @@ void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 	mreal u0 = bb[0]/(1.-aa[0]);
 	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
 }
-void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, bool difr)
+void static mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *uu=dat+n*n;
 	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
@@ -1340,7 +1281,7 @@ uintptr_t MGL_EXPORT mgl_data_tridmat_(uintptr_t *A, uintptr_t *B, uintptr_t *C,
 	delete []s;	return r;
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
+void static mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+2*n;
 	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
@@ -1357,7 +1298,7 @@ void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 	uu[n-1] = bb[n-1];
 	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
 }
-void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
+void static mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
 	dual a0=A->vcthr(i0), b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
@@ -1382,7 +1323,7 @@ void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 	dual u0 = bb[0]/(mreal(1)-aa[0]);
 	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
 }
-void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, bool difr)
+void static mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+n*n;
 	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
