@@ -32,10 +32,10 @@ void mglCanvas::pxl_combine(long id, long n, const void *)
 #endif
 	for(long i=id;i<n;i+=mglNumThr)
 	{
-		unsigned char *cc = C+12*i, c[4];
-		memcpy(c,GB+4*i,4);	// memcpy(c,BDef,4);
-		combine(c,cc+8);	combine(c,cc+4);
-		combine(c,cc);		memcpy(G4+4*i,c,4);
+		unsigned char *cc = C+12*i, c[4], *b=GB+4*i, *g=G4+4*i;
+		c[0]=b[0];	c[1]=b[1];	c[2]=b[2];	c[3]=b[3];
+		combine(c,cc+8);	combine(c,cc+4);	combine(c,cc);
+		g[0]=c[0];	g[1]=c[1];	g[2]=c[2];	g[3]=c[3];
 	}
 }
 //-----------------------------------------------------------------------------
@@ -44,7 +44,9 @@ void mglCanvas::pxl_memcpy(long id, long n, const void *)
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
-	for(long i=id;i<n;i+=mglNumThr)	memcpy(G4+4*i,C+12*i,4);
+	for(long i=id;i<n;i+=mglNumThr)
+	{	unsigned char *g=G4+4*i, *c=C+12*i;
+		g[0]=c[0];	g[1]=c[1];	g[2]=c[2];	g[3]=c[3];	}
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::pxl_backgr(long id, long n, const void *)
@@ -53,13 +55,14 @@ void mglCanvas::pxl_backgr(long id, long n, const void *)
 #pragma omp parallel for
 #endif
 	for(long i=id;i<n;i+=mglNumThr)
-	{	unsigned char c[4];	memcpy(c,GB+4*i,4);	combine(c,G4+4*i);	memcpy(G+3*i,c,3);	}
+	{	unsigned char *b=GB+4*i, c[4]={b[0],b[1],b[2],b[3]}, *g=G+3*i;
+		combine(c,G4+4*i);	g[0]=c[0];	g[1]=c[1];	g[2]=c[2];	}
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::pxl_transform(long id, long n, const void *)
 {
-	const mreal *b = Bp.b;
-	const mreal dx = -Bp.x*Width/2, dy = -Bp.y*Height/2, dz = Depth/2.;
+	const float *b = Bp.b;
+	const float dx = -Bp.x*Width/2, dy = -Bp.y*Height/2, dz = Depth/2.;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
@@ -169,8 +172,8 @@ void mglCanvas::pxl_primdr(long id, long , const void *)
 //-----------------------------------------------------------------------------
 void mglCanvas::pxl_dotsdr(long id, long n, const void *)
 {
-	const mreal *b = Bp.b;
-	const mreal dx = -Bp.x*Width/2, dy = -Bp.y*Height/2, dz = Depth/2.;
+	const float *b = Bp.b;
+	const float dx = -Bp.x*Width/2, dy = -Bp.y*Height/2, dz = Depth/2.;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
@@ -190,8 +193,9 @@ void mglCanvas::pxl_dotsdr(long id, long n, const void *)
 		r[1] = (unsigned char)(255*p.g);
 		r[2] = (unsigned char)(255*p.b);
 		long i0=long(xx)+Width*(Height-1-long(yy));
+		unsigned char *c = C+12*i0;
 		if(i0>=0 && i0<Width*Height && zz>Z[3*i0])
-		{	Z[3*i0]=z;	memcpy(C+12*i0,r,4);	OI[i0]=-1;	}
+		{	Z[3*i0]=z;	c[0]=r[0];	c[1]=r[1];	c[2]=r[2];	c[3]=r[3];	OI[i0]=-1;	}
 	}
 }
 //-----------------------------------------------------------------------------
@@ -226,11 +230,15 @@ void mglCanvas::pnt_plot(long x,long y,mreal z,const unsigned char ci[4], int ob
 	{
 		long i0=x+Width*(Height-1-y);
 		unsigned char *cc = C+12*i0, c[4];
-		memcpy(c,ci,4);
+		c[0]=ci[0];	c[1]=ci[1];	c[2]=ci[2];	c[3]=ci[3];
 		float *zz = Z+3*i0, zf = FogDist*(z/Depth-0.5-FogDz);
 		// try to remove double transparency near vertexes
 		if(fabs(z-zz[0])<1 && OI[i0]==obj_id && abs(cc[0]-ci[0])+abs(cc[1]-ci[1])+abs(cc[2]-ci[2])<5)
-		{	if(cc[3]<ci[3])	memcpy(cc,c,4);	return;	}
+		{
+			if(cc[3]<ci[3])
+			{	cc[0]=c[0];	cc[1]=c[1];	cc[2]=c[2];	cc[3]=c[3];	}
+			return;
+		}
 		if(zf<0)	// add fog
 		{
 			int d = int(255.f-255.f*exp(5.f*zf));
@@ -245,79 +253,94 @@ void mglCanvas::pnt_plot(long x,long y,mreal z,const unsigned char ci[4], int ob
 				zz[2] = zz[1];	combine(cc+8,cc+4);
 				if(z>=zz[0])
 				{	zz[1] = zz[0];	zz[0] = z;	OI[i0]=obj_id;
-					memcpy(cc+4,cc,4);	memcpy(cc,c,4);		}
-				else	{	zz[1] = z;	memcpy(cc+4,c,4);	}
+					cc[4]=cc[0];	cc[0]=c[0];	cc[5]=cc[1];	cc[1]=c[1];
+					cc[6]=cc[2];	cc[2]=c[2];	cc[7]=cc[3];	cc[3]=c[3];	}
+				else
+				{	zz[1] = z;	cc[4]=c[0];	cc[5]=c[1];	cc[6]=c[2];	cc[7]=c[3];	}
 			}
 			else
 			{
 				if(z>=zz[2])	// shift point on slice down and paste new point
 				{	zz[2] = z;	combine(cc+8,c);	}
 				else		// point below the background
-				{	combine(c,cc+8);	memcpy(cc+8,c,4);	}
+				{	combine(c,cc+8);	cc[8]=c[0];	cc[9]=c[1];	cc[10]=c[2];cc[11]=c[3];	}
 			}
 		}
 		if(Quality&MGL_DRAW_FAST)
 		{
 			if(z>=zz[0])	// point upper the background
 			{	zz[0]=z;	combine(cc,c);	OI[i0]=obj_id;	}
+			else
+			{	combine(c,cc);	cc[6]=cc[2];	cc[2]=c[2];	cc[7]=cc[3];	cc[3]=c[3];	}
 		}
 		else
 		{
 			if(z>=zz[0])	// point upper the background
-			{	zz[0]=z;	memcpy(cc,c,4);		OI[i0]=obj_id;	}
+			{	zz[0]=z;	cc[0]=c[0];	cc[1]=c[1];	cc[2]=c[2];	cc[3]=c[3];	OI[i0]=obj_id;	}
 		}
 	}
 }
 //-----------------------------------------------------------------------------
-unsigned char* mglCanvas::col2int(const mglPnt &p,unsigned char *r, int obj_id) const
+inline float mexp(float x)	//	exp(-x) ~ 1/(1+x+x^2/2+x^3/4+x^5/40)
+{	return 1.f/(1.f+x*(1.f+x*x/2.f));	}
+//{	return 1/(1+x*(1+x/2*(1+x/2*(1+x*x/10))));	}
+//{	return exp(-x);	}
+void mglCanvas::col2int(const mglPnt &p,unsigned char *r, int obj_id) const
 {
 //	if(!r)	return r;	// NOTE r must be provided!
-	if(p.a<=0)	{	memset(r,0,4);	return r;	}
+	if(p.a<=0)	{	r[0]=r[1]=r[2]=r[3]=0;	return;	}
 	float b0=0,b1=0,b2=0, ar,ag,ab,dif;
-	const size_t nl = p.sub>=0?p.sub:1-p.sub;
+	const size_t nl = p.sub>=0?p.sub:-1-p.sub;
 	const bool glob = !get(MGL_LOCAL_LIGHT);
 	ar = ag = ab = glob?AmbBr:Sub[nl].AmbBr;
 	dif = glob?DifBr:Sub[nl].DifBr;
+	const mglLight *gll = glob?light:Sub[nl].light;
 
 	if(mgl_isnum(p.u+p.v+p.w))
 	{
-		float d0,d1,d2,nn;
 		for(long i=0;i<10;i++)
 		{
-			const mglLight &ll=glob?light[i]:Sub[nl].light[i];
+			const mglLight &ll=gll[i];
 			if(!ll.n)	continue;
 			if(mgl_isnan(ll.q.x))		// source at infinity
 			{
-				nn = 2*(p.u*ll.p.x+p.v*ll.p.y+p.w*ll.p.z) / (p.u*p.u+p.v*p.v+p.w*p.w+1e-6);
-				d0 = ll.p.x - p.u*nn;
-				d1 = ll.p.y - p.v*nn;
-				d2 = ll.p.z - p.w*nn;
-				nn = 1 + d2/sqrt(d0*d0+d1*d1+d2*d2+1e-6);
+				const mglPoint &lp = ll.p;
+				float nn = 2*(p.u*lp.x+p.v*lp.y+p.w*lp.z) / (p.u*p.u+p.v*p.v+p.w*p.w+1e-6f);
+				float d0 = lp.x - p.u*nn;
+				float d1 = lp.y - p.v*nn;
+				float d2 = lp.z - p.w*nn;
+				nn = 1 + d2/sqrt(d0*d0+d1*d1+d2*d2+1e-6f);
 
-				nn = exp(-ll.a*nn)*ll.b*2;
-				b0 += nn*ll.c.r;
-				b1 += nn*ll.c.g;
-				b2 += nn*ll.c.b;
+//				nn = exp(-ll.a*nn)*ll.b*2;
+				nn = mexp(ll.a*nn)*ll.b*2.f;
+				const mglColor &lc = ll.c;
+				b0 += nn*lc.r;
+				b1 += nn*lc.g;
+				b2 += nn*lc.b;
 			}
 			else		// diffuse and specular light
 			{
-				d0 = ll.q.x-p.x;	// direction to light source
-				d1 = ll.q.y-p.y;
-				d2 = ll.q.z-p.z;
-				nn = 1+(d0*ll.p.x+d1*ll.p.y+d2*ll.p.z)/sqrt(d0*d0+d1*d1+d2*d2+1e-6);
-				float bb = exp(-3*ll.a*nn);	nn = bb*dif*2;
-				ar += nn*ll.c.r;
-				ag += nn*ll.c.g;
-				ab += nn*ll.c.b;
+				const mglPoint &lp=ll.p, &lq=ll.q;
+				float d0 = lq.x-p.x;	// direction to light source
+				float d1 = lq.y-p.y;
+				float d2 = lq.z-p.z;
+				float nn = 1+(d0*lp.x+d1*lp.y+d2*lp.z)/sqrt(d0*d0+d1*d1+d2*d2+1e-6f);
+//				float bb = exp(-3*ll.a*nn);	nn = bb*dif*2;
+				float bb = mexp(3*ll.a*nn);	nn = bb*dif*2;
+				const mglColor &lc = ll.c;
+				ar += nn*lc.r;
+				ag += nn*lc.g;
+				ab += nn*lc.b;
 
-				nn = 2*(p.u*d0+p.v*d1+p.w*d2) / (p.u*p.u+p.v*p.v+p.w*p.w+1e-6);
+				nn = 2*(p.u*d0+p.v*d1+p.w*d2) / (p.u*p.u+p.v*p.v+p.w*p.w+1e-6f);
 				d0 -= p.u*nn;	d1 -= p.v*nn;	d2 -= p.w*nn;
-				nn = 1 + d2/sqrt(d0*d0+d1*d1+d2*d2+1e-6);
+				nn = 1 + d2/sqrt(d0*d0+d1*d1+d2*d2+1e-6f);
 
-				nn = exp(-ll.a*nn)*bb*ll.b*2;
-				b0 += nn*ll.c.r;
-				b1 += nn*ll.c.g;
-				b2 += nn*ll.c.b;
+//				nn = exp(-ll.a*nn)*bb*ll.b*2;
+				nn = mexp(ll.a*nn)*bb*ll.b*2.f;
+				b0 += nn*lc.r;
+				b1 += nn*lc.g;
+				b2 += nn*lc.b;
 			}
 		}
 		b0 += (ar>1 ? 1:ar)*p.r;	// diffuse light
@@ -336,7 +359,7 @@ unsigned char* mglCanvas::col2int(const mglPnt &p,unsigned char *r, int obj_id)
 	r[2] = (unsigned char)(255*b2);
 //	r[3] = get(MGL_ENABLE_ALPHA) ? (unsigned char)(255*p.a) : 255;
 	r[3] = (unsigned char)((Quality&MGL_DRAW_NORM)?255*p.a:255);
-	return r;
+//	return r;
 }
 //-----------------------------------------------------------------------------
 /// color mixing: color c1 is under color c2 !!!
@@ -403,9 +426,9 @@ void mglCanvas::quad_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 	y1 = long(mgl_min(mgl_min(p1.y,p2.y), mgl_min(p3.y,p4.y)));
 	x2 = long(mgl_max(mgl_max(p1.x,p2.x), mgl_max(p3.x,p4.x)));
 	y2 = long(mgl_max(mgl_max(p1.y,p2.y), mgl_max(p3.y,p4.y)));
-	x1=mgl_max(x1,d->x1);	x2=mgl_min(x2,d->x2);
-	y1=mgl_max(y1,d->y1);	y2=mgl_min(y2,d->y2);
-//	if(x1>x2 || y1>y2)	return;
+	x1=mgl_imax(x1,d->x1);	x2=mgl_imin(x2,d->x2);
+	y1=mgl_imax(y1,d->y1);	y2=mgl_imin(y2,d->y2);
+	if(x1>x2 || y1>y2)	return;
 
 	const float dd = d1.x*d2.y-d1.y*d2.x;
 	const float dsx =-4*(d2.y*d3.x - d2.x*d3.y)*d1.y;
@@ -419,6 +442,12 @@ void mglCanvas::quad_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 	const int oi = d->ObjId, ang=d->angle;
 	const mreal pw = d->PenWidth;
 	const uint64_t pd = d->PDef;
+	
+	mglPnt tmp(p1+d1+d2+d3), pp(p1);
+	if(mgl_isnan(tmp.u) && mgl_isnum(tmp.v))
+	{	pp.u = nr.x;	pp.v = nr.y;	pp.w = nr.z;
+		d1.u=d1.v=d1.w=d2.u=d2.v=d2.w=d3.u=d3.v=d3.w=0;	}
+	
 	for(long j=y1;j<=y2;j++)	for(long i=x1;i<=x2;i++)
 	{
 		if(pd==MGL_SOLID_MASK || visible(i,j,d->m, pw,ang))
@@ -440,10 +469,8 @@ void mglCanvas::quad_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 					u = 2.f*(d2.y*xx - d2.x*yy)/qu;	v = 2.f*(d1.x*yy - d1.y*xx)/qv;
 					if(u*(1.f-u)<0.f || v*(1.f-v)<0.f)	continue;	// second root bad
 				}
-				mglPnt p(p1+d1*u+d2*v+d3*(u*v));
-				if(mgl_isnan(p.u) && mgl_isnum(p.v))
-				{	p.u = nr.x;	p.v = nr.y;	p.w = nr.z;	}
-				pnt_plot(i,j,p.z,col2int(p,r,oi),oi);
+				mglPnt p(pp+d1*u+d2*v+d3*(u*v));	col2int(p,r,oi);
+				if(r[3])	pnt_plot(i,j,p.z,r,oi);
 			}
 		}
 	}
@@ -458,7 +485,7 @@ void mglCanvas::trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 	{	fast_draw(p1,p2,d);	fast_draw(p1,p3,d);	fast_draw(p2,p3,d);	return;	}
 	unsigned char r[4];
 	long y1,x1,y2,x2;
-	const mglPnt d1(p2-p1), d2(p3-p1);
+	mglPnt d1(p2-p1), d2(p3-p1);
 
 	const float tmp = d2.x*d1.y - d1.x*d2.y;
 	if(fabs(tmp)<1e-5)	return;		// points lies on the same line
@@ -471,7 +498,7 @@ void mglCanvas::trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 	y2 = long(mgl_max(p1.y>p2.y?p1.y:p2.y, p3.y));
 	x1=x1>d->x1?x1:d->x1;	x2=x2<d->x2?x2:d->x2;
 	y1=y1>d->y1?y1:d->y1;	y2=y2<d->y2?y2:d->y2;
-//	if(x1>x2 || y1>y2)	return;
+	if(x1>x2 || y1>y2)	return;
 	// default normale
 	const mglPoint nr(mglPoint(p2.x-p1.x,p2.y-p1.y,p2.z-p1.z)^mglPoint(p3.x-p1.x,p3.y-p1.y,p3.z-p1.z));
 	const float x0 = p1.x, y0 = p1.y;
@@ -480,6 +507,12 @@ void mglCanvas::trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 	const int oi = d->ObjId, ang=d->angle;
 	const mreal pw = d->PenWidth;
 	const uint64_t pd = d->PDef;
+
+	mglPnt tp(p1+d1+d2), pp(p1);
+	if(mgl_isnan(tp.u) && mgl_isnum(tp.v))
+	{	pp.u = nr.x;	pp.v = nr.y;	pp.w = nr.z;
+		d1.u=d1.v=d1.w=d2.u=d2.v=d2.w=0;	}
+
 	if(Quality&MGL_DRAW_NORM)	for(long j=y1;j<=y2;j++)	for(long i=x1;i<=x2;i++)
 	{
 		if(pd==MGL_SOLID_MASK || visible(i,j,d->m, pw,ang))
@@ -487,24 +520,22 @@ void mglCanvas::trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3,
 			float xx = (i-x0), yy = (j-y0);
 			float u = dxu*xx+dyu*yy, v = dxv*xx+dyv*yy;
 			if(u<0 || v<0 || u+v>1)	continue;
-			mglPnt p(p1+d1*u+d2*v);
-			if(mgl_isnan(p.u) && mgl_isnum(p.v) && anorm)
-			{	p.u = nr.x;	p.v = nr.y;	p.w = nr.z;	}
-			pnt_plot(i,j,p.z+dz,col2int(p,r,oi),oi);
+			mglPnt p(pp+d1*u+d2*v);	col2int(p,r,oi);
+			if(r[3])	pnt_plot(i,j,p.z+dz,r,oi);
 		}
 	}
 	else
 	{
 		col2int(p1,r,oi);
-		float zz = p1.z+dz;
-		for(long j=y1;j<=y2;j++)	for(long i=x1;i<=x2;i++)
+		float zz = p1.z+dz, dz1=d1.z, dz2=d2.z;
+		if(r[3])	for(long j=y1;j<=y2;j++)	for(long i=x1;i<=x2;i++)
 		{
 			if(pd==MGL_SOLID_MASK || visible(i,j,d->m, pw,ang))
 			{
 				float xx = (i-x0), yy = (j-y0);
 				float u = dxu*xx+dyu*yy, v = dxv*xx+dyv*yy;
 				if(u<0 || v<0 || u+v>1)	continue;
-				pnt_plot(i,j,zz,r,oi);
+				pnt_plot(i,j,zz+dz1*u+dz2*v,r,oi);
 			}
 		}
 	}
@@ -531,31 +562,30 @@ void mglCanvas::line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *
 	x1=x1>dr->x1?x1:dr->x1;	x2=x2<dr->x2?x2:dr->x2;
 	y1=y1>dr->y1?y1:dr->y1;	y2=y2<dr->y2?y2:dr->y2;
 	const float dd = hypot(d.x, d.y);
-//	if(x1>x2 || y1>y2 || dd<1e-5)	return;
-	if(dd<1e-5)	return;
+	if(x1>x2 || y1>y2 || dd<1e-5)	return;
 
 	const float dxv = d.y/dd, dyv =-d.x/dd;
 	const float dxu = d.x/dd, dyu = d.y/dd;
 
 	const uint64_t pd = dr->PDef;
-	const mreal pp = dr->pPos;
+	const mreal pp = dr->pPos, V = (pw-1)*(pw-1)/4, S = (1-pw)/2;
 	if(hor)	for(long i=x1;i<=x2;i++)
 	{
 		y1 = int(p1.y+d.y*(i-p1.x)/d.x - pw - 10/dpw);
 		y2 = int(p1.y+d.y*(i-p1.x)/d.x + pw + 10/dpw);
 		y1=y1>dr->y1?y1:dr->y1;	y2=y2<dr->y2?y2:dr->y2;
-		if(y1>y2)	continue;
 		for(long j=y1;j<=y2;j++)
 		{
 			float xx = (i-p1.x), yy = (j-p1.y);
 			float u = dxu*xx+dyu*yy, v = dxv*xx+dyv*yy;	v = v*v;
-			if(u<0)			v += u*u;
-			else if(u>dd)	v += (u-dd)*(u-dd);
-//			if(v>pw*pw)		continue;
-			if(!(pd & ((uint64_t)1<<long(fmod(pp+u/pw, 16)) ) ))	continue;
-			mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
-			r[3] = v<(pw-1)*(pw-1)/4 ? 255 : mgl_sline(255,dpw*(sqrt(v)+(1-pw)/2));
-			pnt_plot(i,j,p.z+dz,r,oi);
+			if(u<0)			{	v += 16*u*u;	u=0;	}
+			else if(u>dd)	{	v += 16*(u-dd)*(u-dd);	u=dd;	}
+			if( pd & ((uint64_t)1<<(long(pp+u/pw)%16)) )
+			{
+				mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
+				r[3] = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
+				if(r[3])	pnt_plot(i,j,p.z+dz,r,oi);
+			}
 		}
 	}
 	else	for(long j=y1;j<=y2;j++)
@@ -563,19 +593,18 @@ void mglCanvas::line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *
 		x1 = int(p1.x+d.x*(j-p1.y)/d.y - pw - 10/dpw);
 		x2 = int(p1.x+d.x*(j-p1.y)/d.y + pw + 10/dpw);
 		x1=x1>dr->x1?x1:dr->x1;	x2=x2<dr->x2?x2:dr->x2;
-		if(x1>x2)	continue;
-
 		for(long i=x1;i<=x2;i++)
 		{
 			float xx = (i-p1.x), yy = (j-p1.y);
 			float u = dxu*xx+dyu*yy, v = dxv*xx+dyv*yy;	v = v*v;
-			if(u<0)			v += u*u;
-			else if(u>dd)	v += (u-dd)*(u-dd);
-//			if(v>pw*pw)		continue;
-			if(!(pd & ((uint64_t)1<<long(fmod(pp+u/pw, 16)))))		continue;
-			mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
-			r[3] = v<(pw-1)*(pw-1)/4 ? 255 : mgl_sline(255,dpw*(sqrt(v)+(1-pw)/2));
-			pnt_plot(i,j,p.z+dz,r,oi);
+			if(u<0)			{	v += 16*u*u;	u=0;	}
+			else if(u>dd)	{	v += 16*(u-dd)*(u-dd);	u=dd;	}
+			if( pd & ((uint64_t)1<<(long(pp+u/pw)%16)) )
+			{
+				mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
+				r[3] = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
+				if(r[3])	pnt_plot(i,j,p.z+dz,r,oi);
+			}
 		}
 	}
 }
@@ -583,8 +612,10 @@ void mglCanvas::line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *
 void mglCanvas::pnt_fast(long x,long y,mreal z,const unsigned char ci[4], int obj_id)
 {
 	long i0=x+Width*(Height-1-y);
+	unsigned char *cc = C+12*i0;
 	if(ci[3]!=0 && z>Z[3*i0])	// point upper the background
-	{	Z[3*i0]=z;	memcpy(C+12*i0,ci,4);	OI[i0]=obj_id;	}
+	{	Z[3*i0]=z;		OI[i0]=obj_id;
+		cc[0]=ci[0];	cc[1]=ci[1];	cc[2]=ci[2];	cc[3]=ci[3];	}
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::fast_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *dr)
@@ -629,17 +660,18 @@ void mglCanvas::pnt_draw(const mglPnt &p, const mglDrawReg *dr)
 	const long s = long(pw+10/dpw+fabs(pw));
 	const long i1=mgl_max(-s,dr->x1-p.x),i2=mgl_min(s,dr->x2-p.x);
 	const long j1=mgl_max(-s,dr->y1-p.y),j2=mgl_min(s,dr->y2-p.y);
+	const mreal V = (pw-1)*(pw-1)/4, S = (1-pw)/2;
 	if(!(Quality&3))	for(long j=j1;j<=j2;j++)	for(long i=i1;i<=i2;i++)	// fast draw
 	{
 		float v = i*i+j*j;
-		if(v>1+(pw-1)*(pw-1)/4)	continue;
-		pnt_plot(p.x+i,p.y+j,p.z,cs,oi);
+		if(v>1+V)	continue;
+		if(cs[3])	pnt_plot(p.x+i,p.y+j,p.z,cs,oi);
 	}
 	else	for(long j=j1;j<=j2;j++)	for(long i=i1;i<=i2;i++)
 	{
 		float v = i*i+j*j;
-		cs[3] = v<(pw-1)*(pw-1)/4 ? cc : mgl_sline(cc,dpw*(sqrt(v)+(1-pw)/2));
-		pnt_plot(p.x+i,p.y+j,p.z,cs,oi);
+		cs[3] = v<V ? cc : mgl_sline(cc,dpw*(sqrt(v)+S));
+		if(cs[3])	pnt_plot(p.x+i,p.y+j,p.z,cs,oi);
 	}
 }
 //-----------------------------------------------------------------------------
@@ -669,7 +701,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 			float dx=i-q.x, dy=j-q.y, v=dx*dx+dy*dy;
 			int sum = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 			cs[3] = ca*sum/255;
-			pnt_plot(i,j,q.z+1,cs,oi);
+			if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 		}
 	}
 	else
@@ -708,7 +740,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(dx)-ss;	v = dy*dy+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case '+':
@@ -721,7 +753,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(dx)-ss;	v = dy*dy+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'X':
@@ -744,7 +776,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'x':
@@ -757,7 +789,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(dx-dy)-2*ss;	v = dx+dy;	v = v*v+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'S':
@@ -768,7 +800,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				v = fabs(dx)-ss;	if(v<0)	v=0;	v = u*u+v*v;
 				int sum = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 's':
@@ -785,7 +817,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(dx)-ss;	v = (dy+ss)*(dy+ss)+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'D':
@@ -796,7 +828,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				v = fabs(dx+dy)-ss;	if(v<0)	v=0;	v = u*u+v*v;
 				int sum = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'd':
@@ -813,7 +845,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(dx-dy)-ss;	v = (dx+dy+ss)*(dx+dy+ss)+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'Y':
@@ -828,7 +860,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(-0.87*dx+0.5*dy-ss/2)-ss/2;	v = (0.5*dx+0.87*dy)*(0.5*dx+0.87*dy)+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case '*':
@@ -843,7 +875,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(-0.87*dx+0.5*dy)-ss;	v = (0.5*dx+0.87*dy)*(0.5*dx+0.87*dy)+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'T':
@@ -863,7 +895,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 					sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 					sum = sum>255?255:sum;	cs[3] = ca*sum/255;
 				}
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case '^':
@@ -878,7 +910,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(0.55*dx-0.83*dy)-0.9*ss;	v = 0.83*dx+0.55*dy-0.55*ss;	v = v*v+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'V':
@@ -898,7 +930,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 					sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 					sum = sum>255?255:sum;	cs[3] = ca*sum/255;
 				}
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'v':
@@ -913,7 +945,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(0.55*dx-0.83*dy)-0.9*ss;	v = 0.83*dx+0.55*dy+0.55*ss;	v = v*v+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'L':
@@ -933,7 +965,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 					sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 					sum = sum>255?255:sum;	cs[3] = ca*sum/255;
 				}
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case '<':
@@ -948,7 +980,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(0.55*dy-0.83*dx)-0.9*ss;	v = 0.83*dy+0.55*dx+0.55*ss;	v = v*v+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'R':
@@ -968,7 +1000,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 					sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 					sum = sum>255?255:sum;	cs[3] = ca*sum/255;
 				}
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case '>':
@@ -983,7 +1015,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				u = fabs(0.55*dy-0.83*dx)-0.9*ss;	v = 0.83*dy+0.55*dx-0.55*ss;	v = v*v+(u<0?0:u*u);
 				sum += v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'O':
@@ -993,7 +1025,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				v = hypot(dx,dy)-ss;	v=v<0?0:v*v;
 				int sum = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'o':
@@ -1003,7 +1035,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				v = hypot(dx,dy)-ss;	v=v*v;
 				int sum = v<V ? 255 : mgl_sline(255,dpw*(sqrt(v)+S));
 				cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		case 'C':
@@ -1015,7 +1047,7 @@ void mglCanvas::mark_draw(const mglPnt &q, char type, mreal size, mglDrawReg *dr
 				v = dx*dx+dy*dy;
 				sum += v<(2*pw-1)*(2*pw-1)/4 ? 255 : mgl_sline(255,dpw*(sqrt(v)+(1-2*pw)/2));
 				sum = sum>255?255:sum;	cs[3] = ca*sum/255;
-				pnt_plot(i,j,q.z+1,cs,oi);
+				if(cs[3])	pnt_plot(i,j,q.z+1,cs,oi);
 			}
 			break;
 		}
@@ -1075,7 +1107,7 @@ void mglCanvas::glyph_draw(const mglPrim &P, mglDrawReg *d)
 	}
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_addpnts(mreal x1,mreal y1,mreal x2,mreal y2, std::vector<mreal> *b)
+void static mgl_addpnts(mreal x1,mreal y1,mreal x2,mreal y2, std::vector<mreal> *b)
 {
 // 	if(x1>x2)	{	mreal t=x1;	x1=x2;	x2=t;	t=y1;	y1=y2;	y2=t;	}
 	if(y1<y2)	for(int i=long(y1);i<=long(y2)+1;i++)
@@ -1157,7 +1189,7 @@ void mglCanvas::glyph_fill(mreal phi, const mglPnt &pp, mreal f, const mglGlyph
 	const float dz = Width>2 ? 1 : 1e-5*Width;		// provide additional height to be well visible on the surfaces
 	const int oi = d?d->ObjId:-1;
 	unsigned char r[4];	col2int(pp,r,oi);
-	for(long i=0;i<g.nl;i++)	// add bounding points
+	if(r[3])	for(long i=0;i<g.nl;i++)	// add bounding points
 	{
 		long ii=2*i;
 		mreal x = pp.u + g.line[ii]*f, y = pp.v + g.line[ii+1]*f;
@@ -1405,7 +1437,6 @@ void mglCanvas::arrow_plot_3d(long n1, long n2, char st, float ll)
 //-----------------------------------------------------------------------------
 bool mglCanvas::quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4) const
 {
-	unsigned char r[4];
 	long y1,x1,y2,x2;
 	mglPnt d1(p2-p1), d2(p3-p1), d3(p4+p1-p2-p3);
 
@@ -1416,8 +1447,8 @@ bool mglCanvas::quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, c
 	y1 = long(mgl_min(mgl_min(p1.y,p2.y), mgl_min(p3.y,p4.y)));
 	x2 = long(mgl_max(mgl_max(p1.x,p2.x), mgl_max(p3.x,p4.x)));
 	y2 = long(mgl_max(mgl_max(p1.y,p2.y), mgl_max(p3.y,p4.y)));
-	x1=mgl_max(x1,0);	x2=mgl_min(x2,Width);
-	y1=mgl_max(y1,0);	y2=mgl_min(y2,Height);
+	x1=mgl_imax(x1,0);	x2=mgl_imin(x2,Width);
+	y1=mgl_imax(y1,0);	y2=mgl_imin(y2,Height);
 //	if(x1>x2 || y1>y2)	return;
 
 	const float dd = d1.x*d2.y-d1.y*d2.x;
@@ -1454,7 +1485,6 @@ bool mglCanvas::quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, c
 //-----------------------------------------------------------------------------
 bool mglCanvas::trig_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3) const
 {
-	unsigned char r[4];
 	long y1,x1,y2,x2;
 	const mglPnt d1(p2-p1), d2(p3-p1);
 
