@@ -21,6 +21,7 @@
 #define PLOT_PNL_H
 //-----------------------------------------------------------------------------
 #include <QWidget>
+#include <qabstractitemmodel.h>
 //-----------------------------------------------------------------------------
 class QMenu;
 class QTimer;
@@ -34,9 +35,10 @@ class mglDrawScript;
 class InfoDialog;
 class AnimParam;
 class DatPanel;
-class mglVar;
 class StyleDialog;
 class QPrinter;
+class NewCmdDialog;
+class SubplotDialog;
 //-----------------------------------------------------------------------------
 class PlotPanel : public QWidget
 {
@@ -44,8 +46,10 @@ Q_OBJECT
 public:
 	QMenu *menu;
 	QMathGL *mgl;
-	mglDrawScript *draw;		///< Class for drawing MGL script
-	const QTextEdit *textMGL;	///< Editor with MGL script body
+	mglDrawScript *draw;	///< Class for drawing MGL script
+	QTextEdit *textMGL;		///< Editor with MGL script body
+	NewCmdDialog *newCmdDlg;
+	SubplotDialog *subplotDlg;
 	PlotPanel(QWidget *wp=0);
 	~PlotPanel();
 	void setMGLFont(const QString &path);
@@ -75,6 +79,13 @@ private slots:
 	void pressF9();
 	void stop();
 	void setStyle(int id);
+	void setSubId(int id);
+	void deleteSelected();
+	void hideSelected();
+	void putCmd(const QString &cmd);
+	void insCmd(const QString &cmd);
+	void movePlotUp();
+	void movePlotDown();
 
 private:
 	bool gifOn, jpgOn;
@@ -89,6 +100,8 @@ private:
 	QMenu *popup;
 	QPrinter *printer;
 	StyleDialog *stlDialog;
+	int objId;
+	int subId;
 	
 	void toolTop(QBoxLayout *l);
 	void toolLeft(QBoxLayout *l);
