@@ -30,10 +30,24 @@
 #endif
 //-----------------------------------------------------------------------------
 std::wstring str, opt;
+std::vector<std::string> anim;
 mglParse p(true);
+void prop_func(char id, const char *val, void *)
+{	p.AddParam(id<='9' ? id-'0' : id-'a'+10, val);	}
 //-----------------------------------------------------------------------------
 int show(mglGraph *gr)
 {
+	if(anim.size()>0)
+	{
+		for(size_t i=0;i<anim.size();i++)
+		{
+			gr->NewFrame();
+			p.AddParam(0,anim[i].c_str());
+			p.Execute(gr,str.c_str());
+			gr->EndFrame();
+		}
+		return gr->GetNumFrame();
+	}
 	p.Execute(gr,str.c_str());
 	printf("%s\n",gr->Message());
 	return 0;
@@ -83,6 +97,8 @@ int main(int argc, char **argv)
 		{	mgl_strncpy(iname, argv[optind][0]=='-'?"":argv[optind],256);	break;	}
 	}
 
+	std::string ids;
+	std::vector<std::string> par;
 	bool mgld=(*iname && iname[strlen(iname)-1]=='d');
 	if(!mgld)
 	{
@@ -91,23 +107,28 @@ int main(int argc, char **argv)
 		if(fp)
 		{
 			wchar_t ch;
-			while(!feof(fp) && size_t(ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
+			std::string text;
+			while(!feof(fp) && size_t(ch=fgetwc(fp))!=WEOF)
+			{	str.push_back(ch);	text.push_back(ch);	}
 			fclose(fp);
+
+			double a1, a2, da;
+			mgl_parse_comments(text.c_str(), a1, a2, da, anim, ids, par);
+//			if(!ids.empty())	dr->gr->dialog(ids,par);
 		}
 		else	{	printf("No file for MGL script\n");	return 0;	}
 	}
 
 #if USE_FLTK
 	mgl_ask_func = mgl_ask_fltk;
-	Fl_Preferences pref(Fl_Preferences::USER,"abalakin","mgllab");
-	static const char *sch[4]={"base","gtk+","plastic","gleam"};
-	int scheme;	pref.get("scheme",scheme,2);
-	Fl::scheme(sch[scheme]);
+	mgl_progress_func = mgl_progress_fltk;
 	mglFLTK gr(mgld?NULL:show, *iname?iname:"mglview");
 #else
 	mgl_ask_func = mgl_ask_qt;
 	mglQT gr(mgld?NULL:show, *iname?iname:"mglview");
 #endif
+	gr.SetPropFunc(prop_func,NULL);
+	gr.MakeDialog(ids, par);
 	if(gray)	gr.Gray(gray);
 
 	if(mgld)
