@@ -72,6 +72,8 @@ using mglCanvasWnd::Window;
 	void Adjust();		///< Adjust size of bitmap to window size
 	void GotoFrame(int d);	///< Show arbitrary frame (use relative step)
 	void Animation();		///< Run slideshow (animation) of frames
+	void MakeDialog(const char *ids, char const * const *args, const char *title=""){}	// TODO
+
 
 protected:
 	QScrollArea *scroll;	///< Scrolling area
@@ -100,6 +102,7 @@ QMathGL::QMathGL(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 	autoResize = false;	draw_par = 0;	draw_func = 0;
 	dotsRefr = true;
 	gr = new mglCanvas;	appName = "MathGL";
+	setMinimumSize(gr->GetWidth(),gr->GetHeight());
 	popup = 0;	grBuf = 0;	draw = 0;	prevQuality=MGL_DRAW_NORM;
 	phi = tet = per = 0;	x0=y0=xe=ye=0;
 	x1 = y1 = ax1 = ay1 = 0;	x2 = y2 = ax2 = ay2 = 1;
@@ -155,6 +158,7 @@ void QMathGL::setGraph(HMGL GR)	///< Set grapher object
 	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	gr=gg;	mgl_use_graph(gg,1);
 	gr->SetEventFunc(mgl_qt_event_func, NULL);
+	setMinimumSize(gr->GetWidth(),gr->GetHeight());
 }
 //-----------------------------------------------------------------------------
 void QMathGL::paintEvent(QPaintEvent *)
@@ -332,6 +336,7 @@ void QMathGL::update()
 		if(mgl_is_frames(gr))	mgl_end_frame(gr);
 		setlocale(LC_NUMERIC, loc.c_str());
 		gr->AskStop(false);
+		setMinimumSize(gr->GetWidth(),gr->GetHeight());
 	}
 	else if(mgl_get_num_frame(gr)>0)
 	{
@@ -612,6 +617,13 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 			refresh();	x0 = xe;	y0 = ye;
 		}
 	}
+/*	else if(mgl_get_flag(gr,MGL_SHOW_POS) && !(ev->button()&Qt::LeftButton))
+	{
+		mglPoint p = gr->CalcXYZ(ev->x(), ev->y(), true);
+		if(mgl_isnan(p.x))	mousePos = "";
+		else	mousePos.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
+		emit posChanged(mousePos);
+	}*/
 	ev->accept();
 }
 //-----------------------------------------------------------------------------
@@ -851,7 +863,8 @@ void QMathGL::setSize(int w, int h)
 {
 	resize(w, h);
 	if(w!=pic.width() || h!=pic.height())	// update only when image size is changed
-	{	mgl_set_size(gr,w,h);	update();	}
+	{	mgl_set_size(gr,w,h);	update();
+		setMinimumSize(gr->GetWidth(),gr->GetHeight());	}
 }
 //-----------------------------------------------------------------------------
 void QMathGL::about()
@@ -1232,7 +1245,7 @@ MGL_EXPORT QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, Q
 		bb->addAction(a);		oo->addAction(a);
 		a = new QAction(QPixmap(down_1_xpm), _("Move down"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(shiftDown()));
-		a->setToolTip(_("Move graphics up down 1/3 of its height."));
+		a->setToolTip(_("Move graphics down 1/3 of its height."));
 		bb->addAction(a);		oo->addAction(a);
 		a = new QAction(QPixmap(right_1_xpm), _("Move right"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(shiftRight()));
